import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ClientPopupComponent} from "../../pages/bon-de-livraison/client-popup/client-popup.component";
import {MatDialog} from "@angular/material/dialog";
import {FormBuilder, FormGroup} from "@angular/forms";
import {ContactService} from "../../_services/contact.service";
import {CustomHttpResponse, StoreList} from "../../_models";

@Component({
  selector: 'app-bon-client',
  templateUrl: './bon-client.component.html',
  styleUrls: ['./bon-client.component.scss']
})
export class BonClientComponent implements OnInit {

  @Input() flowType: "bon-reception" | "bon-livraison" | "achat-commande" | undefined;

  private _readOnly: boolean = false;
  get readOnly(): any { return this._readOnly }
  @Input() set readOnly(readOnly: any) {
    if(readOnly) {
      this.searchClientForm.get('SEARCH_KEYWORD')?.disable({ onlySelf: true });
    }

    this._readOnly = readOnly;
  }

  type: any;
  displayType: string | undefined;

  private _chosenCustomer: any = undefined;
  get chosenCustomer(): any { return this._chosenCustomer }
  @Input() set chosenCustomer(chosenCustomer: any) {
    if(chosenCustomer) {
      console.log('CHOSEN CUSTOMER', chosenCustomer);
    }
    this.chosenCustomerChange.emit(this._chosenCustomer = chosenCustomer);
  }
  @Output() private chosenCustomerChange: EventEmitter<any> = new EventEmitter<any>();

  searchClientForm = this.formBuilder.group({
    SEARCH_KEYWORD: ['']
  });

  constructor(private dialog: MatDialog,
              private formBuilder: FormBuilder,
              private contactService: ContactService) { }

  ngOnInit(): void {
  }

  searchClientFormSubmitter() {
    if(this.readOnly) {
      return;
    }

    const formData = this.searchClientForm.getRawValue();

    if (this.flowType == 'bon-reception') {
      // we're setting Tous les Brand by default.
      this.type = '';
    }

    if (this.flowType == 'bon-livraison') {
      // It can be either Client or Supplier. we're setting Supplier by default.
      this.type = '';
    }

    if (this.flowType == 'achat-commande') {
      // It can be only Supplier
      this.type = 'supplier';
    }

    const payload = {
      filter_type: 'store',
      keyword: formData.SEARCH_KEYWORD,
      store_type: this.type,
      store_brand: '',
      store_category: ''
    };

    this._searchCustomers(payload);
  }

  _searchCustomers(payload: any): Promise<any> {
    return new Promise((resolve, reject) => {
      this.contactService.getContactList(payload, 1, 10).subscribe(
        (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            if(res.data.length === 0 || res.data.length > 1) {
              // If result comes empty or contains more than 1 data
              this.openUpClientPopup(payload);
            } else {
              // If result is exactly one row
              this.chosenCustomer = res.data[0] as StoreList;
            }
          }
        },
        err => {

        }
      )
    })
  }

  openUpClientPopup(payload: any) {
    if (this.flowType == 'bon-reception') {
      this.displayType = 'reception-contact-popup';
    }

    if (this.flowType == 'bon-livraison') {
      this.displayType = 'livrasion-contact-popup';
    }

    if (this.flowType == 'achat-commande') {
      this.displayType = 'achat-commande-contact-popup';
    }

    const dialog = this.dialog.open(ClientPopupComponent, {
      data: {
        displayType: this.displayType,
        searchPayload: payload,
        currentCustomer: this.chosenCustomer
      },
      maxHeight: "90vh",
      minWidth:"70vw"
    });

    dialog.afterClosed().subscribe((res) => {
      if (res !== undefined && res !== null) {
        if(res.status === 'success') {
          this.chosenCustomer = res.data;
          this.searchClientForm.get("SEARCH_KEYWORD")?.setValue("");
        }
      }
    });
  }
}
