import {Component, Input, OnDestroy, OnInit} from '@angular/core';
import {CustomHttpResponse, LinkedDocuments} from "../../_models";
import {Subscription} from "rxjs";
import {CommonService} from "../../_services/common.service";
import {NgxSpinnerService} from "ngx-spinner";

@Component({
  selector: 'app-bon-linked-documents',
  templateUrl: './bon-linked-documents.component.html',
  styleUrls: ['./bon-linked-documents.component.scss']
})
export class BonLinkedDocumentsComponent implements OnInit, OnDestroy {

  @Input() flowType: string | undefined;

  _document_no: string | undefined;
  get document_no(): string | undefined { return this._document_no }
  @Input() set document_no(document_no: string | undefined) {
    if(document_no !== undefined && document_no !== null && !document_no.includes('*')) {
      this.loadDependencies();
    }

    this._document_no = document_no;
  }

  linkedDocuments: LinkedDocuments | undefined;

  subscriptions: Subscription[] | undefined;

  constructor(private commonService: CommonService,
              private loadingService: NgxSpinnerService) { }

  ngOnInit(): void {
  }

  ngOnDestroy() {
    if(this.subscriptions && this.subscriptions.length > 0) {
      this.subscriptions.forEach((_s) => {
        if(_s) {
          _s.unsubscribe();
        }
      });
    }
  }

  async loadDependencies() {
    await this.loadingService.show();

    let document_type = "";
    if(this.flowType === 'bon-livraison') {
      document_type = "BL";
    } else if(this.flowType === 'bon-reception') {
      document_type = "BR";
    } else if(this.flowType === 'transfer-de-stock') {
      document_type = "TR";
    } else if(this.flowType === 'achat-commande') {
      document_type = "AC";
    }

    const payload = {
      document_type,
      document_no: this.document_no as string
    };

    await this._getLinkedDocuments(payload);

    await this.loadingService.hide();
  }

  _getLinkedDocuments(data: { document_type: string, document_no: string }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.commonService.getLinkedDocuments(data).subscribe({
        next: (res: CustomHttpResponse<LinkedDocuments>) => {
          if(res.status === 'success') {
            this.linkedDocuments = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

}
