import {Component, Inject, OnInit} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {MatSelectionListChange} from "@angular/material/list";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {APP_CONSTANTS} from "../../_helpers/constants";
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-fusionner-type',
  templateUrl: './fusionner-type.component.html',
  styleUrls: ['./fusionner-type.component.scss']
})
export class FusionnerTypeComponent implements OnInit {

  submitted = false;
  fusionnerTypeForm: FormGroup = this.formBuilder.group({
    TYPE: [[APP_CONSTANTS.FUSIONNER_BOTH], Validators.required]
  });

  xTitle: string = 'X';
  yTitle: string = 'Y';

  typeOptions: { label: string, type: string, description: string }[] = [];

  constructor(public dialogRef: MatDialogRef<FusionnerTypeComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private formBuilder: FormBuilder,
              private translateService: TranslateService) { }

  ngOnInit(): void {
    this.xTitle = this.data.x;
    this.yTitle = this.data.y;

    this.typeOptions = [
      {
        label: this.translateService.instant('KFUSIONNER_TYPE_X_TO_Y', {XTITLE: this.xTitle , YTITLE: this.yTitle}),
        description: this.translateService.instant('KFUSIONNER_TYPE_DETAILS_X_TO_Y', {XTITLE: this.xTitle , YTITLE: this.yTitle}),
        type: APP_CONSTANTS.FUSIONNER_X_TO_Y
      },
      {
        label: this.translateService.instant('KFUSIONNER_TYPE_Y_TO_X', {YTITLE: this.yTitle, XTITLE: this.xTitle}),
        description: this.translateService.instant('KFUSIONNER_TYPE_DETAILS_Y_TO_X', {YTITLE: this.yTitle, XTITLE: this.xTitle}),
        type: APP_CONSTANTS.FUSIONNER_Y_TO_X
      },
      {
        label: this.translateService.instant('KFUSIONNER_TYPE_KEEP_BOTH', {XTITLE: this.xTitle , YTITLE: this.yTitle}),
        description: this.translateService.instant('KFUSIONNER_TYPE_NOTHING_CHANGES'),
        type: APP_CONSTANTS.FUSIONNER_BOTH
      },
    ];
  }

  get typeForm() {
    return this.fusionnerTypeForm;
  }

  onTypeChange(event: MatSelectionListChange) {
    let chosenOption = [...event.options];
    chosenOption = chosenOption.map(r => r.value);

    this.fusionnerTypeForm.get('TYPE')?.setValue(chosenOption);
  }

  fusionnerTypeFormSubmitter() {
    this.submitted = true;
    if(this.fusionnerTypeForm.invalid) {
      return;
    }

    const formData = this.fusionnerTypeForm.getRawValue();
    const [chosenType] = formData.TYPE;

    this.dialogRef.close({ status: 'success', data: chosenType });
    this.submitted = false;
  }

}
