import { Component, Inject, OnInit } from '@angular/core';
import { MAT_DIALOG_DATA } from '@angular/material/dialog';
import { PageEvent } from '@angular/material/paginator';
import { NgxSpinnerService } from 'ngx-spinner';
import { CustomHttpResponse } from 'src/app/_models';
import { CommonService } from 'src/app/_services/common.service';

@Component({
  selector: 'app-historique',
  templateUrl: './historique.component.html',
  styleUrls: ['./historique.component.scss']
})
export class HistoriqueComponent implements OnInit {
  
  documentDetail:any;
  historiqueData:any=[];

  isLoading = false;
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  displayedColumnsTypeDeMaterial:string[] = ['DOCUMENT_NO', 'USER_NAME','SUMMARY', 'DATED'];

  constructor(@Inject(MAT_DIALOG_DATA) public data: any,
              private loadingService: NgxSpinnerService,
              private commonService: CommonService) { }

  ngOnInit(): void {
    this.documentDetail = this.data;
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    this._getHistoriqueInfo(this.documentDetail, this.currentPage, this.pageSize);
    await this.loadingService.hide();
  }

  _getHistoriqueInfo(data: any, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        if(res.status === "success") {
          this.historiqueData = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.commonService.searchHistorique(data, page, pageSize).subscribe( observer);
  }

  pageChanged(event: PageEvent) {
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._getHistoriqueInfo(this.documentDetail, this.currentPage, this.pageSize);
  }


}
