import {Component, Inject, OnInit} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {APP_CONSTANTS} from "../../_helpers/constants";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {MatSelectionListChange} from "@angular/material/list";

@Component({
  selector: 'app-selection-copy-type',
  templateUrl: './selection-copy-type.component.html',
  styleUrls: ['./selection-copy-type.component.scss']
})
export class SelectionCopyTypeComponent implements OnInit {

  documentType: 'BR' | 'BL' | 'TR' | undefined;

  submitted = false;
  selectionTypeForm: FormGroup = this.formBuilder.group({
    TYPE: [null, Validators.required]
  });

  selectionTypeList = APP_CONSTANTS.SELECTION_TYPES_LIST;

  constructor(public dialogRef: MatDialogRef<SelectionCopyTypeComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private formBuilder: FormBuilder) { }

  ngOnInit(): void {
    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('documentType')) {
        this.documentType = this.data.documentType;

        this.selectionTypeList = this.selectionTypeList.filter((row) => {
          return row.value.substring(0, 2) == this.documentType;
        });
      }
    }
  }

  get typeForm() {
    return this.selectionTypeForm;
  }

  onTypeChange(event: MatSelectionListChange) {
    let chosenOption = [...event.options];
    chosenOption = chosenOption.map(r => r.value);

    this.selectionTypeForm.get('TYPE')?.setValue(chosenOption);
  }

  selectionTypeFormSubmitter() {
    this.submitted = true;
    if(this.selectionTypeForm.invalid) {
      return;
    }

    const formData = this.selectionTypeForm.getRawValue();
    const [chosenType] = formData.TYPE;

    this.dialogRef.close({ status: 'success', data: chosenType });
    this.submitted = false;
  }

}
