import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, Router } from '@angular/router';
import { Observable } from 'rxjs';
import { TokenStorageService } from '../_services/token-storage.service';

@Injectable({
  providedIn: 'root'
})

// AuthGuard is used to check whether the user is logged in or not
// If not logged in, redirect the user to the Login route.
// Mainly used with all Routes other than Login route.
export class AuthGuard implements CanActivate {

  isLoggedIn: boolean = false;

  constructor(private router: Router,
              private tokenStorageService: TokenStorageService) {
  }

  canActivate(next: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | Promise<boolean> | boolean {
    this.isLoggedIn = (this.tokenStorageService.getToken() !== undefined && this.tokenStorageService.getToken() !== null);


    if (!this.isLoggedIn) {
      this.router.navigate(['/login']); // go to login if not authenticated
      return false;
    }

    return true;
  }

}
