import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { Observable } from 'rxjs';
import { APP_CONSTANTS } from '../_helpers/constants';
import { DialogService } from '../_services/dailog.service';
import { TokenStorageService } from '../_services/token-storage.service';

@Injectable({
  providedIn: 'root'
})

// AuthenticatedGuard is used to check whether the user is logged in or not
// If already logged in, redirect the user to the Home route.
// Mainly used with Login Route
export class AuthPermissionGuard implements CanActivate {

  isLoggedIn: boolean = false;
  rolePermission:any;

  constructor(private dialogService: DialogService,
              private translateService:TranslateService) {
  }

  canActivate(next: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | Promise<boolean> | boolean {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
      console.log(this.rolePermission[next.data['id']])
      if(!this.rolePermission[next.data['id']]){
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return false;
      }
    }
    return true;
  }

}
