import { Injectable } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, Router } from '@angular/router';
import { Observable } from 'rxjs';
import { TokenStorageService } from '../_services/token-storage.service';
import {APP_CONSTANTS} from "../_helpers/constants";

@Injectable({
  providedIn: 'root'
})

// AuthenticatedGuard is used to check whether the user is logged in or not
// If already logged in, redirect the user to the Home route.
// Mainly used with Login Route
export class AuthenticatedGuard implements CanActivate {

  isLoggedIn: boolean = false;

  constructor(private router: Router,
              private tokenStorageService: TokenStorageService) {
  }

  canActivate(next: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | Promise<boolean> | boolean {
    this.isLoggedIn = (this.tokenStorageService.getToken() !== undefined && this.tokenStorageService.getToken() !== null);

    if (this.isLoggedIn) {
      this.router.navigate([APP_CONSTANTS.HOME_ROUTE]); // go to login if not authenticated
      return false;
    }

    return true;
  }

}
