import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {CustomHttpResponse, AchatCommandeList, BonAchatCommande} from "../_models";
import {environment} from "../../environments/environment";

@Injectable({
  providedIn: 'root'
})
export class AchatCommandeService {

  constructor(private httpClient: HttpClient) { }

  getAchatCommandeList(data: any, page: number, pageSize: number): Observable<CustomHttpResponse<AchatCommandeList[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<AchatCommandeList[]>>(environment.apiUrl + "/api/achat/list", data, options);
  }

  createAchatCommande(data: BonAchatCommande): Observable<CustomHttpResponse<BonAchatCommande>> {
    return this.httpClient.post<CustomHttpResponse<BonAchatCommande>>(environment.apiUrl + `/api/achat/create`, data);
  }

  modifyAchatCommande(document_no: string, data: BonAchatCommande): Observable<CustomHttpResponse<BonAchatCommande>> {
    return this.httpClient.post<CustomHttpResponse<BonAchatCommande>>(environment.apiUrl + `/api/achat/modify/${document_no}`, data);
  }

  deleteAchatCommande(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/achat/remove/${document_no}`);
  }

  splittingAchatCommande(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.get<CustomHttpResponse<any>>(environment.apiUrl + `/api/achat/split/${document_no}`);
  }


  getAchatCommandeDetail(NUMBON: any): Observable<CustomHttpResponse<BonAchatCommande>> {
    return this.httpClient.get<CustomHttpResponse<BonAchatCommande>>(environment.apiUrl + `/api/achat/view/${NUMBON}`);
  }

  generatePdf(DOC_NO: string, lang: string) {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      lang
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get(environment.apiUrl + `/api/achat/view_pdf/${DOC_NO}`, options);
  }

}
