import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { ArticleCategoryInfo, CustomHttpResponse } from '../_models';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';

@Injectable({
  providedIn: 'root'
})
export class ArticleCategoryService {

  constructor(private httpClient: HttpClient) { }

  searchCategoryInfo(data: { keyword: string }, page: number, pageSize: number = 10): Observable<CustomHttpResponse<ArticleCategoryInfo[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<ArticleCategoryInfo[]>>(environment.apiUrl + '/api/article_categories/list', data, options);
  }

  deleteCategoryById(cat_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/article_categories/remove/${cat_id}`);
  }

  createCategory(data: any): Observable<CustomHttpResponse<ArticleCategoryInfo>> {
    return this.httpClient.post<CustomHttpResponse<ArticleCategoryInfo>>(environment.apiUrl + `/api/article_categories/create`, data);
  }

  getCategoryDetailById(cat_id: number): Observable<CustomHttpResponse<ArticleCategoryInfo>> {
    return this.httpClient.get<CustomHttpResponse<ArticleCategoryInfo>>(environment.apiUrl + `/api/article_categories/view/${cat_id}`);
  }

  modifyCategoryById(cat_id: number, data: any): Observable<CustomHttpResponse<ArticleCategoryInfo>> {
    return this.httpClient.post<CustomHttpResponse<ArticleCategoryInfo>>(environment.apiUrl + `/api/article_categories/modify/${cat_id}`, data);
  }

}
