import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {CustomHttpResponse, Store} from "../_models";
import {Article, ArticleCategories, ArticleMaterialTypes, ArticleVat} from "../_models/article";

@Injectable({
  providedIn: 'root'
})
export class ArticleService {

  constructor(private httpClient: HttpClient) { }

  searchArticleList(payload: any, page = 1, pageSize = 10): Observable<CustomHttpResponse<Article[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<Article[]>>(environment.apiUrl + `/api/articles/list`, payload, options);
  }

  getArticleDetail(article_id: number): Observable<CustomHttpResponse<Article>> {
    return this.httpClient.get<CustomHttpResponse<Article>>(environment.apiUrl + `/api/articles/detail/${article_id}`);
  }

  getArticleCategories(): Observable<CustomHttpResponse<ArticleCategories[]>> {
    return this.httpClient.get<CustomHttpResponse<ArticleCategories[]>>(environment.apiUrl + "/api/articles/list_categories");
  }

  getArticleMaterialTypes(): Observable<CustomHttpResponse<ArticleMaterialTypes[]>> {
    return this.httpClient.get<CustomHttpResponse<ArticleMaterialTypes[]>>(environment.apiUrl + "/api/articles/list_material_types");
  }

  getArticleVatList(): Observable<CustomHttpResponse<ArticleVat[]>> {
    return this.httpClient.get<CustomHttpResponse<ArticleVat[]>>(environment.apiUrl + "/api/articles/list_vat");
  }

  mergeTwoArticles(data: {}) {
    return this.httpClient.post(environment.apiUrl + `/api/articles/fusionner`, data);
  }

  createArticle(data: {}): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + '/api/articles/create', data);
  }

  updateArticle(article_id: number, data: {}): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + `/api/articles/modify/${article_id}`, data);
  }

  removeArticle(article_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/articles/remove/${article_id}`);
  }

}
