import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
import { environment } from '../../environments/environment';
import {APP_CONSTANTS} from "../_helpers/constants";
import {CustomHttpResponse} from "../_models";

@Injectable({
  providedIn: 'root'
})
export class AuthService {

  apiUrl: string;

  constructor(private http: HttpClient) {
    this.apiUrl = `${environment.apiUrl}`;
  }

  refreshToken(token: string): Observable<CustomHttpResponse<any>> {
    const payload = {
      refresh_token: token
    };

    return this.http.post<CustomHttpResponse<any>>(this.apiUrl + '/api/refresh_token', payload);
  }

  login(payload: { username: string, password: string }): Observable<CustomHttpResponse<any>> {
    return this.http.post<CustomHttpResponse<any>>(this.apiUrl + '/auth/login', payload);
  }

  getCurrentUser() {
    const user = localStorage.getItem(APP_CONSTANTS.USER_KEY);
    if (user !== undefined && user !== null) {
      return JSON.parse(user);
    }

    return undefined;
  }

}
