import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import {environment} from "../../environments/environment";
import {CustomHttpResponse, LivraisonExpedition} from '../_models';
import {BonLivraison, ListLivraison} from '../_models/livraison';

@Injectable({
  providedIn: 'root'
})
export class BonDeLivraisonService {

  constructor(private httpClient: HttpClient) { }

  getLivraisonList(data: any, page: number, pageSize: number): Observable<CustomHttpResponse<ListLivraison[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<ListLivraison[]>>(environment.apiUrl + "/api/livraison/list", data, options);
  }

  createLivraison(data: BonLivraison): Observable<CustomHttpResponse<BonLivraison>> {
    return this.httpClient.post<CustomHttpResponse<BonLivraison>>(environment.apiUrl + `/api/livraison/create`, data);
  }

  deleteLivraison(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/livraison/remove/${document_no}`);
  }

  modifyLivrasion(document_no: string, data: BonLivraison): Observable<CustomHttpResponse<BonLivraison>> {
    return this.httpClient.post<CustomHttpResponse<BonLivraison>>(environment.apiUrl + `/api/livraison/modify/${document_no}`, data);
  }

  createLivraisonExpedition(data: LivraisonExpedition): Observable<CustomHttpResponse<LivraisonExpedition>> {
    return this.httpClient.post<CustomHttpResponse<LivraisonExpedition>>(environment.apiUrl + `/api/livraison_expedition/create`, data);
  }

  modifyLivraisonExpedition(expedition_id: number, data: LivraisonExpedition): Observable<CustomHttpResponse<LivraisonExpedition>> {
    return this.httpClient.post<CustomHttpResponse<LivraisonExpedition>>(environment.apiUrl + `/api/livraison_expedition/modify/${expedition_id}`, data);
  }

  getLivraisonExpedition(document_no: string): Observable<CustomHttpResponse<LivraisonExpedition>> {
    return this.httpClient.get<CustomHttpResponse<LivraisonExpedition>>(environment.apiUrl + `/api/livraison_expedition/view/${document_no}`);
  }

  generatePdf(DOC_NO: string, lang: string) {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      lang
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get(environment.apiUrl + `/api/livraison/view_pdf/${DOC_NO}`, options);
  }

  getLivraisonDetail(NUMBON:any): Observable<CustomHttpResponse<BonLivraison>> {
    return this.httpClient.post<CustomHttpResponse<BonLivraison>>(environment.apiUrl + `/api/livraison/view/`+NUMBON,'');
  }
  getArticlebyId(art_id:any,loc_id:any){
    return this.httpClient.get(environment.apiUrl + `/api/common/article_detail/`+art_id+'?location_id='+loc_id);
  }
}
