import {HttpClient, HttpHeaders} from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import {Observable} from "rxjs";
import {BonReception, CustomHttpResponse, ListReception} from "../_models";

@Injectable({
  providedIn: 'root'
})
export class BonDeReceptionService {

  constructor(private httpClient: HttpClient) { }

  createReception(data: BonReception): Observable<CustomHttpResponse<BonReception>> {
    return this.httpClient.post<CustomHttpResponse<BonReception>>(environment.apiUrl + `/api/reception/create`, data);
  }

  modifyReception(document_no: string, data: BonReception): Observable<CustomHttpResponse<BonReception>> {
    return this.httpClient.post<CustomHttpResponse<BonReception>>(environment.apiUrl + `/api/reception/modify/${document_no}`, data);
  }

  getReceptionDetail(NUMBON:{}): Observable<CustomHttpResponse<BonReception>> {
    return this.httpClient.get<CustomHttpResponse<BonReception>>(environment.apiUrl + `/api/reception/view/` + NUMBON);
  }

  deleteReception(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/reception/remove/${document_no}`);
  }

  generatePdf(DOC_NO: string, lang: string) {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      lang
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get(environment.apiUrl + `/api/reception/view_pdf/${DOC_NO}`, options);
  }

  // JayaRaj Changes in Bon de reception
  getReceptionList(data: { document_no: string, start_date: string, end_date: string }, page= 1, pageSize= 10): Observable<CustomHttpResponse<ListReception[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<ListReception[]>>(environment.apiUrl + '/api/reception/list', data, options);
  }
}
