import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {CustomHttpResponse, Store, Contact, ContactList, StoreList} from "../_models";
import {Brand, BrandList} from "../_models/brand";

@Injectable({
  providedIn: 'root'
})
export class ContactService {

  constructor(private httpClient: HttpClient) {}

  getContactList(payload: { filter_type: string, keyword: string, store_type?: string, store_brand?: string, store_category?: string }, page: number, pageSize: number): Observable<CustomHttpResponse<ContactList[] | StoreList[] | BrandList[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: {
        page: page,
        page_size: pageSize
      }
    };

    return this.httpClient.post<CustomHttpResponse<ContactList[] | StoreList[] | BrandList[]>>(environment.apiUrl + "/api/contact/list", payload, options);
  }

  getStoreListByBrand(data: { brand_id: number }): Observable<CustomHttpResponse<any[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: data
    };

    return this.httpClient.get<CustomHttpResponse<any[]>>(environment.apiUrl + "/api/contact/getstorebybrand", options);
  }

  createContact(data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + "/api/contact/createcontact", data);
  }

  updateContact(data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + "/api/contact/updatecontact", data);
  }

  createBrand(data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + "/api/contact/createbrand", data);
  }

  updateBrand(data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + "/api/contact/updatebrand", data);
  }

  createStore(data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + "/api/contact/createstore", data);
  }

  updateStore(data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + "/api/contact/updatestore", data);
  }

  getBrandDetail(data: { brand_id: number }): Observable<CustomHttpResponse<Brand>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: data
    };

    return this.httpClient.get<CustomHttpResponse<Brand>>(environment.apiUrl + "/api/contact/getbranddetail", options);
  }

  getStoreDetail(data: { store_id: number, store_type: string }): Observable<CustomHttpResponse<Store>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: data
    };

    return this.httpClient.get<CustomHttpResponse<Store>>(environment.apiUrl + "/api/contact/getstoredetail", options);
  }

  getContactDetail(data: { CTID: number }): Observable<CustomHttpResponse<Contact>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const options = {
      headers: headers,
      params: data
    };

    return this.httpClient.get<CustomHttpResponse<Contact>>(environment.apiUrl + "/api/contact/getcontactdetail", options);
  }

  mergeTwoStores(data: {}) {
    return this.httpClient.post(environment.apiUrl + `/api/contact/fusionner_stores`, data);
  }

  deleteContact(contact_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/contact/deletecontact/${contact_id}`);
  }

  deleteBrand(brand_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/contact/deletebrand/${brand_id}`);
  }

  deleteStore(data: { store_type: string, store_id: number }): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + '/api/contact/deletestore', data);
  }

}
