import {Injectable} from '@angular/core';
import {MatDialog} from '@angular/material/dialog';
import {lastValueFrom, Observable} from 'rxjs';
import {ConfirmComponent} from '../dialogs/confirm/confirm.component';
import {CustomAlertComponent, CustomAlertData} from "../dialogs/custom-alert/custom-alert.component";
import {TranslateService} from "@ngx-translate/core";

export interface ConfirmDialogData {
  title: string;
  message: string;
  confirmText?: string;
  cancelText?: string;
}

@Injectable({
  providedIn: 'root'
})
export class DialogService {

  constructor(private dialog: MatDialog,
              private translateService: TranslateService) { }

  confirmDialog(data: ConfirmDialogData, disableClose: boolean = true): Observable<boolean> {
    return this.dialog
      .open(ConfirmComponent, {
        data,
        width: '400px',
        disableClose,
      })
      .afterClosed();
  }

  displayAlert(data: CustomAlertData): Promise<boolean> {
    if(typeof data.message === 'object') {
      // If the message property contains an object
      // Ex: { LGILBL: "E_UNIQUE_LABEL" }
      let errorMessages: string[] = Object.values(data.message);
      data.message = errorMessages.reduce((acc, item, i) => {
        if (acc !== "") {
          acc += "<br>";
        }

        acc += `<span>${this.translateService.instant(item)}</span>`;

        return acc;
      }, "");
    }

    return lastValueFrom(this.dialog
      .open(CustomAlertComponent, {
        data,
        width: '400px',
        disableClose: false,
      })
      .afterClosed());
  }
}
