import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { CustomHttpResponse, DepotTNT } from '../_models';

@Injectable({
  providedIn: 'root'
})
export class DepotTntService {

  apiUrl: string;
  
  constructor(private http: HttpClient) { 
    this.apiUrl = `${environment.apiUrl}`;
  }

  searchDepotTNTInfo(data: { keyword: string}, page: number, pageSize: number): Observable<CustomHttpResponse<DepotTNT[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<DepotTNT[]>>(environment.apiUrl + '/api/depot_tnt/list', data, options);
  }

  createDepotTnt(data:{label:string,status:string}): Observable<CustomHttpResponse<DepotTNT[]>>{
    return this.http.post<CustomHttpResponse<DepotTNT[]>>(environment.apiUrl + '/api/depot_tnt/create', data);
  }

  deleteDepotTnt(depot_tnt_id: number): Observable<CustomHttpResponse<any>> {
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/depot_tnt/remove/${depot_tnt_id}`);
  }

  viewDepotTntDetail(depot_tnt_id: number | undefined): Observable<CustomHttpResponse<DepotTNT>> {
    return this.http.get<CustomHttpResponse<DepotTNT>>(environment.apiUrl + `/api/depot_tnt/view/`+depot_tnt_id);
  }

  modifyDepotTnt(data:DepotTNT,depot_tnt_id:number | undefined): Observable<CustomHttpResponse<DepotTNT>>{
    return this.http.post<CustomHttpResponse<DepotTNT>>(environment.apiUrl + `/api/depot_tnt/modify/${depot_tnt_id}`,data);
  }

}
