import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { CustomHttpResponse, Fabriquant } from '../_models';

@Injectable({
  providedIn: 'root'
})
export class FabriquantService {

  constructor(private httpClient: HttpClient) { }

  searchFabriquantInfo(data: { keyword: string }, page: number, pageSize: number = 10): Observable<CustomHttpResponse<Fabriquant[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<Fabriquant[]>>(environment.apiUrl + '/api/manufacturer/list', data, options);
  }

  deleteFabriquantById(fab_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/manufacturer/remove/ ${fab_id}`);
  }

  createFabriquant(data: Fabriquant): Observable<CustomHttpResponse<Fabriquant>> {
    return this.httpClient.post<CustomHttpResponse<Fabriquant>>(environment.apiUrl + `/api/manufacturer/create`, data);
  }

  getFabriquantDetailById(fab_id: number): Observable<CustomHttpResponse<Fabriquant>> {
    return this.httpClient.get<CustomHttpResponse<Fabriquant>>(environment.apiUrl + `/api/manufacturer/view/ ${fab_id}`);
  }

  modifyFabriquantById(fab_id: number, data: any): Observable<CustomHttpResponse<Fabriquant>> {
    return this.httpClient.post<CustomHttpResponse<Fabriquant>>(environment.apiUrl + `/api/manufacturer/modify/ ${fab_id}`, data);
  }

}
