import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {environment} from "../../environments/environment";
import { CustomHttpResponse } from '../_models';
import { Observable } from 'rxjs';
import {BonInventory, InventoryList} from '../_models/inventory';
import {Article} from "../_models/article";

@Injectable({
  providedIn: 'root'
})
export class InventoryService {

  constructor(private httpClient: HttpClient) { }

  searchInventory(data: { document_no: string, start_date: string, end_date: string }, page: number, pageSize: number = 10): Observable<CustomHttpResponse<InventoryList[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<InventoryList[]>>(environment.apiUrl + '/api/inventory/getinventorylist', data, options);
  }

  generateInventoryArticles(data: { location_id: number }, page: number, pageSize: number = 50): Observable<CustomHttpResponse<Article[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<Article[]>>(environment.apiUrl + '/api/inventory/generate_article', data, options);
  }

  generatePdf(DOC_NO: string, lang: string) {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      lang
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get(environment.apiUrl + `/api/inventory/view_pdf/${DOC_NO}`, options);
  }

  getInventoryDetail(NUMBON:{}): Observable<CustomHttpResponse<BonInventory>> {
    return this.httpClient.get<CustomHttpResponse<BonInventory>>(environment.apiUrl + `/api/inventory/getinventorydetail/` + NUMBON);
  }

  createInventory(data: BonInventory): Observable<CustomHttpResponse<BonInventory>> {
    return this.httpClient.post<CustomHttpResponse<BonInventory>>(environment.apiUrl + `/api/inventory/create`, data);
  }

  modifyInventory(document_no: string, data: BonInventory): Observable<CustomHttpResponse<BonInventory>> {
    return this.httpClient.post<CustomHttpResponse<BonInventory>>(environment.apiUrl + `/api/inventory/modify/${document_no}`, data);
  }

  deleteInventory(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/inventory/remove/${document_no}`);
  }
}
