import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {CustomHttpResponse} from "../_models";
import {BonInventory, InventoryList} from "../_models/inventory";
import {environment} from "../../environments/environment";
import {LigneInfo} from "../_models/ligne";

@Injectable({
  providedIn: 'root'
})
export class LigneService {

  constructor(private httpClient: HttpClient) { }

  searchLigneInfo(data: { keyword: string }, page: number, pageSize: number = 10): Observable<CustomHttpResponse<LigneInfo[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<LigneInfo[]>>(environment.apiUrl + '/api/ligne/list', data, options);
  }

  getLigneDetail(ligne_id: number): Observable<CustomHttpResponse<LigneInfo>> {
    return this.httpClient.get<CustomHttpResponse<LigneInfo>>(environment.apiUrl + `/api/ligne/view/${ligne_id}`);
  }

  createLigne(data: any): Observable<CustomHttpResponse<LigneInfo>> {
    return this.httpClient.post<CustomHttpResponse<LigneInfo>>(environment.apiUrl + `/api/ligne/create`, data);
  }

  modifyLigne(ligne_id: number, data: any): Observable<CustomHttpResponse<LigneInfo>> {
    return this.httpClient.post<CustomHttpResponse<LigneInfo>>(environment.apiUrl + `/api/ligne/modify/${ligne_id}`, data);
  }

  deleteLigne(ligne_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/ligne/remove/${ligne_id}`);
  }

}
