import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { CustomHttpResponse, LivraisonPrestation } from '../_models';
import { Observable } from 'rxjs';
import {HttpClient, HttpHeaders} from "@angular/common/http";

@Injectable({
  providedIn: 'root'
})
export class LivraisonPrestationService {

  constructor(private http: HttpClient) { }

  createLivraisonPrestation(data:LivraisonPrestation): Observable<CustomHttpResponse<LivraisonPrestation>>{
    return this.http.post<CustomHttpResponse<LivraisonPrestation>>(environment.apiUrl + '/api/livraison_prestation/create', data);
  }

  getLivraisonPrestationById(prestation_id:number): Observable<CustomHttpResponse<LivraisonPrestation>>{
    return this.http.get<CustomHttpResponse<LivraisonPrestation>>(environment.apiUrl + `/api/livraison_prestation/view/${prestation_id}`);
  }

  getLivraisonPrestationList(): Observable<CustomHttpResponse<LivraisonPrestation>>{
    return this.http.get<CustomHttpResponse<LivraisonPrestation>>(environment.apiUrl + '/api/common/getLivraisonPrestationList');
  }

  deleteLivraisonPrestationById(prestation_id:number): Observable<CustomHttpResponse<any>>{
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/livraison_prestation/remove/${prestation_id}`);
  }

  modifyLivraisonPrestation(prestation_id:number, payload:LivraisonPrestation): Observable<CustomHttpResponse<LivraisonPrestation[]>>{
    return this.http.post<CustomHttpResponse<LivraisonPrestation[]>>(environment.apiUrl + `/api/livraison_prestation/modify/${prestation_id}`,payload);
  }

  searchLivraisonPrestation(data: { keyword: any}, page: number, pageSize: number): Observable<CustomHttpResponse<LivraisonPrestation[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<LivraisonPrestation[]>>(environment.apiUrl + '/api/livraison_prestation/list', data, options);
  }
}
