import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {CustomHttpResponse, QuickSearchArticle, QuickSearchContact, QuickSearchDocument} from "../_models";
import {environment} from "../../environments/environment";

@Injectable({
  providedIn: 'root'
})
export class QuickSearchService {
  
  dashboardFlag: boolean = false;
  quickSearchFormData:any = [];
  constructor(private httpClient: HttpClient) { }

  searchContactsList(data: { keyword: string }, page: number, pageSize: number): Observable<CustomHttpResponse<QuickSearchContact[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<QuickSearchContact[]>>(environment.apiUrl + "/api/quick_search/contacts", data, options);
  }

  searchArticlesList(data: { keyword: string, location_id: number }, page: number, pageSize: number): Observable<CustomHttpResponse<QuickSearchArticle[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<QuickSearchArticle[]>>(environment.apiUrl + "/api/quick_search/articles", data, options);
  }

  searchDocumentsList(data: { keyword: string }, page: number, pageSize: number): Observable<CustomHttpResponse<QuickSearchDocument[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<QuickSearchDocument[]>>(environment.apiUrl + "/api/quick_search/documents", data, options);
  }

}
