import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { CustomHttpResponse } from '../_models';
import { ReportingPlanification } from '../_models/reporting-planification';

@Injectable({
  providedIn: 'root'
})
export class ReportingPlanificationService {

  apiUrl: string;
  reportingFormData:any = [];
  constructor(private http: HttpClient) { 
    this.apiUrl = `${environment.apiUrl}`;
  }

  searchReportingPlanification(data: { title: string, period: string}, page: number, pageSize: number): Observable<CustomHttpResponse<ReportingPlanification[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<ReportingPlanification[]>>(environment.apiUrl + '/api/planification/list', data, options);
  }

  createReportingPlanification(data:ReportingPlanification): Observable<CustomHttpResponse<ReportingPlanification[]>>{
    return this.http.post<CustomHttpResponse<ReportingPlanification[]>>(environment.apiUrl + '/api/planification/create', data);
  }

  modifyReportingPlanification(plan_id:number, data:ReportingPlanification): Observable<CustomHttpResponse<ReportingPlanification[]>>{
    return this.http.post<CustomHttpResponse<ReportingPlanification[]>>(environment.apiUrl + `/api/planification/modify/${plan_id}`, data);
  }

  getStockLocationDetail(plan_id: number): Observable<CustomHttpResponse<ReportingPlanification>> {
    return this.http.get<CustomHttpResponse<ReportingPlanification>>(environment.apiUrl + `/api/planification/detail/${plan_id}`);
  }

  deleteReportingPlanificationById(plan_id:number): Observable<CustomHttpResponse<any>>{
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/planification/remove/${plan_id}`);
  }

}
