import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {Observable} from "rxjs";
import {
  CustomHttpResponse,
  ReportColumnType,
  ReportDownloadData,
  ReportingType, ReportListData,
  ReportRequest,
  StockMovementType
} from "../_models";
import {environment} from "../../environments/environment";

@Injectable({
  providedIn: 'root'
})
export class ReportingService {

  constructor(private httpClient: HttpClient) { }

  getReportingTypesList(): Observable<CustomHttpResponse<ReportingType[]>> {
    return this.httpClient.get<CustomHttpResponse<ReportingType[]>>(environment.apiUrl + "/api/reporting/report_type_list");
  }

  getStockMovementTypesList(): Observable<CustomHttpResponse<StockMovementType[]>> {
    return this.httpClient.get<CustomHttpResponse<StockMovementType[]>>(environment.apiUrl + "/api/reporting/stock_type_list");
  }

  getReportColumnList(): Observable<CustomHttpResponse<ReportColumnType[]>> {
    return this.httpClient.get<CustomHttpResponse<ReportColumnType[]>>(environment.apiUrl + "/api/reporting/column_selection_list");
  }

  generateReport(data: ReportRequest, page: number, pageSize: number): Observable<CustomHttpResponse<ReportListData[] | ReportDownloadData>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<ReportListData[] | ReportDownloadData>>(environment.apiUrl + "/api/reporting/generate_excel", data, options);
  }

}
