import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { CustomHttpResponse, ServiceExpedition } from '../_models';

@Injectable({
  providedIn: 'root'
})
export class ServiceExpeditionService {

  apiUrl: string;
  
  constructor(private http: HttpClient) { 
    this.apiUrl = `${environment.apiUrl}`;
  }

  searchServiceExpeditionInfo(data: { keyword: string}, page: number, pageSize: number): Observable<CustomHttpResponse<ServiceExpedition[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<ServiceExpedition[]>>(environment.apiUrl + '/api/service_expedition/list', data, options);
  }

  createServiceExpedition(data:{label:string,status:string}): Observable<CustomHttpResponse<ServiceExpedition[]>>{
    return this.http.post<CustomHttpResponse<ServiceExpedition[]>>(environment.apiUrl + '/api/service_expedition/create', data);
  }

  deleteServiceExpedition(service_expedition_id: number): Observable<CustomHttpResponse<any>> {
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/service_expedition/remove/${service_expedition_id}`);
  }

  viewServiceExeditionDetail(service_expedition_id: number): Observable<CustomHttpResponse<ServiceExpedition>> {
    return this.http.get<CustomHttpResponse<ServiceExpedition>>(environment.apiUrl + `/api/service_expedition/view/`+service_expedition_id);
  }

  modifyServiceExpedition(data:ServiceExpedition,service_expedition_id:number): Observable<CustomHttpResponse<ServiceExpedition>>{
    return this.http.post<CustomHttpResponse<ServiceExpedition>>(environment.apiUrl + `/api/service_expedition/modify/${service_expedition_id}`,data);
  }


}
