import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import {Observable} from "rxjs";
import { HttpClient, HttpHeaders } from '@angular/common/http';
import {CustomHttpResponse, StockLocation} from '../_models';

@Injectable({
  providedIn: 'root'
})
export class StockLocationService {

  constructor(private httpClient: HttpClient) { }

  searchStockLocationList(data: { stock_location_lbl: string }, page: number, pageSize: number): Observable<CustomHttpResponse<StockLocation[]>>{
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.post<CustomHttpResponse<StockLocation[]>>(environment.apiUrl + '/api/stocklocation/list', data, options);
  }

  getStockLocationDetail(location_id: number): Observable<CustomHttpResponse<StockLocation>> {
    return this.httpClient.get<CustomHttpResponse<StockLocation>>(environment.apiUrl + `/api/stocklocation/view/${location_id}`);
  }

  createStockLocation(data: { brand_id: number, stock_location_lbl: string, stock_location_status: number }): Observable<CustomHttpResponse<StockLocation>> {
    return this.httpClient.post<CustomHttpResponse<StockLocation>>(environment.apiUrl + `/api/stocklocation/create`, data);
  }

  modifyStockLocation(location_id: number, data: { brand_id: number, stock_location_lbl: string, stock_location_status: number }): Observable<CustomHttpResponse<StockLocation>> {
    return this.httpClient.post<CustomHttpResponse<StockLocation>>(environment.apiUrl + `/api/stocklocation/modify/${location_id}`, data);
  }

  deleteStockLocation(location_id: number): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/stocklocation/remove/${location_id}`);
  }

}
