import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {CustomHttpResponse, ListInventoryTournant, BonTournant} from '../_models';
import { Observable } from 'rxjs';
import {Article} from "../_models/article";

@Injectable({
  providedIn: 'root'
})
export class TournantService {

  constructor(private httpClient: HttpClient) { }

  getTournantList(data:any,page:number, pageSize:number): Observable<CustomHttpResponse<ListInventoryTournant[]>>{
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<ListInventoryTournant[]>>(environment.apiUrl + `/api/inv_tournant/list`, data, options);
  }

  createTournant(data: {}): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + `/api/inv_tournant/create`, data);
  }

  modifyTournant(document_no: string, data: any): Observable<CustomHttpResponse<any>> {
    return this.httpClient.post<CustomHttpResponse<any>>(environment.apiUrl + `/api/inv_tournant/modify/${document_no}`, data);
  }

  generateTournantRandomArticles(data: {}): Observable<CustomHttpResponse<Article[]>> {
    return this.httpClient.post<CustomHttpResponse<Article[]>>(environment.apiUrl + `/api/inv_tournant/generate_randomarticle`, data);
  }

  getTournantDetail(document_no: string): Observable<CustomHttpResponse<BonTournant>> {
    return this.httpClient.get<CustomHttpResponse<BonTournant>>(environment.apiUrl + `/api/inv_tournant/view/${document_no}`);
  }

  deleteTournant(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/inv_tournant/remove/${document_no}`);
  }

  generatePdf(DOC_NO: string, lang: string) {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      lang
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get(environment.apiUrl + `/api/inv_tournant/view_pdf/${DOC_NO}`, options);
  }

}
