import { Injectable } from '@angular/core';
import {HttpClient, HttpHeaders} from "@angular/common/http";
import {environment} from "../../environments/environment";
import { CustomHttpResponse, BonTransferDeStock, TransferDeStockList } from '../_models';
import { Observable } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class TransferDeStockService {

  constructor(private httpClient: HttpClient) { }

  createTransferDeStock(data: BonTransferDeStock): Observable<CustomHttpResponse<BonTransferDeStock>> {
    return this.httpClient.post<CustomHttpResponse<BonTransferDeStock>>(environment.apiUrl + `/api/transferdestock/create`, data);
  }

  modifyTransferDeStock(document_no: string, data: BonTransferDeStock): Observable<CustomHttpResponse<BonTransferDeStock>> {
    return this.httpClient.post<CustomHttpResponse<BonTransferDeStock>>(environment.apiUrl + `/api/transferdestock/modify/${document_no}`, data);
  }

  getTransferDeStockDetail(NUMBON:{}): Observable<CustomHttpResponse<BonTransferDeStock>> {
    return this.httpClient.get<CustomHttpResponse<BonTransferDeStock>>(environment.apiUrl + `/api/transferdestock/detail/` + NUMBON);
  }

  deleteTransferDeStock(document_no: string): Observable<CustomHttpResponse<any>> {
    return this.httpClient.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/transferdestock/remove/${document_no}`);
  }

  searchTransferDeStock(data: { document_no: string, start_date: string, end_date: string }, page:number, pageSize:number): Observable<CustomHttpResponse<TransferDeStockList[]>>{
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const queryParams = {
      page: page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: queryParams
    };

    return this.httpClient.post<CustomHttpResponse<TransferDeStockList[]>>(environment.apiUrl + `/api/transferdestock/list`, data, options);
  }

  generatePdf(DOC_NO: string, lang: string) {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      lang
    };

    const options = {
      headers: headers,
      params: params
    };

    return this.httpClient.get(environment.apiUrl + `/api/transferdestock/view_pdf/${DOC_NO}`, options);
  }

}
