import { Injectable } from '@angular/core';
import {environment} from "../../environments/environment";
import {HttpClient, HttpHeaders} from "@angular/common/http";
import { CustomHttpResponse, TypeDeMaterial } from '../_models';
import { Observable } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class TypeDeMaterialService {

  constructor(private http: HttpClient) { }

  createTypeDeMaterial(data:TypeDeMaterial): Observable<CustomHttpResponse<TypeDeMaterial>>{
    return this.http.post<CustomHttpResponse<TypeDeMaterial>>(environment.apiUrl + '/api/materialtype/create', data);
  }

  getTypeDeMaterialById(material_id:number): Observable<CustomHttpResponse<TypeDeMaterial>>{
    return this.http.get<CustomHttpResponse<TypeDeMaterial>>(environment.apiUrl + `/api/materialtype/view/${material_id}`);
  }

  deleteTypeDeMaterialById(material_id:number): Observable<CustomHttpResponse<any>>{
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/materialtype/remove/${material_id}`);
  }

  modifyTypeDeMaterial(material_id:number, payload:TypeDeMaterial): Observable<CustomHttpResponse<TypeDeMaterial[]>>{
    return this.http.post<CustomHttpResponse<TypeDeMaterial[]>>(environment.apiUrl + `/api/materialtype/modify/${material_id}`,payload);
  }

  searchTypeDeMaterial(data: { keyword: any}, page: number, pageSize: number): Observable<CustomHttpResponse<TypeDeMaterial[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<TypeDeMaterial[]>>(environment.apiUrl + '/api/materialtype/list', data, options);
  }
  
}
