import { HttpClient, HttpHeaders } from '@angular/common/http';
import { environment } from '../../environments/environment';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { CustomHttpResponse, Role, UserManagement } from '../_models';

@Injectable({
  providedIn: 'root'
})
export class UserManagementService {
  apiUrl: string;
  constructor(private http: HttpClient) { 
    this.apiUrl = `${environment.apiUrl}`;
  }

  searchUserInfo(data: { keyword: string; date_created: string }, page: number, pageSize: number): Observable<CustomHttpResponse<UserManagement[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<UserManagement[]>>(environment.apiUrl + '/api/user/list', data, options);
  }

  deleteUser(user_id: number): Observable<CustomHttpResponse<any>> {
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/user/remove/${user_id}`);
  }

  restoreUser(user_id: number): Observable<CustomHttpResponse<any>> {
    return this.http.get<CustomHttpResponse<any>>(environment.apiUrl + `/api/user/restore/${user_id}`);
  }

  getUserRoleList(): Observable<CustomHttpResponse<Role>>{
    return this.http.get<CustomHttpResponse<Role>>(environment.apiUrl + '/api/common/getRolesList');
  }

  getUserPermission(user_id:number): Observable<CustomHttpResponse<UserManagement>>{
    return this.http.get<CustomHttpResponse<UserManagement>>(environment.apiUrl + '/api/user/view_roles/'+user_id);
  }

  createUserAPI(data:any): Observable<CustomHttpResponse<UserManagement>>{
    return this.http.post<CustomHttpResponse<UserManagement>>(environment.apiUrl + '/api/user/create',data);
  }

  getUserDetail(user_id: number): Observable<CustomHttpResponse<UserManagement>> {
    return this.http.get<CustomHttpResponse<UserManagement>>(environment.apiUrl + `/api/user/view/`+user_id);
  }

  modifyUser(data: any, user_id: any): Observable<CustomHttpResponse<UserManagement>> {
    return this.http.post<CustomHttpResponse<UserManagement>>(environment.apiUrl + `/api/user/modify/${user_id}`, data);
  }
}
