import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { CustomHttpResponse } from '../_models';
import { UserPermission, UserRole } from '../_models/user-role';
@Injectable({
  providedIn: 'root'
})
export class UserRolesService {

  apiUrl: string;
  
  constructor(private http: HttpClient) { 
    this.apiUrl = `${environment.apiUrl}`;
  }

  searchUserRoleInfo(data: { keyword: any; date_created: any }, page: number, pageSize: number): Observable<CustomHttpResponse<UserRole[]>> {
    const headers = new HttpHeaders();
    headers.append('Content-Type', 'application/json');

    const params = {
      page,
      page_size: pageSize
    };

    const options = {
      headers: headers,
      params: params
    };
    return this.http.post<CustomHttpResponse<UserRole[]>>(environment.apiUrl + '/api/user/roles_list', data, options);
  }

  getUserPermission(): Observable<CustomHttpResponse<UserPermission[]>>{
    return this.http.get<CustomHttpResponse<UserPermission[]>>(environment.apiUrl + '/api/user/getMenusList');
  }

  createUserRoleAPI(data:any): Observable<CustomHttpResponse<UserRole[]>>{
    return this.http.post<CustomHttpResponse<UserRole[]>>(environment.apiUrl + '/api/user/create_roles', data);
  }

  deleteUserRole(role_id: number): Observable<CustomHttpResponse<any>> {
    return this.http.delete<CustomHttpResponse<any>>(environment.apiUrl + `/api/user/remove_roles/${role_id}`);
  }

  viewRoleDetail(role_id:number): Observable<CustomHttpResponse<UserRole>>{
    return this.http.get<CustomHttpResponse<UserRole>>(environment.apiUrl + `/api/user/view_roles/${role_id}`);
  }

  modifyUserRoleAPI(data:any,role_id:number): Observable<CustomHttpResponse<UserPermission>>{
    return this.http.post<CustomHttpResponse<UserPermission>>(environment.apiUrl + `/api/user/modify_roles/${role_id}`,data);
  }

}
