import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { AuthGuard } from './_guards/auth.guard';
import { OptionsComponent } from './pages/options/options.component';
import {AuthenticatedGuard} from "./_guards/authenticated.guard";

const routes: Routes = [
  { path: '', redirectTo: 'login', pathMatch: 'full' },
  { path: 'login', loadChildren: () => import('./pages/login/login.module').then(m => m.LoginModule), canActivate: [AuthenticatedGuard]},
  { path: 'dashboard', loadChildren: () => import('./pages/dashboard/dashboard.module').then(m => m.DashboardModule),canActivate: [AuthGuard] },
  { path: 'contact', loadChildren: () => import('./pages/contact/contact.module').then(m => m.ContactModule), canActivate: [AuthGuard]},
  { path: 'article', loadChildren: () => import('./pages/article/article.module').then(m => m.ArticleModule), canActivate: [AuthGuard]},
  { path: 'options/storage-location', loadChildren: () => import('./pages/storage-location/storage-location.module').then(m => m.StorageLocationModule), canActivate: [AuthGuard]},
  { path: 'inventory', loadChildren: () => import('./pages/inventory/inventory.module').then(m => m.InventoryModule), canActivate: [AuthGuard]},
  { path: 'options/ligne', loadChildren: () => import('./pages/ligne/ligne.module').then(m => m.LigneModule), canActivate: [AuthGuard]},
  { path: 'transfer-de-stock', loadChildren: () => import('./pages/transfer-de-stock/transfer-de-stock.module').then(m => m.TransferDeStockModule), canActivate: [AuthGuard]},
  { path: 'options/fabriquant', loadChildren: () => import('./pages/fabriquant/fabriquant.module').then(m => m.FabriquantModule), canActivate: [AuthGuard]},
  { path: 'inventory-tournant', loadChildren: () => import('./pages/inventory-tournant/inventory-tournant.module').then(m => m.InventoryTournantModule), canActivate: [AuthGuard]},
  { path: 'bon-de-livraison', loadChildren: () => import('./pages/bon-de-livraison/bon-de-livraison.module').then(m => m.BonDeLivraisonModule), canActivate: [AuthGuard]},
  { path: 'options/type-de-material', loadChildren: () => import('./pages/type-de-material/type-de-material.module').then(m => m.TypeDeMaterialModule), canActivate: [AuthGuard]},
  { path: 'bon-de-reception', loadChildren: () => import('./pages/bon-de-reception/bon-de-reception.module').then(m => m.BonDeReceptionModule), canActivate: [AuthGuard]},
  { path: 'options/user', loadChildren: () => import('./pages/user-management/user-management.module').then(m => m.UserManagementModule), canActivate: [AuthGuard]},
  { path: 'achat-commande', loadChildren: () => import('./pages/achat-commande/achat-commande.module').then(m => m.AchatCommandeModule), canActivate: [AuthGuard]},
  { path: 'options/roles', loadChildren: () => import('./pages/user-role/user-role.module').then(m => m.UserRoleModule), canActivate: [AuthGuard]},
  { path: 'options/service-expedition', loadChildren: () => import('./pages/service-expedition/service-expedition.module').then(m => m.ServiceExpeditionModule), canActivate: [AuthGuard]},
  { path: 'options/depot-tnt', loadChildren: () => import('./pages/depot-tnt/depot-tnt.module').then(m => m.DepotTntModule), canActivate: [AuthGuard]},
  { path: 'reporting', loadChildren: () => import('./pages/reporting/reporting.module').then(m => m.ReportingModule), canActivate: [AuthGuard]},
  { path: 'reporting-planification', loadChildren: () => import('./pages/reporting-planification/reporting-planification.module').then(m => m.ReportingPlanificationModule), canActivate: [AuthGuard]},
  { path: 'quick-search', loadChildren: () => import('./pages/quick-search/quick-search.module').then(m => m.QuickSearchModule), canActivate: [AuthGuard]},
  { path: 'livraison-prestation', loadChildren: () => import('./pages/livraison-prestation/livraison-prestation.module').then(m => m.LivraisonPrestationModule), canActivate: [AuthGuard]},
  { path: 'settings', loadChildren: () => import('./pages/settings/settings.module').then(m => m.SettingsModule), canActivate: [AuthGuard]},
  { path: 'article-category', loadChildren: () => import('./pages/article-category/article-category.module').then(m => m.ArticleCategoryModule), canActivate: [AuthGuard]},
  { path: 'attendue-de-reception', loadChildren: () => import('./pages/attendue-de-reception/attendue-de-reception.module').then(m => m.AttendueDeReceptionModule), canActivate: [AuthGuard]},
  { path: 'options',component: OptionsComponent}
];

@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule { }
