import { Component, ViewChild } from '@angular/core';
import { TokenStorageService } from './_services/token-storage.service';
import {Router, ActivatedRoute, NavigationStart, NavigationEnd} from '@angular/router';
import { MatSidenav } from '@angular/material/sidenav';
import { TranslateService } from '@ngx-translate/core';
import {APP_CONSTANTS} from "./_helpers/constants";
import {DataSharingService} from "./_services/data-sharing.service";
declare var $:any;
@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent {

  @ViewChild('sidenav') sidenav!: MatSidenav;

  title = 'capgestiondestocks';
  isLoggedIn: boolean = false;

  previousRoute: string | undefined;
  currentRoute: string | undefined;

  constructor(private tokenStorageService: TokenStorageService,
              private router: Router,
              private translateService: TranslateService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    if(this.tokenStorageService.getToken()) {
      this.isLoggedIn = true;
      this.dataSharingService.isUserLoggedIn.next(this.isLoggedIn);
    } else {
      this.isLoggedIn = false;
      this.dataSharingService.isUserLoggedIn.next(this.isLoggedIn);
    }

    const lang = this.translateService.getBrowserLang();
    if (lang != null) {
      this.translateService.setDefaultLang(lang);
    } else {
      this.translateService.setDefaultLang(APP_CONSTANTS.DEFAULT_LANG);
    }

    //Router subscriber
    this.router.events.subscribe((event: any) => {
      if (event instanceof NavigationEnd) {
        //do something on end activity
        this.previousRoute = this.currentRoute;
        this.currentRoute = event.url;

        if(!this.currentRoute.includes('/contact')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/storage-location')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATIONT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/fabriquant')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/ligne')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/type-de-material')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/depot-tnt')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/roles')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ROLE_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ROLE_LIST_FORM);
        }
        if(!this.currentRoute.includes('/options/service-expedition')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_SERVICE_EXPEDITION_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_SERVICE_EXPEDITION_LIST_FORM);
        }
        if(!this.currentRoute.includes('/user')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_USER_MANAGEMENT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/reporting-planification')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_REPORTING_PLANIFICATION_LIST_FORM);
        }
        if(!this.currentRoute.includes('/quick-search')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_QUICK_SEARCH_LIST_FORM);
        }
        if(!this.currentRoute.includes('/article')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST_FORM);
        }
        if(!this.currentRoute.includes('/bon-de-livraison')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST_FORM);
        }
        if(!this.currentRoute.includes('/bon-de-reception')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_RECEPTION_LIST_FORM);
        }
        if(!this.currentRoute.includes('/transfer-de-stock')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_TRANSFERT_DE_STOCK_LIST_FORM);
        }
        if(!this.currentRoute.includes('/inventory')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_LIST_FORM);
        }
        if(!this.currentRoute.includes('/inventory-tournant')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_INVENTAIRE_TOURNANT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/achat-commande')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST_FORM);
        }
        if(!this.currentRoute.includes('/attendue-de-reception')) {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST);
          this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ATTENDUE_LIST_FORM);
        }
      }
    });
  }

  logout(): void {
    this.isLoggedIn = false;
    this.dataSharingService.isUserLoggedIn.next(this.isLoggedIn);
    this.tokenStorageService.signOut();
  }



}
