import { NgModule, CUSTOM_ELEMENTS_SCHEMA  } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { HttpClient, HttpClientModule, HttpHeaders } from '@angular/common/http';

import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { MatCardModule } from '@angular/material/card';
import { MatDividerModule } from '@angular/material/divider';
import {MAT_FORM_FIELD_DEFAULT_OPTIONS, MatFormFieldModule} from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatSelectModule } from '@angular/material/select';
import { MatInputModule } from '@angular/material/input';
import { FontAwesomeModule } from '@fortawesome/angular-fontawesome';
import { HeaderComponent } from './header/header.component';
import {MatToolbarModule} from '@angular/material/toolbar';
import {MatSidenavModule} from '@angular/material/sidenav';
import {MatButtonModule} from '@angular/material/button';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { authInterceptorProviders } from './_helpers/auth.interceptor';
import { NgxMatIntlTelInputModule } from 'ngx-mat-intl-tel-input';
import { MatTableModule } from '@angular/material/table';
import { MatPaginatorModule } from '@angular/material/paginator';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatSortModule } from '@angular/material/sort';
import { MatDialogModule } from '@angular/material/dialog';
import {TranslateLoader, TranslateModule} from '@ngx-translate/core';
import { Observable } from 'rxjs';
import { ConfirmComponent } from './dialogs/confirm/confirm.component';
import { environment } from '../environments/environment';
import { CommonModule } from '@angular/common';
import {MatDatepickerModule} from '@angular/material/datepicker';
import { MatNativeDateModule } from '@angular/material/core';
import { DatePipe } from '@angular/common';
import {MatCheckboxModule} from "@angular/material/checkbox";
import { PdfViewerComponent } from './dialogs/pdf-viewer/pdf-viewer.component';
import {NgxExtendedPdfViewerModule} from "ngx-extended-pdf-viewer";
import {NgxSpinnerModule} from "ngx-spinner";
import { CustomAlertComponent } from './dialogs/custom-alert/custom-alert.component';
import { OptionsComponent } from './pages/options/options.component';
import {ComponentsModule} from "./_components/components.module";
import {SideNavMenuModule} from "./side-nav-menu/module";

export class CustomLoader implements TranslateLoader {
  apiUrl: string;

  constructor(private http: HttpClient) {
    this.apiUrl = `${environment.apiUrl}`;
  }

  public getTranslation(lang: string): Observable<any> {
      const headers = new HttpHeaders();
      headers.append('Content-Type', 'application/json');
      const params :any = {};

      params['lang'] = lang;

      const options = {
          headers: headers,
          params: params
      }

      return this.http.get(this.apiUrl+'/api/translation/get_list_translation', options);
  }
}

@NgModule({
    declarations: [
        AppComponent,
        HeaderComponent,
        ConfirmComponent,
        PdfViewerComponent,
        CustomAlertComponent,
        OptionsComponent
    ],
    imports: [
        BrowserModule,
        AppRoutingModule,
        BrowserAnimationsModule,
        MatCardModule,
        MatDividerModule,
        MatFormFieldModule,
        MatIconModule,
        MatSelectModule,
        MatInputModule,
        FontAwesomeModule,
        MatToolbarModule,
        MatSidenavModule,
        MatButtonModule,
        HttpClientModule,
        MatAutocompleteModule,
        NgxMatIntlTelInputModule,
        MatTableModule,
        MatPaginatorModule,
        MatProgressBarModule,
        MatProgressSpinnerModule,
        MatSortModule,
        MatDialogModule,
        MatCheckboxModule,
        TranslateModule.forRoot({
            loader: {
                provide: TranslateLoader,
                useClass: CustomLoader,
                deps: [HttpClient]
            },
            isolate: false
        }),
        MatDatepickerModule,
        MatNativeDateModule,
        NgxExtendedPdfViewerModule,
        NgxSpinnerModule,
        SideNavMenuModule
    ],
    providers: [
      authInterceptorProviders,
      MatDatepickerModule,
      DatePipe,
      { provide: MAT_FORM_FIELD_DEFAULT_OPTIONS, useValue: { floatLabel: 'always' } }
    ],
    schemas: [
        CUSTOM_ELEMENTS_SCHEMA
    ],
    bootstrap: [AppComponent, HeaderComponent]
})
export class AppModule { }
