import {Component, Inject, OnInit} from '@angular/core';
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {TranslateService} from "@ngx-translate/core";

export interface CustomAlertData {
  title: string;
  message: string | any;
  okText?: string;
  cancelText?: string;

  translated?: boolean;
  titleTranslateParam?: any;
  messageTranslateParam?: any;
}

@Component({
  selector: 'app-custom-alert',
  templateUrl: './custom-alert.component.html',
  styleUrls: ['./custom-alert.component.scss']
})
export class CustomAlertComponent implements OnInit {

  title: string = "";
  message: string = "";

  okButtonText: string = "OK";
  cancelButtonText: string = "Cancel";

  translated: boolean = false;
  titleTranslateParam: any;
  messageTranslateParam: any;

  constructor(public dialogRef: MatDialogRef<CustomAlertComponent>,
              @Inject(MAT_DIALOG_DATA) public data: CustomAlertData,
              private translateService: TranslateService) { }

  ngOnInit(): void {
    this.translated = this.data.translated || false;

    this.titleTranslateParam = this.data.titleTranslateParam || {};
    this.messageTranslateParam = this.data.messageTranslateParam || {};

    this.title = this.translated ? this.data.title : this.translateService.instant(this.data.title, this.titleTranslateParam);
    this.message = this.translated ? this.data.message : this.translateService.instant(this.data.message, this.messageTranslateParam);

    this.okButtonText = this.data.okText || "OK";
    this.cancelButtonText = this.data.cancelText || "Cancel";
  }

}
