import { Component, OnInit, ViewChild } from '@angular/core';
import { TokenStorageService } from '../_services/token-storage.service';
import { Router, ActivatedRoute } from '@angular/router';
import { DataSharingService } from '../_services/data-sharing.service';
import { MatSidenav } from '@angular/material/sidenav';
import {APP_CONSTANTS} from "../_helpers/constants";
import {DeviceDetectorService} from "ngx-device-detector";

@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.scss']
})
export class HeaderComponent implements OnInit {
  @ViewChild('sidenav') sidenav!: MatSidenav;

  isLoggedIn:boolean = false;
  username :any;
  isExpanded = true;
  showSubmenu: boolean = false;
  isShowing = false;
  showSubSubMenu: boolean = false;

  rolePermission:any;

  isMobile = false;

  constructor(private tokenStorageService: TokenStorageService,
              private router: Router,
              private dataSharingService: DataSharingService,
              private deviceDetectorService: DeviceDetectorService) {
  }

  ngOnInit(): void {
    this.isMobile = this.deviceDetectorService.isMobile();

    this.dataSharingService.isUserLoggedIn.subscribe((isLoggedIn: boolean) => {
      this.isLoggedIn = isLoggedIn;

      if(this.isLoggedIn) {
        this.username =  this.tokenStorageService.getUser().user_name;

        const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
        if(user_role_permission != null) {
          this.rolePermission = JSON.parse(user_role_permission);
          console.log(this.rolePermission);
        }
      }
    })
  }

  close(reason: string) {
    this.sidenav.close();
  }

  goBack(){

  }

  Disconnect() {
    this.isLoggedIn = false;
    this.dataSharingService.isUserLoggedIn.next(this.isLoggedIn);
    this.tokenStorageService.signOut();

    this.router.navigate(['/']);
    this.sidenav.close();
  }

}
