import {Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {
  AchatCommandeList,
  BonAchatCommande,
  BonAchatCommandeItem,
  BrandInfoList, BrandPrixVente, CustomHttpResponse,
  LigneInfoList,
  StoreInfoList
} from "../../../_models";
import {Article, SearchArticle} from "../../../_models/article";
import {MatDialog} from "@angular/material/dialog";
import {CommonService} from "../../../_services/common.service";
import {AuthService} from "../../../_services/auth.service";
import {AchatCommandeService} from "../../../_services/achat-commande.service";
import {DatePipe} from "@angular/common";
import {ActivatedRoute, Router} from "@angular/router";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {TranslateService} from "@ngx-translate/core";
import * as moment from "moment";
import {
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {firstValueFrom, lastValueFrom, Observable} from "rxjs";
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {ModifyItemComponent} from "../../bon-de-reception/modify-item/modify-item.component";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { AttendueArticle, AttendueDetail } from 'src/app/_models/attendue-de-reception';
import { AttendueDeReceptionService } from 'src/app/_services/attendue-de-reception.service';
import {SearchArticleComponent} from "../../article/search-article/search-article.component";

@Component({
  selector: 'app-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.scss']
})
export class EditComponent implements OnInit {

  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;
  @ViewChild('rechercheRefInput') rechercheRefInputEl: ElementRef | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],

    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    SEARCH_REF: [""]
  })

  bonAchatCommandeItemForm: FormGroup = this.formBuilder.group({
    ITEM_SUPPLIER_BRAND: ["", Validators.required],
    ITEM_CLIENT_BRAND: [""],
    ITEM_CLIENT_STORE: [""],

    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    LABEL: [{ value: "", disabled: true }, Validators.required],
    QTE: ["0", [Validators.required, Validators.min(1)]],
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    PRIX_DE_VENTE: ['', [ValidateGreaterThan, Validators.minLength(APP_CONSTANTS.PRIX_DE_VENTE_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.PRIX_DE_VENTE_MAX_LENGTH)]],
    TOTAL_PRIX_DE_VENTE: [{ value: 0, disabled: true }, [Validators.required, ValidateGreaterThan]],
    MATERIAL_ACTIVABLE_STATUS: [null],
    ITEM_STATUS: [APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING, Validators.required],
    FRAISDEPORT: []
  });

  newArticleSelection: Article | undefined;

  bonDocumentNum: string | undefined = "*********";
  bonAchatCommande: BonAchatCommande | undefined;
  bonAchatCommandeItems: BonAchatCommandeItem[] = [];
  bonAchatCommandeArticles: Article[] = [];

  defaultPrixDeVente:number = 0;
  totalArticles: number = 0;
  totalQte: number = 0;
  totalPT: number = 0;
  prixdevente: number = 0;
  totalprixdevente: number = 0;
  shippingPrice:number = 0;
  brandPrixVente: BrandPrixVente | undefined;
  ligneList: LigneInfoList[] = [];
  clientBrandList: BrandInfoList[] = [];
  supplierBrandList: BrandInfoList[] = [];
  storeList: StoreInfoList[] = [];

  fullKeywordMatch: boolean = false;
  isKeywordSerial: boolean = false;
  searchRefArticlesList: Article[] | undefined = [];

  displayedColumns: string[] = [ 'ITEM_BRAND', 'REFERENCE', 'LABEL', 'QTE', 'PRIX_UNIT', 'PRIX_TOTAL', 'PRIX_DE_VENTE', 'TOTAL_PRIX_DE_VENTE', 'SHIPPING_PRICE', 'ITEM_CUSTOMER', 'ITEM_STATUS', 'ACTIVE_STATUS', 'ACTIONS'];

  customLigneInput: boolean = false;

  // Used to decide whether the values can be editable or not
  readOnly: boolean = false;
  pageTitle: string = "KMODIFY_ACHAT";

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  rolePermission:any;

  constructor(public dialog: MatDialog,
              private commonService: CommonService,
              private authService: AuthService,
              private achatCommandeService: AchatCommandeService,
              private attendueDeReceptionService: AttendueDeReceptionService,
              private router: Router,
              private route: ActivatedRoute,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    this.loadDependencies();
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res === 'custom-ligne') {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
          this.customLigneInput = true;
        } else {
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
          this.customLigneInput = false;
        }
      }
    );

    this.bonDocumentForm.get("SEARCH_REF")?.valueChanges.subscribe(
      (value) => {
        if(value === "" || value === null) {
          this.searchRefArticlesList = undefined;
        }
      }
    );
  }

  async loadDependencies() {
    await this.loadingService.show();

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });

    firstValueFrom(this.route.data).then(
      (data: any) => {
        if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
          this.readOnly = data.readOnly;
          this.__changeToDetailMode();
        }
      }
    );
    await this._getLigneList();
    await this._getBrandList();

    firstValueFrom(this.route.params).then(
      (params) => {
        if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
          this.bonDocumentNum = params["document_no"];

          this._getBonAchatCommandeDetail(this.bonDocumentNum);
        } else {
          // If the Achat/Commande document not available
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KACHAT_INVALID_DOCUMENT'
          }).then(
            () => {
              this.router.navigate(['/achat-commande/list']);
            }
          );
        }
      }
    )

    await this.loadingService.hide();
  }

  get material_activable_status() {
    return this.newArticleSelection && this.newArticleSelection?.material_activable_status;
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST_FORM);
  }

  __changeToDetailMode() {
    this.pageTitle = "KDETAIL_ACHAT_COMMANDE";

    this.bonDocumentForm.get('LIGNE_INFO')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('LIGNE_CUSTOM_LABEL')?.disable({ onlySelf: true });
  }

  achatItemStatus(status: number): string {
    const statusList = APP_CONSTANTS.ACHAT_ITEM_STATUS;

    const statusData = statusList.find((r) => r.value == status);
    let result = "";
    switch (statusData?.value) {
      case APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING:
        result = `<span class='badge bg-secondary'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_ITEM_STATUS_INPROGRESS:
        result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_ITEM_STATUS_VALIDATED:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.ACHAT_ITEM_STATUS_CANCELLED:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  async onCloseDocument() {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    this.returnBack();
  }

  onBrandChange() {
    const brand_id = this.bonAchatCommandeItemForm.get('ITEM_CLIENT_BRAND')?.value;
    this.bonAchatCommandeItemForm.get('ITEM_CLIENT_STORE')?.setValue('');

    if(brand_id === undefined || brand_id === null || brand_id === '') {
      this.storeList = [];
      return;
    }

    const payload = {
      type: 'client',
      brand_id
    };

    this._getStoreList(payload);
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
    if(listCache !== undefined) {
      let listAchatCommande = listCache.data as AchatCommandeList[];
      listAchatCommande = listAchatCommande.map((row) => {
        if(row.document_no == payload.document_no) {
          row.achatcommande_id = payload.achatcommande_id;
          row.created_at = payload.created_at;
          row.created_userid = payload.created_userid;
          row.created_username = payload.created_username;
          row.dated = payload.dated;
          row.document_no = payload.document_no;
          row.ligne_id = payload.ligne_id;
          row.ligne_lbl = payload.ligne_lbl;
          row.modified_at = payload.modified_at;
          row.modified_userid = payload.modified_userid;
          row.modified_username = payload.modified_username;
          row.reception_document_no = payload.reception_document_no;
          row.reception_id = payload.reception_id;
          row.status = payload.status;
          row.store_id = payload.customer?.store_id;
          row.store_name = payload.customer?.store_nom;
          row.store_type = payload.customer?.store_type;
        }
        return row;
      });

      listCache.data = listAchatCommande;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST, JSON.stringify(listCache));
    }
  }

  __deleteUpdateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ACHAT_LIST);
    if(listCache !== undefined) {
      let listAchatCommande: any[] = listCache.data;
      listAchatCommande = (listAchatCommande as AchatCommandeList[]).filter(r => r.document_no != id);

      listCache.data = listAchatCommande;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ACHAT_LIST, JSON.stringify(listCache));

      this.__updateListCache(listCache.data);
    }
}


  async onSubmitDocument(type: 'validate' | 'invoiced' = 'validate') {
    if(this.readOnly) {
      return;
    }

    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(type === 'validate'){
      if(!this.rolePermission['1_5']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.isBonAchatCommandeModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACHAT_NO_ARTICLE_ADDED'
      })

      return;
    }

    const documentFormData = this.bonDocumentForm.getRawValue();
    const bonArticles: BonAchatCommandeItem[] = [...this.bonAchatCommandeItems];
    const bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    const bonStatus = type === 'invoiced' ? APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_INVOICED : (this.bonAchatCommande?.status ? this.bonAchatCommande?.status : APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_PENDING);
    const ligneInfo = documentFormData.LIGNE_INFO;
    const bonAttendueArticles: AttendueArticle[] = [];
    bonArticles.forEach(article =>{
      const article_detail = {
        achat_item_id : article.item_id,
        supplier_brand_id: article.supplier_brand_id,
        client_brand_id: article.client_brand_id,
        client_store_id : article.client_store_id,
        article_id: article.article_id,
        article_reference: article.article_reference,
        article_desc: article.article_desc,
        qte: article.qte,
        qte_received: article.qte_received,
        unit_price: article.unit_price,
        total_price: article.total_price,
        material_activable_status: article.material_activable_status,
        item_status: article.item_status
      }
      bonAttendueArticles.push(article_detail);
    })
    const bonDocument: AttendueDetail = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      articles: bonAttendueArticles,
      total_price: bonTotalPrix,
      status: bonStatus,
      linked_document_no: documentFormData.NUMBON,
      linked_document_type: APP_CONSTANTS.DOCUMENT_TYPE_ACHAT,
      achat_commande_id: this.bonAchatCommande?.achatcommande_id
    };
    this._createAttendueDeReception(bonDocument);
  }

  _createAttendueDeReception(bonDocument: AttendueDetail): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.attendueDeReceptionService.createAttendueDeReception(bonDocument).subscribe( {
        next: (res: CustomHttpResponse<AttendueDetail>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            this.__resetListCache();
            this.returnBack();
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  generateAchatCommandePdf(print?: boolean) {
    if(!this.isBonAchatCommandeModifiable || (this.bonDocumentNum === undefined)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACHAT_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.achatCommandeService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      } else {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_ACHAT_COMMANDE_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack() {
    this.router.navigate(['/achat-commande/list']);
  }

  checkPrixdeventePrice(event: any): Promise<any>{
    const brand_id = event.value;
    const article_id = this.bonAchatCommandeItemForm.get("ARTICLE_ID")?.value;
    let payload = {
      brand_id : brand_id,
      article_id: article_id
    }
    return new Promise((resolve, reject) => {
      this.commonService.getPrixDeVenteBasedOnBrand(payload).subscribe({
        next: (res: CustomHttpResponse<BrandPrixVente>) => {
          if(res.status === 'success') {
            this.brandPrixVente = res.data;
            this.bonAchatCommandeItemForm.get("PRIX_DE_VENTE")?.setValue(this.brandPrixVente.prix_de_vente);
            this.calculatePrixTotal();
          }else {
            this.bonAchatCommandeItemForm.get("PRIX_DE_VENTE")?.setValue(this.defaultPrixDeVente.toString());
            this.calculatePrixTotal();
          }
          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            const brandList = res.data;

            // Filter only Client Brands
            this.clientBrandList = brandList.filter((row) => row.BRTYPE === 'client');

            // Filter only Supplier Brands
            this.supplierBrandList = brandList.filter((row) => row.BRTYPE === 'supplier');
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _getStoreList(payload: { type: string, brand_id: number }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StoreInfoList[]>) => {
          if(res.status === 'success') {
            this.storeList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStores(payload).subscribe(observer);
    });
  }

  _searchArticles(data: { keyword: string, location_id: number | null }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      let searchObs: Observable<CustomHttpResponse<SearchArticle>> | undefined;

      searchObs = this.commonService.searchArticlesListByRef(data);

      if(searchObs === undefined) {
        return;
      }

      searchObs.subscribe({
        next: (res: CustomHttpResponse<SearchArticle>) => {
          if(res.status === 'success') {
            this.fullKeywordMatch = res.data.fullMatch || false;
            this.isKeywordSerial = res.data.isSerial || false;
            this.searchRefArticlesList = res.data.rows;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  modifyInvoicedStatus(){
    if(this.bonAchatCommande){
      this.bonAchatCommande.status = APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_INVOICED;
      this._modifyAchatCommande(this.bonDocumentNum, this.bonAchatCommande).then(
        ()=>{
          this.router.navigate(['/achat-commande/list']);
        }
      )
    }
  }

  _getBonAchatCommandeDetail(document_no: string | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.achatCommandeService.getAchatCommandeDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonAchatCommande>) => {
          if(res.status === "success") {
            const bonAchatCommandeData = res.data;
            this.bonAchatCommande = bonAchatCommandeData;
            if(!this.rolePermission['1_6'] && this.bonAchatCommande.status === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_VALIDATED) {
              this.dialogService.displayAlert({
                title: this.translateService.instant('KERROR'),
                message: this.translateService.instant('KACCESS_DENIED')
              }).then(() => {
                this.returnBack();
              });
              return;
            }
            this.__refreshBonDetails(bonAchatCommandeData);
          } else {
            // If the bon achat commande not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/achat-commande/list']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __refreshBonDetails(bonAchatCommande: BonAchatCommande) {
    if(bonAchatCommande !== undefined && bonAchatCommande !== null) {
      this.bonDocumentNum = bonAchatCommande.document_no;
      this.bonAchatCommandeItems = bonAchatCommande.articles;
      this.bonAchatCommandeArticles = bonAchatCommande?.custom_articles ? bonAchatCommande?.custom_articles : [];

      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalPT = 0;
      this.prixdevente = 0;
      this.totalprixdevente = 0;
      this.shippingPrice = 0;

      this.bonAchatCommandeItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        this.totalPT += +row.total_price;
        this.prixdevente += +row.prix_de_vente;
        this.totalprixdevente += +row.total_prix_de_vente;
        this.shippingPrice += +row.shipping_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(bonAchatCommande.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonAchatCommande.dated).format(CUSTOM_DATE_FORMAT));

      if(bonAchatCommande.ligne_lbl !== undefined && bonAchatCommande.ligne_lbl !== null && bonAchatCommande.ligne_lbl !== "") {
        if(bonAchatCommande.ligne_id === undefined || bonAchatCommande.ligne_id === null || bonAchatCommande.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonAchatCommande.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonAchatCommande.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonAchatCommande.ligne_lbl);
        }
      }
    }
  }

  __serialsListDisplay(serialsArr: string[]) {
    return serialsArr.join(", ");
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get isBonAchatCommandeModifiable() {
    return this.bonAchatCommande !== undefined;
  }

  get isDocumentValidated(): boolean {
    return this.bonAchatCommande !== undefined && this.bonAchatCommande !== null && this.bonAchatCommande.status === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_VALIDATED;
  }

  get isAttendueReceptionStatus(): boolean {
    return this.bonAchatCommande !== undefined && this.bonAchatCommande !== null && this.bonAchatCommande.attendue_reception_status === true;
  }

  get isDocumentCancelled(): boolean {
    return this.bonAchatCommande !== undefined && this.bonAchatCommande !== null && this.bonAchatCommande.status === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_CANCELLED;
  }

  get isDocumentInvoiced(): boolean {
    return this.bonAchatCommande !== undefined && this.bonAchatCommande !== null && this.bonAchatCommande.status === APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_INVOICED;
  }

  addBonAchatCommandeItemFormSubmitter() {
    if(this.readOnly) {
      return;
    }

    if(this.bonAchatCommandeItemForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();
    const itemFormData = this.bonAchatCommandeItemForm.getRawValue();

    let bonArticles: BonAchatCommandeItem[] = [];

    if(this.isBonAchatCommandeModifiable) {
      bonArticles = [...this.bonAchatCommandeItems];
    }

    bonArticles.push({
      supplier_brand_id: +itemFormData.ITEM_SUPPLIER_BRAND,
      client_brand_id: +itemFormData.ITEM_CLIENT_BRAND,
      client_store_id: +itemFormData.ITEM_CLIENT_STORE,
      article_id: +itemFormData.ARTICLE_ID,
      article_reference: itemFormData.REFERENCE,
      article_desc: itemFormData.LABEL,
      qte: +itemFormData.QTE,
      unit_price: +itemFormData.PRIX_UNIT,
      total_price: +itemFormData.PRIX_TOTAL,
      prix_de_vente : +itemFormData.PRIX_DE_VENTE,
      total_prix_de_vente: +itemFormData.TOTAL_PRIX_DE_VENTE,
      material_activable_status : itemFormData.MATERIAL_ACTIVABLE_STATUS,
      shipping_price: +itemFormData.FRAISDEPORT,
      item_status: itemFormData.ITEM_STATUS
    })

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    let bonStatus = APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_PENDING;

    if(this.isBonAchatCommandeModifiable) {
      bonStatus = this.bonAchatCommande?.status ? this.bonAchatCommande?.status : APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_PENDING;
    }

    const ligneInfo = documentFormData.LIGNE_INFO;

    const user_detail = this.authService.getCurrentUser();
    const user_id = user_detail ? user_detail.USID : null;

    const bonDocument: BonAchatCommande = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),

      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,
      created_userid: user_id
    };

    console.log('payload: ', bonDocument);

    this._modifyAchatCommande(this.bonDocumentNum, bonDocument).then(
      () => {
        this.__resetNewLineItemForm();
      }
    )
  }

  removeLineItem(currentItem: BonAchatCommandeItem) {
    this.loadingService.show();

    let bonArticles: BonAchatCommandeItem[] = [...this.bonAchatCommandeItems];
    bonArticles = bonArticles.filter((row) => row.item_id !== currentItem.item_id);

    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);

    const bonStatus = this.bonAchatCommande?.status ? this.bonAchatCommande?.status : APP_CONSTANTS.ACHAT_DOCUMENT_STATUS_PENDING;

    const ligneInfo = documentFormData.LIGNE_INFO;

    const user_detail = this.authService.getCurrentUser();
    const user_id = user_detail ? user_detail.USID : null;

    let bonDocument: BonAchatCommande = {
      ...this.bonAchatCommande,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),

      articles: bonArticles,

      total_price: bonTotalPrix,
      status: bonStatus,
      modified_userid: user_id
    };

    delete bonDocument.achatcommande_id;

    this._modifyAchatCommande(this.bonDocumentNum, bonDocument);
  }

  _modifyAchatCommande(document_no: string | undefined, bonDocument: BonAchatCommande ): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.achatCommandeService.modifyAchatCommande(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonAchatCommande>) => {
          this.loadingService.hide();

          if(res.status === "success") {
            const bonAchatCommandeData = res.data;
            this.bonAchatCommande = bonAchatCommandeData;
            this.__updateListCache(bonAchatCommandeData);
            this.__refreshBonDetails(bonAchatCommandeData);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
            return;
          }
          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __resetNewLineItemForm() {
    this.newArticleSelection = undefined;
    this.bonAchatCommandeItemForm.patchValue({
      ITEM_SUPPLIER_BRAND: "",
      ITEM_CLIENT_BRAND: "",
      ITEM_CLIENT_STORE: "",

      ARTICLE_ID: "",
      REFERENCE: "",
      LABEL: "",
      QTE: "0",
      PRIX_UNIT: "",
      PRIX_TOTAL: "0",
      MATERIAL_ACTIVABLE_STATUS: null,
      ITEM_STATUS: APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING,
    })

    this.searchRefArticlesList = undefined;

    this.bonDocumentForm.patchValue({
      SEARCH_REF: ""
    });
  }

  async searchArticles(event: any | undefined): Promise<any> {
    return new Promise(async (resolve, reject) => {
      if(event !== undefined && (event.keyCode == 38 || event.keyCode == 40 || event.keyCode == 13)) {
        return;
      }

      let payload = {
        location_id: null,
        keyword: ''
      };

      const searchRefKeyword = this.bonDocumentForm.get("SEARCH_REF")?.value;
      if(searchRefKeyword === '' || searchRefKeyword === null) return;
      payload = {
        ...payload,
        keyword: searchRefKeyword
      };

      const searchResult = await this._searchArticles(payload);

      resolve(searchResult);
    })
  }

  async onScannerSearchArticle() {
    const searchRef = this.bonDocumentForm.get('SEARCH_REF')?.value;
    if(searchRef == undefined || searchRef == "") {
      return;
    }

    await this.searchArticles(searchRef);

    if(this.searchRefArticlesList === undefined || this.searchRefArticlesList === null) {
      return;
    }

    if(this.searchRefArticlesList.length > 0 && this.fullKeywordMatch) {
      if(this.isKeywordSerial) {
        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      } else {
        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      }
    } else {
      this.openSearchArticlePopup();
    }
  }

  async openSearchArticlePopup() {
    const searchKeyword = this.bonDocumentForm.get('SEARCH_REF')?.value;

    const data = {
      currentArticle: this.newArticleSelection,
      stockLocation: '',
      searchKeyword: searchKeyword
    };

    const dialog = this.dialog.open(SearchArticleComponent, {
      data,
      maxHeight: "90vh"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        this.newArticleSelection = res.data;

        this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);
        console.log(this.bonAchatCommandeItemForm.getRawValue());

        this.__fillNewLineItemForm('new');
        console.log('SEARCH RES: ', res);
      }
    });
  }

  searchSelected(event: any, directCall: boolean = false) {
    if(directCall) {
      this.newArticleSelection = event;
    } else {
      this.newArticleSelection = event.option.value;
    }

    this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);

    this.__fillNewLineItemForm('new');
  }

  __fillNewLineItemForm(type: 'new' | 'existing') {
    let unitPrix = 0;
    if(this.newArticleSelection?.public_prix !== undefined && this.newArticleSelection?.public_prix !== null) {
      unitPrix = this.newArticleSelection?.public_prix;
    }

    if(this.newArticleSelection !== undefined && this.newArticleSelection !== null) {
      this.defaultPrixDeVente = + this.newArticleSelection?.article_prix_de_vente;
    }

    this.bonAchatCommandeItemForm.patchValue({
      ARTICLE_ID: this.newArticleSelection?.id,
      REFERENCE: this.newArticleSelection?.reference,
      LABEL: this.newArticleSelection?.label
    });

    if(type === "new") {
      this.bonAchatCommandeItemForm.patchValue({
        QTE: "0",
        PRIX_UNIT: unitPrix.toString(),
        PRIX_TOTAL: "0",
        ITEM_STATUS: APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING,
        PRIX_DE_VENTE: this.defaultPrixDeVente?.toString(),
        FRAISDEPORT: null,
        ITEM_SUPPLIER_BRAND: "",
        ITEM_CLIENT_BRAND: ""
      });
      this.onBrandChange();
    }
  }

  confirmResetNewArticle(): Promise<any> {
    const message = `<h3>${this.translateService.instant('KCONFIRM_DISCARD_NEW_ITEM')}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async modifyLineItem(item: BonAchatCommandeItem) {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    const data = {
      item: item,
      numeroType: "achat-commande-numero-series",
      document_no: this.bonDocumentNum,
      document: this.bonAchatCommande,
    };

    this.openUpModifyItemPopup(data);
  }

  openUpModifyItemPopup(data?: {}) {
    const dialogRef = this.dialog.open(ModifyItemComponent,
      {
        maxWidth: '45vw',
        data: data || {}
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          this.loadingService.show();
          const bonDocument = result.data;
          this._modifyAchatCommande(this.bonDocumentNum, bonDocument);
        }
      }
    });
  }

  calculatePrixTotal(){
    const formData = this.bonAchatCommandeItemForm.getRawValue();

    let totalPrix = 0;
    let totalprixdevente = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.bonAchatCommandeItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.bonAchatCommandeItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_DE_VENTE = formData.PRIX_DE_VENTE;
    if(PRIX_DE_VENTE === null || PRIX_DE_VENTE === '' || PRIX_DE_VENTE === 0) {
      this.bonAchatCommandeItemForm.get("TOTAL_PRIX_DE_VENTE")?.setValue(totalprixdevente.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    totalprixdevente = parseInt(QTE) * parseFloat(PRIX_DE_VENTE);
    this.bonAchatCommandeItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
    this.bonAchatCommandeItemForm.get("TOTAL_PRIX_DE_VENTE")?.setValue(totalprixdevente.toString());
    }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_ACHAT
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }

  confirmRemoveAchatCommande(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_CANCEL_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeAchatCommande() {
    if(this.bonDocumentNum == undefined){
      return
    }

    if((this.isDocumentValidated && !this.rolePermission['1_7']) || (!this.isDocumentValidated && !this.rolePermission['1_4'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }

    const removeConfirmation = await this.confirmRemoveAchatCommande(this.bonDocumentNum);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteAchatCommande(this.bonDocumentNum);
    }
  }

  async _deleteAchatCommande(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          const bonAchatCommandeData = res.data;
          this.bonAchatCommande = bonAchatCommandeData;
          this.__refreshBonDetails(bonAchatCommandeData);
          const split_doc_no = res.data.splitted_document_no;
          if(split_doc_no){
            await this.dialogService.displayAlert({
              title: 'KSUCCESS',
              message: this.translateService.instant('KACHAT_CANCELED_SEPARATED_SUCCESS', { DOCUMENT_NO: split_doc_no })
            })
          }else { 
            await this.dialogService.displayAlert({
              title: 'KSUCCESS',
              message: this.translateService.instant('KACHAT_CANCELED_SUCCESS')
            })
          }
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.achatCommandeService.splittingAchatCommande(document_no).subscribe(observer);
  }

}
