import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormControl, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { ArticleCategoryInfo, CustomHttpResponse } from 'src/app/_models';
import { ArticleCategoryService } from 'src/app/_services/article-category.service';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-add-article-category',
  templateUrl: './add-article-category.component.html',
  styleUrls: ['./add-article-category.component.scss']
})
export class AddArticleCategoryComponent implements OnInit {


  submitted: boolean = false;
  createCategoryForm: FormGroup = this.formBuilder.group({
    category_status: new FormControl('', [Validators.required]),
    category_label: new FormControl('', [Validators.required, Validators.maxLength(APP_CONSTANTS.ARTICLE_CATEGORY_LABEL_MAX_LENGTH), Validators.minLength(APP_CONSTANTS.ARTICLE_CATEGORY_LABEL_MIN_LENGTH)]),
  });

  statusList = APP_CONSTANTS.ARTICLE_CATEGORY_STATUS;

  constructor(private formBuilder: FormBuilder,
              private router: Router,
              private articleCategoryService: ArticleCategoryService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService:DataSharingService) {
  }

  ngOnInit(): void {
  }

  get fr() {
    return this.createCategoryForm.controls;
  }

  createCategorySubmitter() {
    this.submitted = true;
    if(this.createCategoryForm.invalid) {
      return;
    }

    const formData = this.createCategoryForm.getRawValue();
    const payload = {
      category_label: formData.category_label,
      category_status: formData.category_status
    };

    this._addCategoryInfo(payload);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_ARTICLE_CATEGORY_LIST_FORM);
  }

  async _addCategoryInfo(payload: { category_label: string, category_status: number }) {
    const observer = {
      next: async (res: CustomHttpResponse<ArticleCategoryInfo>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/article-category/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.articleCategoryService.createCategory(payload).subscribe(observer);
  }


}
