import { Component, OnInit } from '@angular/core';
import {DatePipe, Location} from '@angular/common';
import { AuthService } from '../../../_services/auth.service';
import { FormArray, FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {BrandInfoList, CustomHttpResponse, Manufacturers} from "../../../_models";
import {Article, ArticleCategories, ArticleMaterialTypes, ArticleVat} from "../../../_models/article";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {ArticleService} from "../../../_services/article.service";
import * as moment from "moment";
import {CUSTOM_DATE_API_FORMAT} from "../../../_helpers/custom-date-formats";
import {CommonService} from "../../../_services/common.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {FusionerArticleComponent} from "../fusioner-article/fusioner-article.component";
import {MatDialog} from "@angular/material/dialog";
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { firstValueFrom } from 'rxjs';

@Component({
  selector: 'app-edit-article',
  templateUrl: './edit-article.component.html',
  styleUrls: ['./edit-article.component.scss']
})
export class EditArticleComponent implements OnInit {

  submitted: boolean = false;
  modifyArticleForm: FormGroup = this.formBuilder.group({
    ARTCATID: ['', [Validators.required]],
    ARTMATID: ['', [Validators.required]],
    ARTLBL: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_LABEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_LABEL_MAX_LENGTH)]],
    ARTDESC: ['', [Validators.minLength(APP_CONSTANTS.ARTICLE_DESC_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_DESC_MAX_LENGTH)]],
    ARTDTEDEBDISP: [{ value: new Date(), disabled: true }],
    ARTDTEOBSL: [{ value: new Date(), disabled: true }],
    ARTDERNIERDTACHAT: [{ value: '', disabled: true }],
    ARTTRACE: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_TRACE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_TRACE_MAX_LENGTH)]],
    ARTDURGUART: [''],
    ARTPOID: ['', [Validators.minLength(APP_CONSTANTS.ARTICLE_POID_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_POID_MAX_LENGTH)]],
    ARTDIM1: [''],
    ARTDIM2: [''],
    ARTDIM3: [''],
    ARTMNFID: ['', [Validators.required]],
    ARTCONSREF: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_CONSREF_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_CONSREF_MAX_LENGTH)]],
    ARTREFFAC: ['', [Validators.maxLength(APP_CONSTANTS.ARTICLE_REFERENCE_FACTURE_MAX_LENGTH)]],
    ARTREF: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.ARTICLE_REF_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTICLE_REF_MAX_LENGTH)]],
    ARTDELLIV: [''],
    ARTTRESHLD: [''],
    ARTCMDQTE: [''],
    ARTASSUR: [''],
    ARTPUBPRIXHT: [''],
    ARTPURPRIXHT: [''],
    ARTPRIXVENTE: ['' , [Validators.required, Validators.pattern(APP_CONSTANTS.MIN_NUM_PATTERN),Validators.minLength(APP_CONSTANTS.ARTPRIXVENTE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTPRIXVENTE_MAX_LENGTH)]],
    ARTSUPPLIER: [''],
    ARTAPR1RECP: [''],
    ARTVATID: [''],
    ARTIMG: [''],
    ARTBRAND: new FormArray([])
  });

  id: any;
  getData: any=[];
  label: any;

  categoriesList: ArticleCategories[] = [];
  materialtypesList: ArticleMaterialTypes[] = [];
  manufacturerList: Manufacturers[] = [];
  vatList: ArticleVat[] = [];
  brandlist: BrandInfoList[] = [];
  supplierBrandList: BrandInfoList[] = [];
  uploadFile: any;

  article_id: number | undefined;
  articleDetail: Article | undefined;
  imgURL: string | undefined;

  rolePermission: any;
  PrixPub_PrixAchat_Permission:boolean = false;

  readOnly: boolean = false;
  pageTitle: string = 'KMODIFY_ARTICLE';

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  constructor(private location: Location,
              private authService: AuthService,
              private commonService: CommonService,
              private articleService: ArticleService,
              private formBuilder: FormBuilder,
              private dialog: MatDialog,
              private router: Router,
              private translateService: TranslateService,
              private datePipe: DatePipe,
              private route: ActivatedRoute,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService: DataSharingService) {
    }

  ngOnInit(): void {
    this.route.params.subscribe(params => {
      this.id = params["article_id"];

      this.article_id = +params["article_id"];
      this.loadDependencies();
    });

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];

        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }

  _changeToDetailMode() {
    this.pageTitle = 'KDETAIL_ARTICLE';

    this.modifyArticleForm.get('ARTCATID')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTMATID')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTLBL')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDESC')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDTEDEBDISP')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDTEOBSL')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDERNIERDTACHAT')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTTRACE')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDURGUART')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTPOID')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDIM1')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDIM2')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDIM3')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTMNFID')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTCONSREF')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTREFFAC')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTREF')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTDELLIV')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTTRESHLD')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTCMDQTE')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTASSUR')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTPUBPRIXHT')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTPURPRIXHT')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTSUPPLIER')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTAPR1RECP')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTVATID')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTIMG')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTIMG')?.disable({ onlySelf: true });
    this.modifyArticleForm.get('ARTPRIXVENTE')?.disable({ onlySelf: true });
  }

  async loadDependencies() {
    await this.loadingService.show();
    const userPermissions = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(userPermissions != null) {
      this.rolePermission = JSON.parse(userPermissions);
      this.PrixPub_PrixAchat_Permission = this.rolePermission['11_1'];
    }
    await this.getListCategory();
    await this.getMaterialTypeList();
    await this.getVatList();
    await this.getManufacturerList();
    await this.getBrandList();
    await this.getArticleDetail();
    this._refreshArticleData(this.articleDetail);
    this.setBrandListDisable();
    await this.loadingService.hide();
  }

  get brandField(){
    return (this.modifyArticleForm.get('ARTBRAND') as FormArray);
  }

  setBrandListDisable(){
    this.brandField.value.forEach((brand:any) => {
      if(brand.brand_id){
        this.brandlist.map((brandDetail: BrandInfoList) =>{
          if(brandDetail.BRID == brand.brand_id){
            brandDetail.disabled = true;
          }
        })
      }
    })
  }

  selectBrand(index: number){
    const selectedData = this.brandField.value[index].brand_id;
    this.brandField.value.forEach((brand:any) => {
      if(brand.brand_id){
        if(selectedData != brand.brand_id){
          this.brandlist.map((brandDetail: BrandInfoList) =>{
            if(brandDetail.BRID == brand.brand_id){
              brandDetail.disabled = true;
            }
          })
        }else{
          this.brandlist.map((brandDetail: BrandInfoList) =>{
            if(brandDetail.BRID == brand.brand_id){
              brandDetail.disabled = false;
            }
          })
        }
      }
    })
  }

  createBrand(){
    this.brandField.push(this.getBrand(null));
  }

  getBrand(brandDetail:any){
    let newBrand = this.formBuilder.group({
      brand_id: ["" ,[Validators.required]],
      prix_de_vente: ["", [Validators.required, Validators.pattern(APP_CONSTANTS.MIN_NUM_PATTERN),Validators.minLength(APP_CONSTANTS.ARTPRIXVENTE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.ARTPRIXVENTE_MAX_LENGTH)]]
    });

    if(brandDetail){
      newBrand = this.formBuilder.group({
        brand_id: [{value: brandDetail.brand_id, disabled: this.readOnly}, [Validators.required]],
        prix_de_vente: [{value: brandDetail.prix_de_vente, disabled: this.readOnly}, [Validators.required]]
      });
    }
    return newBrand;

  }

  deleteBrand(index:any){
    this.brandlist.forEach((brand) => {
      if(brand.BRID == this.brandField.value[index].brand_id){
        brand.disabled = false
      }
    });
    this.brandField.removeAt(index);
  }

  getListCategory(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleCategories[]>) => {
          if(res.status === 'success') {
            this.categoriesList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleCategories().subscribe(observer);
    });
  }

  getMaterialTypeList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleMaterialTypes[]>) => {
          if(res.status === 'success') {
            this.materialtypesList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleMaterialTypes().subscribe(observer);
    });
  }

  getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            const brandData = res.data;
            brandData.forEach((value: BrandInfoList) =>{
              value.disabled = false;
            })
            this.brandlist = brandData;

            // Filter only Supplier Brands
            this.supplierBrandList = this.brandlist.filter((row) => row.BRTYPE === 'supplier');

            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  getManufacturerList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<Manufacturers[]>) => {
          if(res.status === 'success') {
            this.manufacturerList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListManufacturers().subscribe(observer);
    });
  }

  getVatList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticleVat[]>) => {
          if(res.status === 'success') {
            this.vatList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleVatList().subscribe(observer);
    });
  }

  getArticleDetail(): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      if(this.article_id === undefined || this.article_id === null) {
        resolve("INVALID_ARTICLE");
        return;
      }

      const observer = {
        next: (res: CustomHttpResponse<Article>) => {
          if(res.status === 'success') {
            this.imgURL = res.imageurl;
            this.articleDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.articleService.getArticleDetail(this.article_id).subscribe(observer);
    })
  }

  _refreshArticleData(articleDetail: Article | undefined) {
    if(articleDetail === undefined || articleDetail === null) {
      return;
    }

    this.modifyArticleForm.get('ARTCATID')?.setValue(articleDetail.category_id);
    this.modifyArticleForm.get('ARTMATID')?.setValue(articleDetail.material_id);
    this.modifyArticleForm.get('ARTLBL')?.setValue(articleDetail.label);
    this.modifyArticleForm.get('ARTDESC')?.setValue(articleDetail.description);
    this.modifyArticleForm.get('ARTDTEDEBDISP')?.setValue(articleDetail.start_date_avail);
    this.modifyArticleForm.get('ARTDTEOBSL')?.setValue(articleDetail.obsolescence);
    this.modifyArticleForm.get('ARTDERNIERDTACHAT')?.setValue(articleDetail.dernier_achat_date);
    this.modifyArticleForm.get('ARTTRACE')?.setValue(articleDetail.tracability);
    this.modifyArticleForm.get('ARTDURGUART')?.setValue(articleDetail.duree_garantie?.toString());
    this.modifyArticleForm.get('ARTPOID')?.setValue(articleDetail.poids?.toString());
    this.modifyArticleForm.get('ARTDIM1')?.setValue(articleDetail.dimm1?.toString());
    this.modifyArticleForm.get('ARTDIM2')?.setValue(articleDetail.dimm2?.toString());
    this.modifyArticleForm.get('ARTDIM3')?.setValue(articleDetail.dimm3?.toString());
    this.modifyArticleForm.get('ARTMNFID')?.setValue(articleDetail.manufacturer_id);
    this.modifyArticleForm.get('ARTCONSREF')?.setValue(articleDetail.const_reference);
    this.modifyArticleForm.get('ARTREFFAC')?.setValue(articleDetail.reference_facture);
    this.modifyArticleForm.get('ARTREF')?.setValue(articleDetail.reference);
    this.modifyArticleForm.get('ARTDELLIV')?.setValue(articleDetail.delay_livraison?.toString());
    this.modifyArticleForm.get('ARTTRESHLD')?.setValue(articleDetail.threshold?.toString());
    this.modifyArticleForm.get('ARTCMDQTE')?.setValue(articleDetail.commande_qte?.toString());
    this.modifyArticleForm.get('ARTASSUR')?.setValue(articleDetail.assurance?.toString());
    this.modifyArticleForm.get('ARTPUBPRIXHT')?.setValue(articleDetail.public_prix?.toString());
    this.modifyArticleForm.get('ARTPURPRIXHT')?.setValue(articleDetail.purchase_prix?.toString());
    this.modifyArticleForm.get('ARTSUPPLIER')?.setValue(articleDetail.article_supplier_id);
    this.modifyArticleForm.get('ARTAPR1RECP')?.setValue(articleDetail.apres_1_reception);
    this.modifyArticleForm.get('ARTVATID')?.setValue(articleDetail.vat_id);
    this.modifyArticleForm.get('ARTPRIXVENTE')?.setValue(articleDetail.article_prix_de_vente);
    articleDetail.article_brand.forEach((value:any) =>{
      this.brandField.push(this.getBrand(value));
    })
  }

  async openFusionerPopup() {
    const data = {
      currentArticle: this.articleDetail
    };

    const dialog = await this.dialog.open(FusionerArticleComponent, {
      data,
      maxHeight: "90vh"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        const fusionnerType = res.fusionnerType;

        if(fusionnerType === APP_CONSTANTS.FUSIONNER_BOTH) {
          return;
        }

        if(fusionnerType === APP_CONSTANTS.FUSIONNER_Y_TO_X) {
          this.articleDetail = res.data;
          if(this.articleDetail !== undefined && this.articleDetail !== null) {
            this._refreshArticleData(this.articleDetail);
          }

          return;
        }

        if(fusionnerType === APP_CONSTANTS.FUSIONNER_X_TO_Y) {
          this.articleDetail = res.data;

          const queryParams = {
            id: this.articleDetail?.id
          };

          this.router.navigate(
            [],
            {
              relativeTo: this.route,
              queryParams: queryParams,
              queryParamsHandling: 'merge', // remove to replace all query params by provided
            });

          if(this.articleDetail !== undefined && this.articleDetail !== null) {
            this._refreshArticleData(this.articleDetail);
          }

          return;
        }
      }
    });
  }

  get fr() {
    return this.modifyArticleForm.controls;
  }

  modifyArticleFormSubmitter() {
    this.submitted = true;

    if(this.modifyArticleForm.invalid){
      return;
    }

    this.updateArticle();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ARTICLE_LIST);
    if(listCache !== undefined) {
      let listLigneInfo = listCache.data as Article[];
      listLigneInfo = listLigneInfo.map((row) => {
        if(row.id == this.article_id) {
          row.id = this.article_id;
          row.apres_1_reception = payload.ARTAPR1RECP;
          row.assurance = payload.ARTASSUR;
          row.category_id = payload.ARTCATID;
          row.commande_qte = payload.ARTCMDQTE;
          row.const_reference = payload.ARTCONSREF;
          row.reference_facture = payload.ARTREFFAC;
          row.delay_livraison = payload.ARTDELLIV;
          row.description = payload.ARTDESC;
          row.dimm1 = payload.ARTDIM1;
          row.dimm2 = payload.ARTDIM2;
          row.dimm3 = payload.ARTDIM3;
          row.duree_garantie = payload.ARTDURGUART;
          row.image = payload.ARTIMG;
          row.label = payload.ARTLBL;
          row.threshold = payload.ARTTRESHLD;
          row.tracability = payload.ARTTRACE;
          row.article_supplier_id = payload.ARTSUPPLIER;
          row.vat_id = payload.ARTVATID;
          row.poids = payload.ARTPOID;
          row.material_id = payload.ARTMATID;
          row.public_prix = payload.ARTPUBPRIXHT;
          row.purchase_prix = payload.ARTPURPRIXHT;
          row.reference = payload.ARTREF;
          row.manufacturer_id = payload.ARTMNFID;
          row.obsolescence = payload.ARTDTEOBSL;
          row.dernier_achat_date = payload.ARTDERNIERDTACHAT;
          row.start_date_avail = payload.ARTDTEDEBDISP;
        }
        return row;
      });

      listCache.data = listLigneInfo;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ARTICLE_LIST, JSON.stringify(listCache));
    }
  }

  async updateArticle() {
    if(this.article_id === undefined || this.article_id === null) {
      return;
    }

    const formData = this.modifyArticleForm.getRawValue();
    const payload: any = new FormData();
    payload.append("ARTREF", formData.ARTREF);
    payload.append("ARTLBL", formData.ARTLBL);
    payload.append("ARTDESC", formData.ARTDESC);
    payload.append("ARTCATID", formData.ARTCATID);
    payload.append("ARTMATID", formData.ARTMATID);
    payload.append("ARTDTEDEBDISP", formData.ARTDTEDEBDISP ? moment(formData.ARTDTEDEBDISP).format(CUSTOM_DATE_API_FORMAT) : "");
    payload.append("ARTDTEOBSL", formData.ARTDTEOBSL ? moment(formData.ARTDTEOBSL).format(CUSTOM_DATE_API_FORMAT) : "");
    payload.append("ARTDERNIERDTACHAT", formData.ARTDERNIERDTACHAT ? moment(formData.ARTDERNIERDTACHAT).format(CUSTOM_DATE_API_FORMAT) : "");
    payload.append("ARTTRACE", formData.ARTTRACE);
    payload.append("ARTDURGUART", formData.ARTDURGUART);
    payload.append("ARTPOID", formData.ARTPOID);
    payload.append("ARTDIM1", formData.ARTDIM1);
    payload.append("ARTDIM2", formData.ARTDIM2);
    payload.append("ARTDIM3", formData.ARTDIM3);
    payload.append("ARTMNFID", formData.ARTMNFID);
    payload.append("ARTCONSREF", formData.ARTCONSREF);
    payload.append("ARTREFFAC", formData.ARTREFFAC);
    payload.append("ARTTRESHLD", formData.ARTTRESHLD);
    payload.append("ARTCMDQTE", formData.ARTCMDQTE);
    payload.append("ARTSUPPLIER", formData.ARTSUPPLIER);
    payload.append("ARTASSUR", formData.ARTASSUR);
    payload.append("ARTPUBPRIXHT", formData.ARTPUBPRIXHT);
    payload.append("ARTPURPRIXHT", formData.ARTPURPRIXHT);
    payload.append("ARTVATID", formData.ARTVATID);
    payload.append("ARTIMG", this.uploadFile);
    payload.append("ARTAPR1RECP", formData.ARTAPR1RECP);
    payload.append("ARTDELLIV", formData.ARTDELLIV);
    payload.append("ARTPRIXVENTE", formData.ARTPRIXVENTE);
    payload.append("ARTBRAND", JSON.stringify(formData.ARTBRAND));

    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(formData);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['article/article-list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.articleService.updateArticle(this.article_id, payload).subscribe(observer);
  }

  uploadLogo(fileInput:any) {
    if (fileInput.target.files && fileInput.target.files[0]) {
      this.uploadFile = fileInput.target.files[0];
      this.modifyArticleForm.get('ARTIMG')?.setValue(this.uploadFile.name, { onlySelf: true });
      this.modifyArticleForm.patchValue({
        ARTIMG: this.uploadFile.name
      });
      this.modifyArticleForm.updateValueAndValidity();
    } else {
      this.modifyArticleForm.get('ARTIMG')?.setValue(this.translateService.instant('KARTICLE_CHOOSE_IMAGE'), { onlySelf: true });
      this.uploadFile = undefined;
    }
  }

}
