import {Component, Inject, OnInit} from '@angular/core';
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from "@angular/material/dialog";
import {TranslateService} from "@ngx-translate/core";
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {DialogService} from "../../../_services/dailog.service";
import {lastValueFrom} from "rxjs";
import {ArticleService} from "../../../_services/article.service";
import {NgxSpinnerService} from "ngx-spinner";
import {Article} from "../../../_models/article";
import {Store} from "../../../_models";
import {FusionnerTypeComponent} from "../../../_components/fusionner-type/fusionner-type.component";
import {APP_CONSTANTS} from "../../../_helpers/constants";

@Component({
  selector: 'app-fusioner-article',
  templateUrl: './fusioner-article.component.html',
  styleUrls: ['./fusioner-article.component.scss']
})
export class FusionerArticleComponent implements OnInit {

  fusionnerForm: FormGroup = this.formBuilder.group({
    CHOSEN_FUSIONNER_ARTICLE: [undefined, Validators.required]
  });
  currentArticle: Article | undefined;

  constructor(public dialogRef: MatDialogRef<FusionerArticleComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private translateService: TranslateService,
              private formBuilder: FormBuilder,
              private dialogService: DialogService,
              private articleService: ArticleService,
              private loadingService: NgxSpinnerService,
              private dialog: MatDialog) { }

  ngOnInit(): void {
    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('currentArticle')) {
        this.currentArticle = this.data.currentArticle;
      }
    }
  }

  onFusionnerArticleChange(event: any) {
    this.fusionnerForm.get('CHOSEN_FUSIONNER_ARTICLE')?.setValue(event);
  }

  get _getChosenFusionnerArticle() {
    return this.fusionnerForm.get('CHOSEN_FUSIONNER_ARTICLE');
  }

  confirmFusionnerArticle(parentArticle: Article | undefined, chosenArticle: Article | undefined, type: string): Promise<any> {
    let message = "";

    if(type === APP_CONSTANTS.FUSIONNER_Y_TO_X) {
      message =  `<h3>${this.translateService.instant('KFUSIONNER_ARTICLE_CONFIRMATION_CHILD_PARENT', { CHOSEN_REFERENCE: chosenArticle?.reference, CHOSEN_LABEL: chosenArticle?.label, PARENT_REFERENCE: parentArticle?.reference, PARENT_LABEL: parentArticle?.label})}</h3>`;
    }

    if(type === APP_CONSTANTS.FUSIONNER_X_TO_Y) {
      message =  `<h3>${this.translateService.instant('KFUSIONNER_ARTICLE_CONFIRMATION_PARENT_CHILD', { PARENT_REFERENCE: parentArticle?.reference, PARENT_LABEL: parentArticle?.label, CHOSEN_REFERENCE: chosenArticle?.reference, CHOSEN_LABEL: chosenArticle?.label})}</h3>`;
    }

    if(type === APP_CONSTANTS.FUSIONNER_BOTH) {
      message = this.translateService.instant('KFUSIONNER_ARTICLE_NOTHING_CHANGES')
    }

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KFUSIONNER_ARTICLE_CONTINUE_MERGE',
      message: message
    }));
  }

  async confirmFusionnerType(parentArticle: Article | undefined, chosenArticle: Article | undefined): Promise<any> {
    return new Promise<any>((resolve) => {
      const dialog = this.dialog.open(FusionnerTypeComponent, {
        data: {
          x: `${parentArticle?.reference} - ${parentArticle?.label}`,
          y: `${chosenArticle?.reference} - ${chosenArticle?.label}`
        },
        width: '400px',
        disableClose: true,
      });

      dialog.afterClosed().subscribe((res) => {
        resolve(res);
      })
    })
  }

  async onFusionnerSubmitter() {
    console.log("FORM_DATA", this.fusionnerForm.getRawValue());

    if(this.fusionnerForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KFUSIONNER_ARTICLE_NO_ARTICLE_CHOOSEN'
      });
      return;
    }

    const formData = this.fusionnerForm.getRawValue();

    const confirmationTypeRes = await this.confirmFusionnerType(this.currentArticle, formData.CHOSEN_FUSIONNER_ARTICLE);

    if(confirmationTypeRes === undefined || confirmationTypeRes === null || confirmationTypeRes === '') {
      return;
    }

    const confirmationType = confirmationTypeRes.data;
    if(confirmationType === APP_CONSTANTS.FUSIONNER_BOTH) {
      this.dialogRef.close({ status: "success", fusionnerType: confirmationType, data: [] });
      return;
    }

    const confirmation = await this.confirmFusionnerArticle(this.currentArticle, formData.CHOSEN_FUSIONNER_ARTICLE, confirmationType);

    if(confirmation) {
      this.loadingService.show();

      let x = undefined;
      let y = undefined;

      if(confirmationType === APP_CONSTANTS.FUSIONNER_Y_TO_X) {
        x = this.currentArticle?.id;
        y = formData.CHOSEN_FUSIONNER_ARTICLE.id;
      }

      if(confirmationType === APP_CONSTANTS.FUSIONNER_X_TO_Y) {
        x = formData.CHOSEN_FUSIONNER_ARTICLE.id;
        y = this.currentArticle?.id;
      }

      const payload = {
        main_article_id: x,
        child_article_id: y
      };

      this.callFusionnerArticleAPI(payload, confirmationType);
      return;
    }

    return;
  }

  async callFusionnerArticleAPI(payload: any, fusionnerType: string) {
    this.articleService.mergeTwoArticles(payload).subscribe(
      async (res: any) => {
        this.loadingService.hide();

        if(res.status === 'success') {
          const mergedMainArticle = res.data;

          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });

          this.dialogRef.close({ status: "success", data: mergedMainArticle, fusionnerType });
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      err => {
        this.loadingService.hide();
      }
    )
  }

  onClickClose() {

  }

}
