import {Component, Inject, OnInit} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {Article} from "../../../_models/article";
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from "@angular/material/dialog";
import {TranslateService} from "@ngx-translate/core";
import {DialogService} from "../../../_services/dailog.service";
import {ArticleService} from "../../../_services/article.service";
import {NgxSpinnerService} from "ngx-spinner";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {lastValueFrom} from "rxjs";
import {FusionnerTypeComponent} from "../../../_components/fusionner-type/fusionner-type.component";

@Component({
  selector: 'app-search-article',
  templateUrl: './search-article.component.html',
  styleUrls: ['./search-article.component.scss']
})
export class SearchArticleComponent implements OnInit {

  searchArticleForm: FormGroup = this.formBuilder.group({
    CHOSEN_ARTICLE: [undefined, Validators.required]
  });
  currentArticle: Article | undefined;
  chosenStockLocation: string | undefined;
  searchKeyword: string | undefined;

  constructor(public dialogRef: MatDialogRef<SearchArticleComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private translateService: TranslateService,
              private formBuilder: FormBuilder,
              private dialogService: DialogService) { }

  ngOnInit(): void {
    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('currentArticle')) {
        this.currentArticle = this.data.currentArticle;
      }

      if(this.data.hasOwnProperty('stockLocation')) {
        this.chosenStockLocation = this.data.stockLocation;
      }

      if(this.data.hasOwnProperty('searchKeyword')) {
        this.searchKeyword = this.data.searchKeyword;
      }
    }
  }

  onSearchArticleChange(event: any) {
    this.searchArticleForm.get('CHOSEN_ARTICLE')?.setValue(event);
  }

  get _getChosenArticle() {
    return this.searchArticleForm.get('CHOSEN_ARTICLE');
  }

  async onSearchArticleFormSubmitter() {
    console.log("FORM_DATA", this.searchArticleForm.getRawValue());

    if(this.searchArticleForm.invalid) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KFUSIONNER_ARTICLE_NO_ARTICLE_CHOOSEN'
      });
      return;
    }

    this.dialogRef.close({ status: "success", data: this._getChosenArticle?.value });
  }

  onClickClose() {

  }

}
