import { Component, OnInit } from '@angular/core';
import { AuthService } from '../../../_services/auth.service';
import { FormBuilder, FormGroup, FormControl } from '@angular/forms';
import { Router } from '@angular/router';
import {PageEvent} from '@angular/material/paginator';
import {DatePipe} from '@angular/common';
import { MatDialog } from '@angular/material/dialog';
import { DataSharingService} from '../../../_services/data-sharing.service';
import { BonDeLivraisonService} from '../../../_services/bon-de-livraison.service';
import {CustomHttpResponse} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {ListLivraison} from "../../../_models/livraison";
import {lastValueFrom} from "rxjs";
import {DialogService} from "../../../_services/dailog.service";
import { TranslateService } from '@ngx-translate/core';
import { DashboardService } from 'src/app/_services/dashboard.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';

@Component({
  selector: 'app-bon-de-livraison-list',
  templateUrl: './bon-de-livraison-list.component.html',
  styleUrls: ['./bon-de-livraison-list.component.scss']
})
export class BonDeLivraisonListComponent implements OnInit {

  searchForm: FormGroup = this.formBuilder.group({
    document_no: [''],
    start_date: [{ value: '', disabled: true }],
    end_date: [{ value: '', disabled: true }],
    status: [''],
    is_return_pending: [false]
  });

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];

  livraisonList: ListLivraison[] | undefined;
  displayedColumnsTransfer: string[] = ['DATED', 'DOCUMENT_NO', 'STORE', 'PT', 'PRET_MATERIEL', 'SAV', 'RETURN_STATUS', 'status','actions'];

  rolePermission:any;

  statusList = APP_CONSTANTS.LIVRAISON_STATUS;

  constructor(private authService: AuthService,
              private formBuilder: FormBuilder,
              private router: Router,
              public dialog: MatDialog,
              public dataSharingService: DataSharingService,
              private datePipe: DatePipe,
              private translateService:TranslateService,
              private bonDeLivraisonService: BonDeLivraisonService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dashboardService: DashboardService) {
  }

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    const statusValue = this.dashboardService.dashboardStatusValue;
    this.dashboardService.dashboardStatusValue = '';
    const returnPendingStatus = this.dashboardService.returnPendingStatus;
    this.dashboardService.returnPendingStatus = false;
    if(statusValue){
      this.searchForm.get('status')?.setValue(parseInt(statusValue));
      this.onSearch();
      return;
    }
    if(returnPendingStatus){
      this.searchForm.get('is_return_pending')?.setValue(returnPendingStatus);
      this.onSearch();
      return;
    }
    this.checkForListCache();
  }

  
  documentStatus(item: ListLivraison, field: string) {
    let result = "";
    if(field === 'LIVRAISON_STATUS'){
      const statusList = APP_CONSTANTS.LIVRAISON_STATUS;
      const statusData = statusList.find((r) => r.value == item.status);
      
      if(statusData == undefined) {
        return result;
      }
  
      switch (item.status) {
        case APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_YES:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_NO:
          result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    if(field === 'PRET_MATERIEL_STATUS'){
      const statusList = APP_CONSTANTS.PRET_MATERIEL_STATUS;
      const statusData = statusList.find((r) => r.value == item.pret_material_status);
      
      if(statusData == undefined) {
        return result;
      }
  
      switch (item.pret_material_status) {
        case APP_CONSTANTS.PRET_MATERIEL_STAT_YES:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.PRET_MATERIEL_STAT_NO:
          result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    if(field === 'SAV_STATUS'){
      const statusList = APP_CONSTANTS.SAV_STATUS;
      const statusData = statusList.find((r) => r.value == item.sav_status);
      
      if(statusData == undefined) {
        return result;
      }
  
      switch (item.sav_status) {
        case APP_CONSTANTS.SAV_STAT_YES:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.SAV_STAT_NO:
          result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    if(field === 'LIVRAISON_ARTICLE_RETURN_STATUS'){
      const statusList = APP_CONSTANTS.LIVRAISON_ARTICLE_RETURN_STATUS;
      const statusData = statusList.find((r) => r.value == item.return_status);
      
      if(statusData == undefined) {
        return result;
      }
      console.log("Stat",item.return_status)
      switch (item.return_status) {
        case APP_CONSTANTS.LIVRAISON_ARTICLE_RETURN_STAT_YES:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.LIVRAISON_ARTICLE_RETURN_STAT_NO || item.return_status == null:
          result = `<span class='badge bg-info'>${this.translateService.instant(statusData?.label)}</span>`;
          console.log('No not working')
          break;
      }
    }

    return result;
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        document_no: formCache.document_no,
        start_date: formCache.start_date,
        end_date: formCache.end_date,
        status: formCache.status,
        is_return_pending: formCache.is_return_pending
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST);
      if(dataCache !== undefined) {

        this.livraisonList = dataCache.data as ListLivraison[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.onSearch();
    }
  }

  _isItemPretMaterialActive(status: number) {
    return status == APP_CONSTANTS.PRET_MATERIEL_STAT_YES;
  }

  _isItemSavActive(status: number) {
    return status == APP_CONSTANTS.SAV_STAT_YES;
  }

  isDocumentValidated(item: ListLivraison): boolean {
    return item.status === APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_YES;
  }

  onSearch() {
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST_FORM, JSON.stringify(formData));
    const payload = {
      document_no: formData.document_no,
      start_date: formData.start_date,
      end_date: formData.end_date,
      status: formData.status,
      is_return_pending: formData.is_return_pending,
    };

    this.getLivraisonList(payload, this.currentPage, this.pageSize);
  }

  __updateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST);
    if(listCache !== undefined) {
      let listLivraison: any[] = listCache.data;
      listLivraison = (listLivraison as ListLivraison[]).filter(r => r.document_no != id);

      listCache.data = listLivraison;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listLivraison.length === 0) {
        this.onSearch();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    const payload = {
      document_no: formData.document_no,
      start_date: formData.start_date,
      end_date: formData.end_date,
      status: formData.status,
      is_return_pending: formData.is_return_pending
    };

    this.getLivraisonList(payload, this.currentPage, this.pageSize);
  }

  getLivraisonList(payload: any, page: number = 1, pageSize: number): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ListLivraison[]>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            this.livraisonList = res.data;

            this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
            this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
            this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST, JSON.stringify(res));
          } else {
            this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST);
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.bonDeLivraisonService.getLivraisonList(payload, page, pageSize).subscribe(observer);
    });
  }

  confirmRemoveLivraison(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeLivraison(document_no: string, item: ListLivraison) {
    if((this.isDocumentValidated(item) && !this.rolePermission['2_17']) || (!this.isDocumentValidated(item) && !this.rolePermission['2_14'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveLivraison(document_no);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteLivraison(document_no);
    }
  }

  async _deleteLivraison(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(document_no);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });

          this.onSearch();
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();

        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.bonDeLivraisonService.deleteLivraison(document_no).subscribe(observer);
  }


}
