import { Component, Inject, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatDialogRef, MAT_DIALOG_DATA } from '@angular/material/dialog';
import { DialogService } from 'src/app/_services/dailog.service';

@Component({
  selector: 'app-client-popup',
  templateUrl: './client-popup.component.html',
  styleUrls: ['./client-popup.component.scss']
})
export class ClientPopupComponent implements OnInit {

  clientForm: FormGroup = this.formBuilder.group({
    CHOSEN_CLIENT: [undefined, Validators.required]
  });

  currentClient: any;
  displayType: any;

  searchFilterBy: string = "store";
  searchKeyword: string = "";
  searchType: "client" | "supplier" = "supplier";

  filterParams: any;

  constructor(public dialogRef: MatDialogRef<ClientPopupComponent>,
              @Inject(MAT_DIALOG_DATA) public data: any,
              private formBuilder: FormBuilder) {
   }

  ngOnInit(): void {
    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('displayType')) {
        this.displayType = this.data.displayType;

        if(this.displayType !== 'default') {
          this.searchFilterBy = "store";
        }
      }

      if(this.data.hasOwnProperty('searchPayload')) {
        this.searchKeyword = this.data.searchPayload.keyword;
        this.searchType = this.data.searchPayload.store_type;
      }

      if(this.data.hasOwnProperty('currentCustomer')) {
        this.currentClient = this.data.currentCustomer;
      }

      this.filterParams = {
        FILTER_TYPE: this.searchFilterBy,
        FILTER_KEYWORD: this.searchKeyword,

        FILTER_STORE_TYPE: this.searchType,
      }
    }
  }

  onChange(event: any) {
    this.clientForm.get('CHOSEN_CLIENT')?.setValue(event);
  }

  get _chosenContact() {
    return this.clientForm.get('CHOSEN_CLIENT');
  }

  onContactSubmitter(){
    let formData = this.clientForm.getRawValue();
    const chosenClient = formData.CHOSEN_CLIENT;

    this.dialogRef.close({ status: "success", data: chosenClient });
  }

}
