import {DatePipe} from '@angular/common';
import {Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {FormBuilder, FormGroup, Validators} from '@angular/forms';
import {MatDialog} from '@angular/material/dialog';
import {ActivatedRoute, Params, Router} from '@angular/router';
import {firstValueFrom, lastValueFrom, Observable, Subscription} from 'rxjs';
import {BonDeLivraisonService} from 'src/app/_services/bon-de-livraison.service';
import {CommonService} from 'src/app/_services/common.service';
import {NumeroSerieComponent} from '../../inventory/numero-serie/numero-serie.component';
import * as moment from "moment";
import {
  CustomHttpResponse,
  LigneInfoList,
  LivraisonPrestation,
  StockLocationList
} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {
  CUSTOM_DATE_API_FORMAT,
  CUSTOM_DATE_FORMAT,
  CUSTOM_DATE_TIME_API_FORMAT,
  CUSTOM_DATE_TIME_FORMAT
} from "../../../_helpers/custom-date-formats";
import {MatSelect} from "@angular/material/select";
import {DialogService} from "../../../_services/dailog.service";
import {ModifyItemComponent} from "../../bon-de-reception/modify-item/modify-item.component";
import {PdfViewerComponent} from "../../../dialogs/pdf-viewer/pdf-viewer.component";
import {TranslateService} from "@ngx-translate/core";
import { BonLivraison, BonLivraisonCustomer, BonLivraisonItem, ListLivraison } from 'src/app/_models/livraison';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {Article, SearchArticle} from "../../../_models/article";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {SaisieExpeditionComponent} from "../saisie-expedition/saisie-expedition.component";
import {DataSharingService} from "../../../_services/data-sharing.service";
import {MatCheckboxChange} from "@angular/material/checkbox";
import {SelectionCopyTypeComponent} from "../../../_components/selection-copy-type/selection-copy-type.component";
import { HistoriqueComponent } from 'src/app/_components/historique/historique.component';
import {cloneDeep} from "lodash";
import { LivraisonPrestationService } from 'src/app/_services/livraison-prestation.service';
import {SearchArticleComponent} from "../../article/search-article/search-article.component";

@Component({
  selector: 'app-edit-bon-de-livraison',
  templateUrl: './edit-bon-de-livraison.component.html',
  styleUrls: ['./edit-bon-de-livraison.component.scss'],
})
export class EditBonDeLivraisonComponent implements OnInit {


  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;
  @ViewChild('stockLocationInput') stockLocationInputEl: MatSelect | undefined;
  @ViewChild('rechercheRefInput') rechercheRefInputEl: ElementRef | undefined;

  bonDocumentForm: FormGroup = this.formBuilder.group({
    NUMBON: [{ value: "", disabled: true }],
    DATE_DU_BON: [{ value: "", disabled: true }],
    PICKUP_DATE: [{ value: ""}],
    SHIPPING_DATE: [{ value: ""}],
    AXESITE: [''],
    DOCUMENT_ASSOC: [''],
    SERVICE_TYPE:[''],
    TRANSPORT:[''],
    RETURN_VOUCHER:[''],
    LIGNE_INFO: [''], // Ligne ID
    LIGNE_CUSTOM_LABEL: [''], // Ligne Label

    PRET_MATERIEL:[''],
    SAV:[''],

    SEARCH_STOCK_LOCATION: [""],
    SEARCH_REF: [""]
  })

  bonLivraisonItemForm: FormGroup = this.formBuilder.group({
    STOCK_SOURCE_ID: ["", Validators.required],
    STOCK_SOURCE_LBL: [{ value: "", disabled: true }, Validators.required],
    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    LABEL: [{ value: "", disabled: true }, Validators.required],
    QTE: [{ value: "0", disabled: true }, [Validators.required, Validators.min(1)]],
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    AVAIL_STOCK: [{ value: "0", disabled: true }],
    RETURN_STATUS: ["0"],
    RETURN_DAYS: [''],
    MATERIAL_ACTIVABLE_STATUS: [null],
    SERIALS_LIST: ['', Validators.required],
    SELECTED_SERIALS_LIST:[''],
    ASSURANCE:[null, [Validators.minLength(APP_CONSTANTS.ASSURANCE_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.ASSURANCE_MAX_LENGTH)]]
  });

  livraisonPrestation: LivraisonPrestation[] = [] // nature de prestation field values

  newArticleSelection: Article | undefined;
  subscriptions: Subscription[] | undefined = [];
  bonDocumentNum: string | undefined = "*********";
  bonLivraison: BonLivraison | undefined;
  bonLivraisonItems: BonLivraisonItem[] = [];
  bonLivraisonArticles: Article[] = [];

  totalArticles: number = 0;
  totalQte: number = 0;
  totalAssurance: number = 0;
  totalPT: number = 0;

  ligneList: LigneInfoList[] = [];
  stockLocationList: StockLocationList[] = [];

  fullKeywordMatch: boolean = false;
  isKeywordSerial: boolean = false;
  serialStockLocation: string = '';
  searchRefArticlesList: Article[] | undefined = undefined;

  displayedColumns: string[] = [ 'REFERENCE', 'LABEL', 'STOCK_SOURCE', 'QTE', 'ASSURANCE', 'RETURN_STATUS', 'ACTIVE_STATUS', 'ACTIONS'];

  chosenCustomer: BonLivraisonCustomer | undefined;

  customLigneInput: boolean = false;

  // Used to decide whether the values can be editable or not
  readOnly: boolean = false;

  pageTitle: string = "KMODIFY_BON_DE_LIVRAISON";

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;
  returnDaysLength = APP_CONSTANTS.RETURN_DAYS_MAX_LENGTH;
  chosenSelectionArticle: BonLivraisonItem[] = [];

  //PRIX_UNIT AND PRIX_TOTAL Permission
  PU_PT_Permission: boolean = false;

  rolePermission:any;

  constructor(public dialog: MatDialog,
    private commonService: CommonService,
    private livraisonService: BonDeLivraisonService,
    private datePipe: DatePipe,
    private router: Router,
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private dataSharingService: DataSharingService,
    private loadingService: NgxSpinnerService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private livraisonPrestationService: LivraisonPrestationService) {
  }

  ngOnInit(): void {
    this.loadDependencies();

    this.bonDocumentForm.get("LIGNE_INFO")?.valueChanges.subscribe(
      (res) => {
        if(res !== undefined && res !== null) {
          if(res === 'custom-ligne') {
            this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue("");
            this.customLigneInput = true;
          } else {
            this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(res.LGILBL);
            this.customLigneInput = false;
          }
        }
      }
    )
  }

  async loadDependencies() {
    await this.loadingService.show();

    firstValueFrom(this.route.data).then(
      (data: any) => {
        if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
          this.readOnly = data.readOnly;
          if(this.readOnly) {
            this.__changeToDetailMode();
          }
        }
      }
    );

    this.bonDocumentForm.patchValue({
      NUMBON: this.bonDocumentNum,
      DATE_DU_BON: moment().format(CUSTOM_DATE_TIME_FORMAT)
    });
    this._getRolePermission();
    await this._getLigneList();
    await this._getStockLocationList();
    await this._getLivraisonPrestation();

    firstValueFrom(this.route.params).then(
      (params) => {
        console.log(params);
        if(params.hasOwnProperty('document_no') && params["document_no"] !== null && params["document_no"] !== "") {
          this.bonDocumentNum = params["document_no"];

          this._getBonLivraisonDetail(this.bonDocumentNum);
        }
      }
    )

    await this.loadingService.hide();
  }

  _getRolePermission(){
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.PU_PT_Permission = this.rolePermission['11_1'];
    if(this.PU_PT_Permission){
      this.displayedColumns =  [ 'REFERENCE', 'LABEL', 'STOCK_SOURCE', 'QTE', 'PRIX_UNIT', 'PRIX_TOTAL', 'ASSURANCE', 'RETURN_STATUS', 'ACTIVE_STATUS', 'ACTIONS'];
    }else{
      this.bonLivraisonItemForm.get('PRIX_UNIT')?.clearValidators();
    }
  }

  get __isDocumentValidated() {
    return this.bonLivraison && this.bonLivraison.status === APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_YES;
  }

  get material_activable_status() {
    return this.newArticleSelection && this.newArticleSelection?.material_activable_status;
  }

  __changeToDetailMode() {
    this.pageTitle = "KDETAIL_BON_DE_LIVRAISON";

    // For Selection button
    if(this.__isDocumentValidated) {
      this.displayedColumns.unshift('ITEM_SELECTION');
    }

    this.bonDocumentForm.get('PICKUP_DATE')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('SHIPPING_DATE')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('AXESITE')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('DOCUMENT_ASSOC')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('SERVICE_TYPE')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('TRANSPORT')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('RETURN_VOUCHER')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('LIGNE_INFO')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('LIGNE_CUSTOM_LABEL')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('PRET_MATERIEL')?.disable({ onlySelf: true });
    this.bonDocumentForm.get('SAV')?.disable({ onlySelf: true });
  }

  __selectionItemExists(row: BonLivraisonItem) {
    if(this.chosenSelectionArticle === undefined || this.chosenSelectionArticle === null) {
      return false;
    }

    return this.chosenSelectionArticle.find(r => r.item_id == row.item_id);
  }

  onArticleSelectionChange(e: MatCheckboxChange, row: BonLivraisonItem) {
    if(e.checked) {
      const itemExists = this.__selectionItemExists(row);
      if(!itemExists) {
        this.chosenSelectionArticle.push(row);
      }
    } else {
      this.chosenSelectionArticle = this.chosenSelectionArticle.filter(r => r.item_id != row.item_id);
    }
  }

  async openUpSelectionsPopup(): Promise<any> {
    return new Promise<any>((resolve) => {
      const dialog = this.dialog.open(SelectionCopyTypeComponent, {
        data: {
          documentType: 'BL'
        },
        width: '400px',
        disableClose: true,
      });

      dialog.afterClosed().subscribe((res) => {
        resolve(res);
      })
    })
  }

  onSelectionItemsClick() {
    if(!this.readOnly) {
      return;
    }

    if(this.chosenSelectionArticle === undefined || this.chosenSelectionArticle === null || this.chosenSelectionArticle.length === 0) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ITEMS_CHOSE_TO_COPY'
      });

      return;
    }

    this.openUpSelectionsPopup().then((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        const type = res.data;
        this.dataSharingService.selectionItems.next({
          type: type,
          items: this.chosenSelectionArticle,
          extras: {
            document_type: "BL",
            document_no: this.bonDocumentNum || '',
            documentDetail: this.bonLivraison
          }
        });

        let route
        if(type === APP_CONSTANTS.SELECTION_BL_TO_BR) {
          route = '/bon-de-reception/create';
        }

        if(route) {
          this.router.navigate([route]);
        }
      }
    });
  }

  async onCloseDocument() {
    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(this.isBonLivraisonModifiable && !this.readOnly && (this.chosenCustomer === undefined || this.chosenCustomer === null)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KLIVRAISON_CLIENT_NOT_ASSIGNED'
      })

      return;
    }

    this.returnBack();
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST);
    if(listCache !== undefined) {
      let listLivraison = listCache.data as ListLivraison[];
      listLivraison = listLivraison.map((row) => {
        if(row.document_no == payload.document_no) {
          console.log(row);
          row.axesite = payload.axesite;
          row.created_at = payload.created_at;
          row.dated = payload.dated;
          row.docs_assoc = payload.docs_assoc;
          row.document_no = payload.document_no;
          row.ligne_id = payload.ligne_id;
          row.ligne_lbl = payload.ligne_lbl;
          row.livraison_id = payload.livraison_id;
          row.modified_at = payload.modified_at;
          row.pickup_date = payload.pickup_date;
          row.pret_material_status = payload.pret_material_status;
          row.return_days = payload.return_days;
          row.return_status = payload.return_status;
          row.return_voucher = payload.return_voucher;
          row.sav_status = payload.sav_status;
          row.service_type = payload.service_type;
          row.shipping_date = payload.shipping_date;
          row.status = payload.status;
          row.store_id = payload.store_id;
          row.store_label = payload.store_label;
          row.total_assurance = payload.total_assurance;
          row.total_price = payload.total_price;
          row.transport = payload.transport;
        }
        return row;
      });
      listCache.data = listLivraison;
      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST, JSON.stringify(listCache));
    }
  }

  __deleteUpdateListCache(id: string) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST);
    if(listCache !== undefined) {
      let listLivraison: any[] = listCache.data;
      listLivraison = (listLivraison as ListLivraison[]).filter(r => r.document_no != id);

      listCache.data = listLivraison;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_LIST, JSON.stringify(listCache));

      this.__updateListCache(listCache.data);
    }
}


  async onSubmitDocument(type: 'ok' | 'validate' = 'ok') {
    if(this.readOnly) {
      return;
    }

    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    if(type === 'validate'){
      if(!this.rolePermission['2_15']) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KACCESS_DENIED'
        })
        return;
      }
    }

    if(!this.isBonLivraisonModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KNO_ARTICLE_ADDED'
      })

      return;
    }

    if(this.chosenCustomer === undefined || this.chosenCustomer === null) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KLIVRAISON_CLIENT_NOT_ASSIGNED'
      })

      return;
    }

    const documentFormData = this.bonDocumentForm.getRawValue();

    const bonArticles: BonLivraisonItem[] = [...this.bonLivraisonItems];
    const bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    const bonTotalAssurance = bonArticles.reduce((acc,b) => acc + (+b.qte * +b.assurance), 0);

    const bonStatus = type === 'validate' ? APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_YES : (this.bonLivraison?.status ? this.bonLivraison?.status : APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_NO);

    const bonCustomer: BonLivraisonCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonLivraison = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      shipping_date: (documentFormData.SHIPPING_DATE !== undefined && documentFormData.SHIPPING_DATE !== null && documentFormData.SHIPPING_DATE !== "" && documentFormData.SHIPPING_DATE.value !== "") ? moment(documentFormData.SHIPPING_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      pickup_date: (documentFormData.PICKUP_DATE !== undefined && documentFormData.PICKUP_DATE !== null && documentFormData.PICKUP_DATE !== "" && documentFormData.PICKUP_DATE.value !== "") ? moment(documentFormData.PICKUP_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      transport:documentFormData.TRANSPORT,
      service_type:documentFormData.SERVICE_TYPE,
      return_voucher:documentFormData.RETURN_VOUCHER,
      pret_material_status: documentFormData.PRET_MATERIEL ? APP_CONSTANTS.PRET_MATERIEL_STAT_YES : APP_CONSTANTS.PRET_MATERIEL_STAT_NO,
      sav_status: documentFormData.SAV ? APP_CONSTANTS.SAV_STAT_YES : APP_CONSTANTS.SAV_STAT_NO,

      customer: bonCustomer,
      articles: bonArticles,

      total_price: bonTotalPrix,
      total_assurance: bonTotalAssurance,
      status: bonStatus,
    };

    this._modifyLivraison(this.bonDocumentNum, bonDocument).then(
      () => {
        this.returnBack(type === 'validate' ? 'consultation' : 'list');
      }
    )
  }

  generateLivraisonPdf(print?: boolean) {
    if(!this.isBonLivraisonModifiable || (this.bonDocumentNum === undefined)) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KLIVRAISON_NO_CONTENT'
      })
      return;
    }

    this.loadingService.show();

    let lang = this.translateService.getBrowserLang();
    if (lang == null) {
      lang = APP_CONSTANTS.DEFAULT_LANG;
    }

    this.livraisonService.generatePdf(this.bonDocumentNum, lang).subscribe((res: any) => {
      if(res.status === 'success') {
        const documentPath = res.imageurl;
        const documentURL = documentPath + '/' + res.data.file;
        this.openUpPdfViewer(documentURL, print);
      }
    });
  }

  openUpPdfViewer(documentUrl: string, print?: boolean) {
    const dialogRef = this.dialog.open(PdfViewerComponent, {
      width: '50vw',
      maxWidth: '50vw',
      minHeight: '40vw',
      data: {
        documentUrl: documentUrl,
        title: 'KAFFICIER_BON_DE_LIVRAISON_DOCUMENT',
        printDocument: print
      }
    });

    dialogRef.afterClosed().subscribe((result: any) => {
      if(result && result.status === 'success') {

      }
    });
  }

  returnBack(type: 'list' | 'consultation' = 'list') {
    if(type === 'list') {
      this.router.navigate(['/bon-de-livraison/list']);
    } else if (type === 'consultation') {
      this.router.navigate(['/bon-de-livraison/detail', this.bonDocumentNum]);
    }
  }

  _getLigneList(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.commonService.getAllLigneList().subscribe({
        next: (res: CustomHttpResponse<LigneInfoList[]>) => {
          if(res.status === 'success') {
            this.ligneList = res.data;
          }

          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _getStockLocationList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.commonService.getAllStockList().subscribe({
        next: (res: CustomHttpResponse<StockLocationList[]>) => {
          if(res.status === 'success') {
            this.stockLocationList = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  _getLivraisonPrestation(): Promise<any> {
    return new Promise((resolve, reject) => {
      this.subscriptions?.push(this.livraisonPrestationService.getLivraisonPrestationList().subscribe({
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            this.livraisonPrestation = res.data;
          }
          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      }));
    });
  }

  _searchArticles(data: { keyword: string, location_id: string }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      let searchObs: Observable<CustomHttpResponse<SearchArticle>> | undefined;

      searchObs = this.commonService.searchArticlesListByRef(data);
      if(searchObs === undefined) {
        return;
      }

      searchObs.subscribe({
        next: (res: CustomHttpResponse<SearchArticle>) => {
          if(res.status === 'success') {
            this.fullKeywordMatch = res.data.fullMatch || false;
            this.isKeywordSerial = res.data.isSerial || false;
            this.serialStockLocation = res.data.serial_stock_location_id || '';
            this.searchRefArticlesList = res.data.rows;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  _getBonLivraisonDetail(document_no: string | undefined): Promise<any> {
    console.log(document_no)
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined || document_no === null || document_no === '') {
        reject();
        return;
      }

      this.livraisonService.getLivraisonDetail(document_no).subscribe( {
        next: (res: CustomHttpResponse<BonLivraison>) => {
          if(res.status === "success") {
            const bonLivraisonData = res.data;
            this.bonLivraison = bonLivraisonData;
            if(!this.rolePermission['2_16'] && this.bonLivraison.status === APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_YES) {
              this.dialogService.displayAlert({
                title: this.translateService.instant('KERROR'),
                message: this.translateService.instant('KACCESS_DENIED')
              }).then(() => {
                this.returnBack();
              });

              return;
            }
            this.__refreshBonDetails(bonLivraisonData);
          } else {
            // If the bon reception not available
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/bon-de-livraison/create']);
              }
            );
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __refreshBonDetails(bonLivraison: BonLivraison) {
    if(bonLivraison !== undefined && bonLivraison !== null) {
      this.bonDocumentNum = bonLivraison.document_no;
      this.bonLivraisonItems = bonLivraison.articles;
      this.bonLivraisonArticles = bonLivraison?.custom_articles ? bonLivraison?.custom_articles : [];

      this.chosenCustomer = bonLivraison.customer;

      this.totalArticles = 0;
      this.totalQte = 0;
      this.totalAssurance = 0;
      this.totalPT = 0;

      this.bonLivraisonItems.forEach((row) => {
        this.totalArticles += 1;
        this.totalQte += +row.qte;
        this.totalAssurance += +row.qte * +row.assurance;
        this.totalPT += +row.total_price;
      })

      this.bonDocumentForm.get("NUMBON")?.setValue(bonLivraison.document_no);
      this.bonDocumentForm.get("DATE_DU_BON")?.setValue(moment(bonLivraison.dated).format(CUSTOM_DATE_FORMAT));
      this.bonDocumentForm.get("SHIPPING_DATE")?.setValue((bonLivraison.shipping_date !== undefined && bonLivraison.shipping_date !== null) ? moment(bonLivraison.shipping_date) : '');
      this.bonDocumentForm.get("PICKUP_DATE")?.setValue((bonLivraison.pickup_date !== undefined && bonLivraison.pickup_date !== null) ? moment(bonLivraison.pickup_date) : '');
      this.bonDocumentForm.get("AXESITE")?.setValue(bonLivraison.axesite);
      this.bonDocumentForm.get("TRANSPORT")?.setValue(bonLivraison.transport);
      this.bonDocumentForm.get("SERVICE_TYPE")?.setValue(bonLivraison.service_type_id || '');
      this.bonDocumentForm.get("RETURN_VOUCHER")?.setValue(bonLivraison.return_voucher);
      this.bonDocumentForm.get("DOCUMENT_ASSOC")?.setValue(bonLivraison.docs_assoc);
      this.bonDocumentForm.get("PRET_MATERIEL")?.setValue(bonLivraison.pret_material_status == APP_CONSTANTS.PRET_MATERIEL_STAT_YES);
      this.bonDocumentForm.get("SAV")?.setValue(bonLivraison.sav_status == APP_CONSTANTS.SAV_STAT_YES);

      if(bonLivraison.ligne_lbl !== undefined && bonLivraison.ligne_lbl !== null && bonLivraison.ligne_lbl !== "") {
        if(bonLivraison.ligne_id === undefined || bonLivraison.ligne_id === null || bonLivraison.ligne_id === "") {
          // Custom Ligne Input
          this.customLigneInput = true;
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue('custom-ligne');
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonLivraison.ligne_lbl);
        } else {
          // Chosen Ligne Input from the dropdown
          const currentLigneItem = this.ligneList.find(r => r.LGIID == bonLivraison.ligne_id)
          this.bonDocumentForm.get("LIGNE_INFO")?.setValue(currentLigneItem);
          this.bonDocumentForm.get("LIGNE_CUSTOM_LABEL")?.setValue(bonLivraison.ligne_lbl);
        }
      }

      if(this.readOnly) {
        this.__changeToDetailMode();
      }
    }
  }

  __serialsListDisplay(serialsArr: string[]) {
    return serialsArr.join(", ");
  }

  ligneSelectOptionCompare = (option: any, value: any) => {
    return option.hasOwnProperty('LGIID') ? (option.LGIID == value?.LGIID) : (option === value);
  }

  get itemReturnStatus() {
    return this.bonLivraisonItemForm.get("RETURN_STATUS");
  }

  get documentAssocie() {
    return this.bonDocumentForm.get("DOCUMENT_ASSOC");
  }

  get isBonLivraisonModifiable() {
    return this.bonLivraison !== undefined;
  }

  onChangeReturnStatus() {
    const returnStatus = this.bonLivraisonItemForm.get('RETURN_STATUS')?.value;
    if(returnStatus === -1) {
      this.bonLivraisonItemForm.get('RETURN_DAYS')?.setValidators([Validators.required,Validators.min(1)])
      this.bonLivraisonItemForm.get('RETURN_DAYS')?.updateValueAndValidity();
    }else{
      this.bonLivraisonItemForm.get('RETURN_DAYS')?.clearValidators();
      this.bonLivraisonItemForm.get('RETURN_DAYS')?.updateValueAndValidity();
    }
  }

  get QTE_VALUE(){
    return this.bonLivraisonItemForm.get('QTE')?.value;
  }

  addBonLivraisonItemFormSubmitter(partialReset: boolean = false) {
    if(this.bonLivraisonItemForm.invalid || this.QTE_VALUE === '0') {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KMANDATORY_FIELDS'
      });
      return;
    }

    this.loadingService.show();
    const documentFormData = this.bonDocumentForm.getRawValue();
    const itemFormData = this.bonLivraisonItemForm.getRawValue();

    let bonArticles: BonLivraisonItem[] = [];
    let duplicationArticleFound: BonLivraisonItem | undefined = undefined;

    if(this.isBonLivraisonModifiable) {
      bonArticles = [...this.bonLivraisonItems];

      duplicationArticleFound = this.bonLivraisonItems.find((item) => {
        if (+item.article_id === +itemFormData.ARTICLE_ID && +item.stock_source === +itemFormData.STOCK_SOURCE_ID) {
          return true;
        }

        return false;
      });
    }

    const itemSerials = itemFormData.SELECTED_SERIALS_LIST.split(',');
    if(this.isBonLivraisonModifiable && duplicationArticleFound !== undefined && duplicationArticleFound !== null) {
      bonArticles = bonArticles.map((rowItem) => {
        if(rowItem.item_id === duplicationArticleFound?.item_id) {
          const quantity = rowItem.qte + (+itemFormData.QTE);
          const unitPrix = +itemFormData.PRIX_UNIT;
          const totalPrix = (quantity * unitPrix).toFixed(2);

          rowItem.qte = quantity;
          rowItem.qte_stock = +itemFormData.AVAIL_STOCK;
          rowItem.unit_price = +unitPrix;
          rowItem.total_price = +totalPrix;
          rowItem.return_status = itemFormData.RETURN_STATUS;
          rowItem.return_days = +itemFormData.RETURN_DAYS;
          rowItem.article_reference = itemFormData.REFERENCE;
          rowItem.article_desc = itemFormData.LABEL;
          rowItem.assurance = itemFormData.ASSURANCE;
          rowItem.material_activable_status = itemFormData.MATERIAL_ACTIVABLE_STATUS;
          rowItem.article_serials = [...rowItem.article_serials, ...itemSerials];
        }

        return rowItem;
      })
    } else {
      bonArticles.push({
        stock_source: +itemFormData.STOCK_SOURCE_ID,
        article_id: +itemFormData.ARTICLE_ID,
        article_reference: itemFormData.REFERENCE,
        article_desc: itemFormData.LABEL,
        assurance: itemFormData.ASSURANCE,
        qte: +itemFormData.QTE,
        qte_stock: +itemFormData.AVAIL_STOCK,
        unit_price: +itemFormData.PRIX_UNIT,
        total_price: +itemFormData.PRIX_TOTAL,
        return_status: itemFormData.RETURN_STATUS,
        return_days: +itemFormData.RETURN_DAYS,
        material_activable_status: itemFormData.MATERIAL_ACTIVABLE_STATUS,
        article_serials: itemSerials
      })
    }

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    const bonTotalAssurance = bonArticles.reduce((acc,b) => acc + (+b.qte * +b.assurance), 0);
    let bonStatus = APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_NO;

    if(this.isBonLivraisonModifiable) {
      bonStatus = this.bonLivraison?.status ? this.bonLivraison?.status : APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_NO;
    }

    const bonCustomer: BonLivraisonCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonLivraison = {
      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      shipping_date: (documentFormData.SHIPPING_DATE !== undefined && documentFormData.SHIPPING_DATE !== null && documentFormData.SHIPPING_DATE !== "" && documentFormData.SHIPPING_DATE.value !== "") ? moment(documentFormData.SHIPPING_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      pickup_date: (documentFormData.PICKUP_DATE !== undefined && documentFormData.PICKUP_DATE !== null && documentFormData.PICKUP_DATE !== "" && documentFormData.PICKUP_DATE.value !== "") ? moment(documentFormData.PICKUP_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      transport: documentFormData.TRANSPORT,
      service_type: documentFormData.SERVICE_TYPE,
      return_voucher: documentFormData.RETURN_VOUCHER,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      pret_material_status: documentFormData.PRET_MATERIEL ? APP_CONSTANTS.PRET_MATERIEL_STAT_YES : APP_CONSTANTS.PRET_MATERIEL_STAT_NO,
      sav_status: documentFormData.SAV ? APP_CONSTANTS.SAV_STAT_YES : APP_CONSTANTS.SAV_STAT_NO,

      customer: bonCustomer,
      articles: bonArticles,

      total_price: bonTotalPrix,
      total_assurance: bonTotalAssurance,
      status: bonStatus,
    };

    console.log('payload: ', bonDocument);
    this._modifyLivraison(this.bonDocumentNum, bonDocument).then(
      () => {
        this.__resetNewLineItemForm(partialReset);
      }
    );
  }

  removeLineItem(currentItem: BonLivraisonItem) {
    if(this.readOnly) {
      return;
    }

    this.loadingService.show();

    let bonArticles: BonLivraisonItem[] = [...this.bonLivraisonItems];
    bonArticles = bonArticles.filter((row) => row.item_id !== currentItem.item_id);

    const documentFormData = this.bonDocumentForm.getRawValue();

    let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
    const bonTotalAssurance = bonArticles.reduce((acc,b) => acc + (+b.qte * +b.assurance), 0);

    const bonStatus = this.bonLivraison?.status ? this.bonLivraison?.status : APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_NO;

    const bonCustomer: BonLivraisonCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : "",
    };

    let bonDocument: BonLivraison = {
      ...this.bonLivraison,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      shipping_date: (documentFormData.SHIPPING_DATE !== undefined && documentFormData.SHIPPING_DATE !== null && documentFormData.SHIPPING_DATE !== "" && documentFormData.SHIPPING_DATE.value !== "") ? moment(documentFormData.SHIPPING_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      pickup_date: (documentFormData.PICKUP_DATE !== undefined && documentFormData.PICKUP_DATE !== null && documentFormData.PICKUP_DATE !== "" && documentFormData.PICKUP_DATE.value !== "") ? moment(documentFormData.PICKUP_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      customer: bonCustomer,
      axesite:documentFormData.AXESITE,
      transport:documentFormData.TRANSPORT,
      service_type:documentFormData.SERVICE_TYPE,
      return_voucher:documentFormData.RETURN_VOUCHER,
      pret_material_status: documentFormData.PRET_MATERIEL ? APP_CONSTANTS.PRET_MATERIEL_STAT_YES : APP_CONSTANTS.PRET_MATERIEL_STAT_NO,
      sav_status: documentFormData.SAV ? APP_CONSTANTS.SAV_STAT_YES : APP_CONSTANTS.SAV_STAT_NO,
      articles: bonArticles,
      total_price: bonTotalPrix,
      total_assurance: bonTotalAssurance,
      status: bonStatus
    };

    delete bonDocument.livraison_id;

    this._modifyLivraison(this.bonDocumentNum, bonDocument);
  }

  _modifyLivraison(document_no: string | undefined, bonDocument: BonLivraison): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(document_no === undefined) {
        reject("INVALID_DOCUMENT_NO");
        return;
      }

      this.livraisonService.modifyLivrasion(document_no, bonDocument).subscribe( {
        next: (res: CustomHttpResponse<BonLivraison>) => {
          this.loadingService.hide();
          if(res.status === "success") {
            const bonLivraisonData = res.data;
            this.bonLivraison = bonLivraisonData;
            this.__updateListCache(bonLivraisonData);
            this.__refreshBonDetails(bonLivraisonData);
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: err => {
          this.loadingService.hide();
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      });
    })
  }

  __resetNewLineItemForm(keepStockLocation: boolean = false) {
    this.newArticleSelection = undefined;
    this.bonLivraisonItemForm.patchValue({
      STOCK_SOURCE_ID: "",
      STOCK_SOURCE_LBL: "",
      ARTICLE_ID: "",
      REFERENCE: "",
      LABEL: "",
      QTE: "0",
      PRIX_UNIT: "",
      PRIX_TOTAL: "0",
      AVAIL_STOCK: "",
      RETURN_STATUS: "0",
      RETURN_DAYS: "",
      MATERIAL_ACTIVABLE_STATUS: null,
      SERIALS_LIST: "",
      SELECTED_SERIALS_LIST:"",
    });
    this.bonLivraisonItemForm.updateValueAndValidity();

    this.searchRefArticlesList = undefined;

    if(keepStockLocation) {
      this.bonDocumentForm.patchValue({
        SEARCH_REF: "",
      });

      this.rechercheRefInputEl?.nativeElement.focus();
    } else {
      this.bonDocumentForm.patchValue({
        SEARCH_STOCK_LOCATION: "",
        SEARCH_REF: "",
      });
    }

    this.bonDocumentForm.updateValueAndValidity();
  }

  onStockLocationChange(event: any) {
    this.stockLocationInputEl?.close();

    if(this.newArticleSelection) {
      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    } else {
      // Changing the Stock location at first
      this.rechercheRefInputEl?.nativeElement.focus();

      this.searchRefArticlesList = undefined;

      // Changing the Stock location when already an article selected
      const referenceValue = this.bonDocumentForm.get("SEARCH_REF")?.value;

      if(referenceValue !== '') {
        this.searchArticles(undefined).then(
          (res) => {
            const currentArticleMatch = this.searchRefArticlesList?.find((r) => r.id === this.newArticleSelection?.id);
            if(currentArticleMatch) {
              this.newArticleSelection = currentArticleMatch;

              this.__fillNewLineItemForm('existing');
            }
          }
        );
      }
    }
  }

  async searchArticles(event: any | undefined): Promise<any> {
    return new Promise(async (resolve, reject) => {
      if(event !== undefined && (event.keyCode == 38 || event.keyCode == 40 || event.keyCode == 13)) {
        return;
      }

      const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;

      let payload = {
        location_id: '',
        keyword: ''
      };

      if(searchStockLocation !== undefined && searchStockLocation !== null && searchStockLocation !== '') {
        payload.location_id = searchStockLocation;
      }

      const searchRefKeyword = this.bonDocumentForm.get("SEARCH_REF")?.value;
      payload = {
        ...payload,
        keyword: searchRefKeyword
      };

      const searchResult = await this._searchArticles(payload);

      resolve(searchResult);
    })
  }

  async onScannerSearchArticle() {
    const searchRef = this.bonDocumentForm.get('SEARCH_REF')?.value;

    if(searchRef == undefined || searchRef == "") {
      return;
    }

    await this.searchArticles(searchRef);

    if(this.searchRefArticlesList === undefined || this.searchRefArticlesList === null) {
      return;
    }

    if(this.searchRefArticlesList.length > 0 && this.fullKeywordMatch) {
      if(this.isKeywordSerial) {
        const serialNo = this.bonDocumentForm.get("SEARCH_REF")?.value;

        if(this.serialStockLocation !== "") {
          this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.setValue(this.serialStockLocation);
        }

        const firstArticle = this.searchRefArticlesList[0];

        const blockedOutSerials = firstArticle.blocked_out_serials || "";
        const formattedBlockedSerials = this.commonService.formatBlockedSerialNumbers(blockedOutSerials);
        const blockedSerialExists = formattedBlockedSerials.find((row) => row.serial_no === serialNo.toUpperCase());

        // Check whether the entered S/N is already blocked or not
        if (blockedSerialExists !== undefined && blockedSerialExists !== null) {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KSN_ALREADY_BLOCKED',
            messageTranslateParam: { DOCUMENTS: `${blockedSerialExists.document_no} (${blockedSerialExists.serial_no})` }
          });

          return;
        }

        this.bonLivraisonItemForm.get('SELECTED_SERIALS_LIST')?.setValue(serialNo.toUpperCase());
        this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));

        this.searchSelected(firstArticle, true, true);
      } else {
        const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
        if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
          this.searchRefArticlesList = undefined;

          this.stockLocationInputEl?.focus();
          this.stockLocationInputEl?.open();
          return;
        }

        const firstArticle = this.searchRefArticlesList[0];
        this.searchSelected(firstArticle, true);
      }
    } else {
      const searchStockLocation = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
      if(searchStockLocation === undefined || searchStockLocation === null || searchStockLocation === '') {
        this.searchRefArticlesList = undefined;

        this.stockLocationInputEl?.focus();
        this.stockLocationInputEl?.open();
        return;
      }

      this.openSearchArticlePopup();
    }
  }

  async openSearchArticlePopup() {
    const searchKeyword = this.bonDocumentForm.get('SEARCH_REF')?.value;
    const stockLocation = this.bonDocumentForm.get('SEARCH_STOCK_LOCATION')?.value;
    const data = {
      currentArticle: this.newArticleSelection,
      stockLocation: stockLocation,
      searchKeyword: searchKeyword
    };

    const dialog = await this.dialog.open(SearchArticleComponent, {
      data,
      maxHeight: "90vh"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        this.newArticleSelection = res.data;

        this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);
        this.bonLivraisonItemForm.get("SERIALS_LIST")?.setValue(this.newArticleSelection?.serial_list);
        console.log(this.bonLivraisonItemForm.getRawValue());

        this.__fillNewLineItemForm('new');
        this.onChangeReturnStatus();
        console.log('SEARCH RES: ', res);

        this.changeQuantity('creation');
      }
    });
  }

  searchSelected(event: any, directCall: boolean = false, directInsert: boolean = false) {
    if(directCall) {
      this.newArticleSelection = event;
    } else {
      this.newArticleSelection = event.option.value;
    }

    this.bonDocumentForm.get("SEARCH_REF")?.setValue(this.newArticleSelection?.reference);

    this.bonLivraisonItemForm.get("SERIALS_LIST")?.setValue(this.newArticleSelection?.serial_list);
    console.log(this.bonLivraisonItemForm.getRawValue());

    this.__fillNewLineItemForm('new');
    this.onChangeReturnStatus();

    if(directInsert) {
      this.addBonLivraisonItemFormSubmitter(true);
      return;
    }

    this.changeQuantity('creation');
  }

  __fillNewLineItemForm(type: 'new' | 'existing') {
    const stockLocationId = this.bonDocumentForm.get("SEARCH_STOCK_LOCATION")?.value;
    const stockLocationDetail = this.stockLocationList.find((_r) => _r.STLID == stockLocationId);

    let unitPrix = 0;
    if(this.newArticleSelection?.public_prix !== undefined && this.newArticleSelection?.public_prix !== null) {
      unitPrix = this.newArticleSelection?.public_prix;
    }

    this.bonLivraisonItemForm.patchValue({
      STOCK_SOURCE_ID: stockLocationDetail?.STLID,
      STOCK_SOURCE_LBL: stockLocationDetail?.STLLBL,
      ARTICLE_ID: this.newArticleSelection?.id,
      REFERENCE: this.newArticleSelection?.reference,
      LABEL: this.newArticleSelection?.label,
      AVAIL_STOCK: this.newArticleSelection?.avail_stock === null ? "0" : (+this.newArticleSelection?.avail_stock).toString(),
      ASSURANCE: this.newArticleSelection?.assurance
    });

    if(type === "new") {
      const itemSelectedSerials = this.bonLivraisonItemForm.get('SELECTED_SERIALS_LIST')?.value;
      const selectedListSerials = itemSelectedSerials ? (typeof itemSelectedSerials === 'string' ? itemSelectedSerials.split(",") : itemSelectedSerials):[];

      this.bonLivraisonItemForm.patchValue({
        QTE: selectedListSerials.length.toString(),
        PRIX_UNIT: unitPrix.toString(),
        PRIX_TOTAL: "0",
        RETURN_STATUS: "0",
        RETURN_DAYS: "",
        SERIALS_LIST: this.bonLivraisonItemForm.get("SERIALS_LIST")?.value
      });
    }
  }

  changeQuantity(type: 'creation' | 'modification') {
    const itemSelectedSerials = this.bonLivraisonItemForm.get('SELECTED_SERIALS_LIST')?.value;
    const selectedListSerials = itemSelectedSerials ? (typeof itemSelectedSerials === 'string' ? itemSelectedSerials.split(",") : itemSelectedSerials):[];

    const itemSerials = this.bonLivraisonItemForm.get("SERIALS_LIST")?.value;
    let oldListSerials = this.commonService.formatSerialNumbers(itemSerials ? (typeof itemSerials === 'string'? itemSerials.split(","): itemSerials) :[]);

    const blockedInSerials = this.newArticleSelection?.blocked_in_serials;
    let blockedOutSerials = this.newArticleSelection?.blocked_out_serials;

    if(this.__isDocumentValidated) {
      const documentItems = cloneDeep(this.bonLivraisonItems);
      const documentSerials = documentItems.map(r => {
        return r.article_serials; // ['SN1', 'SN2']
      }).reduce((acc, current) => {
        current = current.map(row => `${row}${APP_CONSTANTS.BLOCK_SERIALS_SEPARATOR}${this.bonDocumentNum}${APP_CONSTANTS.BLOCK_SERIALS_SEPARATOR}BL`)
        acc = [...acc, ...current];

        return acc;
      }, []).join(",");

      if(blockedOutSerials !== undefined && blockedOutSerials !== null && blockedOutSerials !== "") {
        blockedOutSerials += "," + documentSerials;
      } else {
        blockedOutSerials = documentSerials;
      }
    }

    this.openUpSerialNumPopup({
      oldListSerials,
      selectedSerials: selectedListSerials,
      block_in_serials: blockedInSerials,
      block_out_serials: blockedOutSerials
    });
  }

  viewItemSerials(item: BonLivraisonItem) {
    console.log("item",item);
    const itemSelectedSerials = item.article_serials;
    const selectedListSerials = itemSelectedSerials ? itemSelectedSerials : [];

    // Sending Selected Serials and Old List Serials as same array to display the newly selected Items only.
    let oldListSerials = this.commonService.formatSerialNumbers([...selectedListSerials]);

    this.openUpSerialNumPopup({
      oldListSerials,
      selectedSerials: selectedListSerials,
      readonly: true
    });
  }

  openUpSerialNumPopup(data?: any) {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType:'livraison-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          if(data?.readonly && data?.readonly === true) {
            return;
          }

          this.bonLivraisonItemForm.get('QTE')?.setValue(result.data.length.toString());
          this.bonLivraisonItemForm.get('SELECTED_SERIALS_LIST')?.setValue(result.data.join(','));
          this.unitPrixInputEl?.nativeElement.select();
          this.unitPrixInputEl?.nativeElement.focus();
          this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));
        }
      }
    });
  }

  createExpedition() {
    if(!this.isBonLivraisonModifiable) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KLIVRAISON_NOT_READY_EXPEDITION'
      });

      return;
    }

    const data = {
      document_no: this.bonDocumentNum,
      document: this.bonLivraison,
    };

    this.openUpExpeditionPopup(data);
  }

  openUpExpeditionPopup(data?: any) {
    const dialogRef = this.dialog.open(SaisieExpeditionComponent,
      {
        minWidth: '55vw',
        maxWidth: '65vw',
        maxHeight: '90vh',
        data
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        if(result.data && result.data !== '') {
          let pickup_date:string;
          pickup_date = result.data.modified_at != null? result.data.modified_at : result.data.created_at;
          this.bonDocumentForm.get('PICKUP_DATE')?.setValue(moment(pickup_date).format(CUSTOM_DATE_API_FORMAT));
        }
      }
    });
  }

  confirmResetNewArticle(): Promise<any> {
    const message = `<h3>${this.translateService.instant('KCONFIRM_DISCARD_NEW_ITEM')}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async modifyLineItem(item: BonLivraisonItem) {
    if(this.readOnly) {
      return;
    }

    if(this.newArticleSelection !== undefined) {
      const confirmation = await this.confirmResetNewArticle();
      if(!confirmation) {
        return;
      }

      this.__resetNewLineItemForm();
    }

    const data = {
      item: item,
      document_no: this.bonDocumentNum,
      document: this.bonLivraison,
    };

    this.openUpModifyItemPopup(data);
  }

  openUpModifyItemPopup(data?: {}) {
    const dialogRef = this.dialog.open(ModifyItemComponent,
      {
        maxWidth: '35vw',
        data: {
          numeroType:'livraison-numero-series',
          ...data
        }
      }
    );

    dialogRef.afterClosed().subscribe(result => {
      if(result && result.status === "success") {
        // After popup validation
        console.log(result.data);
        if(result.data && result.data !== '') {
          this.loadingService.show();
          const bonDocument = result.data;
          this._modifyLivraison(this.bonDocumentNum, bonDocument);
        }
      }
    });
  }

  onBonDocumentAssocieChange(documentAss: string) {
    if(this.documentAssocie?.value !== documentAss) {
      this.documentAssocie?.setValue(documentAss);
    }
  }

  onBonCustomerChange(customer: BonLivraisonCustomer) {
    if(this.chosenCustomer !== customer) {
      this.chosenCustomer = customer;

      if(this.isBonLivraisonModifiable) {
        this._updateBonCustomer();
      }
    }
  }

  _updateBonCustomer() {
    const bonCustomer: BonLivraisonCustomer = {
      store_id: this.chosenCustomer ? this.chosenCustomer.store_id : "",
      store_nom: this.chosenCustomer ? this.chosenCustomer.store_nom : "",
      store_type: this.chosenCustomer ? this.chosenCustomer.store_type : ""
    };

    const documentFormData = this.bonDocumentForm.getRawValue();
    const bonStatus = this.bonLivraison?.status ? this.bonLivraison?.status : APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_NO;
    const ligneInfo = documentFormData.LIGNE_INFO;

    const bonDocument: BonLivraison = {
      ...this.bonLivraison,

      dated: moment(documentFormData.DATE_DU_BON, CUSTOM_DATE_TIME_FORMAT).format(CUSTOM_DATE_TIME_API_FORMAT),
      shipping_date: (documentFormData.SHIPPING_DATE !== undefined && documentFormData.SHIPPING_DATE !== null && documentFormData.SHIPPING_DATE !== "" && documentFormData.SHIPPING_DATE.value !== "") ? moment(documentFormData.SHIPPING_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      pickup_date: (documentFormData.PICKUP_DATE !== undefined && documentFormData.PICKUP_DATE !== null && documentFormData.PICKUP_DATE !== "" && documentFormData.PICKUP_DATE.value !== "") ? moment(documentFormData.PICKUP_DATE, CUSTOM_DATE_FORMAT).format(CUSTOM_DATE_API_FORMAT) : '',
      ligne_id: ligneInfo === 'custom-ligne' ? "" : (ligneInfo ? ligneInfo.LGIID : ""),
      ligne_lbl: ligneInfo === 'custom-ligne' ? documentFormData.LIGNE_CUSTOM_LABEL : (ligneInfo ? ligneInfo.LGILBL : ""),
      axesite: documentFormData.AXESITE,
      docs_assoc: documentFormData.DOCUMENT_ASSOC,
      transport:documentFormData.TRANSPORT,
      service_type:documentFormData.SERVICE_TYPE,
      return_voucher:documentFormData.RETURN_VOUCHER,
      pret_material_status: documentFormData.PRET_MATERIEL ? APP_CONSTANTS.PRET_MATERIEL_STAT_YES : APP_CONSTANTS.PRET_MATERIEL_STAT_NO,
      sav_status: documentFormData.SAV ? APP_CONSTANTS.SAV_STAT_YES : APP_CONSTANTS.SAV_STAT_NO,
      customer: bonCustomer,
      articles: this.bonLivraisonItems,

      total_price: this.bonLivraison?.total_price ? this.bonLivraison?.total_price : 0,
      total_assurance: this.bonLivraison?.total_assurance ? this.bonLivraison?.total_assurance : 0,
      status: bonStatus,
    };

    this._modifyLivraison(this.bonDocumentNum, bonDocument);
  }

  calculatePrixTotal(){
    const formData = this.bonLivraisonItemForm.getRawValue();

    let totalPrix = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.bonLivraisonItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.bonLivraisonItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    this.bonLivraisonItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
  }

  openHistorique(){
    const formData = this.bonDocumentForm.getRawValue();
    const data = {
      document_no: formData.NUMBON,
      document_type: APP_CONSTANTS.DOCUMENT_TYPE_LIVRAISON
    };
    this.dialog.open(HistoriqueComponent,
      {
        minWidth: '80%',
        maxHeight: '90vh',
        data: data || {}
      }
    );
  }

  generateBonDeReception() {
    if(!this.readOnly) {
      return;
    }

    this.router.navigate(['/bon-de-reception/create'], { queryParams: { _doc: this.bonDocumentNum, _doc_t: 'BL' } });
  }

  confirmRemoveLivraison(document_no: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: document_no })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeLivraison() {
    if(this.bonDocumentNum == undefined){
      return
    }
    console.log(this.bonDocumentNum)
    if((this.__isDocumentValidated && !this.rolePermission['2_17']) || (!this.__isDocumentValidated && !this.rolePermission['2_14'])) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveLivraison(this.bonDocumentNum);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteLivraison(this.bonDocumentNum);
    }
  }

  async _deleteLivraison(document_no: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__deleteUpdateListCache(document_no);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/bon-de-livraison/list']);
          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();

        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    };

    this.livraisonService.deleteLivraison(document_no).subscribe(observer);
  }

}
