import {Component, ElementRef, Inject, OnInit, ViewChild} from '@angular/core';
import {FormBuilder, Validators} from "@angular/forms";
import {NumeroSerieComponent} from "../../inventory/numero-serie/numero-serie.component";
import {MAT_DIALOG_DATA, MatDialog, MatDialogRef} from "@angular/material/dialog";
import {
  BonAchatCommande,
  BonAchatCommandeItem,
  BonReception,
  BonReceptionItem, BonTransferDeStock, BonTransferDeStockItem, BrandInfoList,
  BrandPrixVente,
  CustomHttpResponse, StoreInfoList
} from "../../../_models";
import {DialogService} from "../../../_services/dailog.service";
import {BonDeReceptionService} from "../../../_services/bon-de-reception.service";
import { BonDeLivraisonService } from '../../../_services/bon-de-livraison.service';
import { CommonService } from '../../../_services/common.service';
import { BonLivraison, BonLivraisonItem } from 'src/app/_models/livraison';
import {BonInventory, BonInventoryItem} from 'src/app/_models/inventory';
import {Article} from "../../../_models/article";
import {NgxSpinnerService} from "ngx-spinner";
import {AuthService} from "../../../_services/auth.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {lastValueFrom} from "rxjs";
import {ValidateGreaterThan} from "../../../_helpers/greater-than-validation";
import {cloneDeep} from "lodash";
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-modify-item',
  templateUrl: './modify-item.component.html',
  styleUrls: ['./modify-item.component.scss']
})
export class ModifyItemComponent implements OnInit {

  @ViewChild('prixUnitInput') unitPrixInputEl: ElementRef | undefined;
  @ViewChild('quantiteInput') quantiteInputEl: ElementRef | undefined;

  modifyItemForm = this.formBuilder.group({
    STOCK_SOURCE_ID: ["", Validators.required],
    STOCK_SOURCE_LBL: [{ value: "", disabled: true }, Validators.required],

    ITEM_SUPPLIER_BRAND: [''], // For Achat/Commande
    ITEM_CLIENT_BRAND: [''], // For Achat/Commande
    ITEM_CLIENT_STORE: [''], // For Achat/Commande
    ITEM_STATUS: [APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING], // For Achat/Commande

    ACHAT_ITEM_ID: [""], // For Reception documents generated from Achat/Commande
    ARTICLE_ID: ["", Validators.required],
    REFERENCE: [{ value: "", disabled: true }, Validators.required],
    LABEL: [{ value: "", disabled: true }, Validators.required],
    QTE: [{ value: "0", disabled: true }, [Validators.required, Validators.min(1)]],
    QTE_ORIGIN: [''], // For Reception documents generated from Achat/Commande
    QTE_RECEIVED: [{ value: "0", disabled: true }], // For Achat/Commande and Reception documents generated from Achat/Commande
    QTE_REMAINING: [''], // For Reception documents generated from Achat/Commande
    PRIX_UNIT: ['', [Validators.required, ValidateGreaterThan]],
    PRIX_TOTAL: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    PRIX_DE_VENTE: ['', [Validators.required, ValidateGreaterThan]],
    TOTAL_PRIX_DE_VENTE: [{ value: "0", disabled: true }, [Validators.required, ValidateGreaterThan]],
    FRAISDEPORT: [],
    AVAIL_STOCK: [{ value: "0", disabled: true }],
    RETURN_STATUS: [0],
    RETURN_DAYS: [''],
    ITEM_BRAND: [''],
    SERIALS_LIST: ['', Validators.required],
    ASSURANCE:[null, [Validators.minLength(APP_CONSTANTS.ASSURANCE_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.ASSURANCE_MAX_LENGTH)]],
    REMOVED_SERIALS_LIST: [''], // For Inventory
    OLD_SERIALS_LIST: [''], // For Inventory

    SELECTED_SERIALS_LIST:[''],
    SELECTED_SERIALS_LIST_BACKUP:[''],
  });

  bonDocumentNumBon: string = "";
  bonDocument: BonReception | any | BonInventory | BonLivraison | BonTransferDeStock;
  currentItem: BonReceptionItem | any | BonInventoryItem | BonLivraisonItem | BonTransferDeStockItem | BonAchatCommandeItem;
  currentArticleDetail: Article | undefined;
  numeroType: any;
  bonItemSerials: any =[];

  brandPrixVente: BrandPrixVente | undefined;

  clientBrandList: BrandInfoList[] = [];
  supplierBrandList: BrandInfoList[] = [];
  storeList: StoreInfoList[] = [];

  brandList: BrandInfoList[] = [];

  newSerialLists:any=[];
  document_stock_location: any;

  achatStatusList = APP_CONSTANTS.ACHAT_ITEM_STATUS;

  // Input Mask
  integerMask = APP_CONSTANTS.INTEGER_MASK;
  decimalMask = APP_CONSTANTS.DECIMAL_MASK;

  constructor(@Inject(MAT_DIALOG_DATA) public data: any,
              public dialogRef: MatDialogRef<ModifyItemComponent>,
              private formBuilder: FormBuilder,
              private dialog: MatDialog,
              private dialogService: DialogService,
              private authService: AuthService,
              private receptionService: BonDeReceptionService,
              private bonDeLivraisonService:BonDeLivraisonService,
              private commonService:CommonService,
              private loadingService: NgxSpinnerService,
              private translateService: TranslateService) { }

  ngOnInit(): void {
    this.numeroType = this.data.numeroType;
    this.document_stock_location = this.data.document.document_stock_location;

    if(this.data !== undefined && this.data !== null) {
      if(this.data.hasOwnProperty('item')) {
        this.currentItem = this.data.item;
        console.log(this.currentItem);
      }

      if(this.data.hasOwnProperty('document_no')) {
        this.bonDocumentNumBon = this.data.document_no;
      }

      if(this.data.hasOwnProperty('document')) {
        this.bonDocument = cloneDeep(this.data.document);
        console.log(this.bonDocument);
      }
    }

    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    let article_id, location_id;

    if(this.numeroType === 'inventory-numero-series') {
      article_id = this.currentItem.article_id;
      location_id = this.bonDocument.stock_location_id;
    }

    if(this.numeroType === 'transfer-de-stock-numero-series') {
      article_id = this.currentItem.article_id;
      location_id = (this.bonDocument as BonTransferDeStock).stock_source_location_id;
    }

    if(this.numeroType !== 'achat-commande-numero-series') {
      this.modifyItemForm.get('PRIX_DE_VENTE')?.clearValidators();
      this.modifyItemForm.get('PRIX_DE_VENTE')?.updateValueAndValidity();
    }

    if (this.numeroType === 'livraison-numero-series' || this.numeroType === 'reception-numero-series' || this.numeroType === 'achat-commande-numero-series') {
      article_id = this.currentItem.article_id;
      location_id = this.currentItem.stock_source;

      // For Achat/Commande module, we don't need to send the Location to fetch the Article detail
      if (this.numeroType === 'achat-commande-numero-series') {
        location_id = null;

        // Set Validation for the Qte field as per the Received Qte.
        const QTE_INPUT = this.modifyItemForm.get('QTE');

        let minQte = 1;
        if(this.currentItem.qte_received !== undefined && this.currentItem.qte_received !== null && this.currentItem.qte_received > 0) {
          minQte = this.currentItem.qte_received;
        }

        QTE_INPUT?.setValidators([Validators.required, Validators.min(minQte)]);
        QTE_INPUT?.updateValueAndValidity();

        // Filter the Achat Item Status List
        this.achatStatusList = this.achatStatusList.filter((row) => {
          if(this.currentItem.item_status == APP_CONSTANTS.ACHAT_ITEM_STATUS_VALIDATED) {
            return row.value === APP_CONSTANTS.ACHAT_ITEM_STATUS_VALIDATED;
          }

          if(this.currentItem.item_status == APP_CONSTANTS.ACHAT_ITEM_STATUS_CANCELLED) {
            return row.value === APP_CONSTANTS.ACHAT_ITEM_STATUS_CANCELLED;
          }

          if(this.currentItem.qte_received == 0 && row.value == APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING) {
            return true;
          }

          if(this.currentItem.qte_received > 0 && row.value == APP_CONSTANTS.ACHAT_ITEM_STATUS_INPROGRESS) {
            return true;
          }

          return false;
        });
      }
    }

    await this._getArticleDetail(article_id, location_id);

    if (this.numeroType === 'achat-commande-numero-series' || this.numeroType === 'reception-numero-series') {
      await this._getBrandList();
    }

    this.bonItemSerials = this.currentArticleDetail?.serial_list?.split(',');

    this.__refreshBonItemDetails(this.currentItem);

    await this.loadingService.hide();
  }

  achatItemStatus(status: number) {
    if(this.numeroType === 'achat-commande-numero-series') {
      const statusData = this.achatStatusList.find((r) => r.value == status);

      return statusData ? statusData.label : "";
    }

    return "";
  }

  achatItemStatusIcon(status: number) {
    if(this.numeroType === 'achat-commande-numero-series') {
      const statusData = this.achatStatusList.find((r) => r.value == status);
      let icon = "";
      switch (statusData?.value) {
        case APP_CONSTANTS.ACHAT_ITEM_STATUS_PENDING:
          icon = "pending_actions";
          break;
        case APP_CONSTANTS.ACHAT_ITEM_STATUS_INPROGRESS:
          icon = "watch_later";
          break;
        case APP_CONSTANTS.ACHAT_ITEM_STATUS_VALIDATED:
          icon = "task_alt";
          break;
        case APP_CONSTANTS.ACHAT_ITEM_STATUS_CANCELLED:
          icon = "remove_shopping_cart";
          break;
      }

      return icon;
    }

    return "";
  }

  get fr() {
    return this.modifyItemForm.controls;
  }

  get itemReturnStatus() {
    return this.modifyItemForm.get("RETURN_STATUS");
  }

  onBrandChange(clearValue: boolean = true) {
    const brand_id = this.modifyItemForm.get('ITEM_CLIENT_BRAND')?.value;

    if(clearValue) {
      this.modifyItemForm.get('ITEM_CLIENT_STORE')?.setValue('');
    }

    if(brand_id === undefined || brand_id === null || brand_id === '') {
      this.storeList = [];
      return;
    }

    const payload = {
      type: 'client',
      brand_id
    };

    this._getStoreList(payload);
  }

  checkPrixdeventePrice(event: any): Promise<any>{
    const brand_id = event.value;
    const article_id = this.modifyItemForm.get("ARTICLE_ID")?.value;
    let payload = {
      brand_id : brand_id,
      article_id: article_id
    }
    return new Promise((resolve, reject) => {
      this.commonService.getPrixDeVenteBasedOnBrand(payload).subscribe({
        next: (res: CustomHttpResponse<BrandPrixVente>) => {
          if(res.status === 'success') {
            this.brandPrixVente = res.data;
            this.modifyItemForm.get("PRIX_DE_VENTE")?.setValue(this.brandPrixVente.prix_de_vente);
            this.calculatePrixTotal();
          }else {
            this.modifyItemForm.get("PRIX_DE_VENTE")?.setValue(this.currentArticleDetail?.article_prix_de_vente.toString());
            this.calculatePrixTotal();
          }
          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
            // Filter only Client Brands
            this.clientBrandList = this.brandList.filter((row) => row.BRTYPE === 'client');

            // Filter only Supplier Brands
            this.supplierBrandList = this.brandList.filter((row) => row.BRTYPE === 'supplier');
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _getStoreList(payload: { type: string, brand_id: number }): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StoreInfoList[]>) => {
          if(res.status === 'success') {
            this.storeList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStores(payload).subscribe(observer);
    });
  }

  _getArticleDetail(article_id: number, location_id: number): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      this.commonService.getArticleDetail(article_id, location_id).subscribe({
        next: (res: CustomHttpResponse<Article>) => {
          if(res.status === 'success') {
            this.currentArticleDetail = res.data;
          }

          resolve(res);
        },
        error: err => {
          this.loadingService.hide();
          reject(err);
        }
      });
    })
  }

  __refreshBonItemDetails(item: any) {

    if(item !== undefined && item !== null) {
      if(this.numeroType === 'inventory-numero-series') {
        const currentitem: BonInventoryItem = item;
        const STOCK_SOURCE_ID =  this.modifyItemForm.get('STOCK_SOURCE_ID');
        STOCK_SOURCE_ID?.clearValidators();

        this.modifyItemForm.patchValue({
          STOCK_SOURCE_ID: this.bonDocument.stock_location_id,
          STOCK_SOURCE_LBL: this.bonDocument.stock_location_lbl,

          ARTICLE_ID: currentitem.article_id,
          REFERENCE: currentitem.article_reference,
          LABEL: currentitem.article_desc,
          QTE: currentitem.qte.toString(),
          PRIX_UNIT: currentitem.unit_price.toString(),
          PRIX_TOTAL: currentitem.total_price.toString(),
          AVAIL_STOCK: this.currentArticleDetail?.avail_stock?.toString(),

          SERIALS_LIST: currentitem.article_serials.join(','),
          OLD_SERIALS_LIST: this.currentArticleDetail?.serial_list,
          REMOVED_SERIALS_LIST: currentitem.removed_serials ? currentitem.removed_serials.join(',') : ""
        });
      } else if(this.numeroType === 'transfer-de-stock-numero-series') {
        const currentitem: BonTransferDeStockItem = item;

        const STOCK_SOURCE_ID =  this.modifyItemForm.get('STOCK_SOURCE_ID');
        STOCK_SOURCE_ID?.clearValidators();

        this.modifyItemForm.patchValue({
          STOCK_SOURCE_ID: (this.bonDocument as BonTransferDeStock).stock_source_location_id,
          STOCK_SOURCE_LBL: (this.bonDocument as BonTransferDeStock).stock_source_location_lbl,

          ARTICLE_ID: currentitem.article_id,
          REFERENCE: currentitem.article_reference,
          LABEL: currentitem.article_desc,
          QTE: currentitem.qte.toString(),
          PRIX_UNIT: currentitem.unit_price.toString(),
          PRIX_TOTAL: currentitem.total_price.toString(),
          AVAIL_STOCK: this.currentArticleDetail?.avail_stock?.toString(),

          SERIALS_LIST: this.currentArticleDetail?.serial_list
        });
      } else if(this.numeroType === 'achat-commande-numero-series') {
        const QTE_INPUT = this.modifyItemForm.get('QTE');
        QTE_INPUT?.enable({ onlySelf: true });

        const SERIALS_LIST = this.modifyItemForm.get('SERIALS_LIST');
        SERIALS_LIST?.clearValidators();
        SERIALS_LIST?.setValidators(null);
        SERIALS_LIST?.updateValueAndValidity();

        const STOCK_SOURCE_ID = this.modifyItemForm.get('STOCK_SOURCE_ID');
        STOCK_SOURCE_ID?.clearValidators();
        STOCK_SOURCE_ID?.setValidators(null);
        STOCK_SOURCE_ID?.updateValueAndValidity();

        this.modifyItemForm.patchValue({
          ITEM_SUPPLIER_BRAND: item.supplier_brand_id,
          ITEM_CLIENT_BRAND: item.client_brand_id,
          ITEM_CLIENT_STORE: item.client_store_id,

          ITEM_STATUS: item.item_status,

          ARTICLE_ID: item.article_id,
          REFERENCE: item.article_reference,
          LABEL: item.article_desc,
          QTE: item.qte.toString(),
          QTE_RECEIVED: item.qte_received.toString(),
          PRIX_UNIT: item.unit_price.toString(),
          PRIX_TOTAL: item.total_price.toString(),
          PRIX_DE_VENTE: item.prix_de_vente.toString(),
          TOTAL_PRIX_DE_VENTE: item.total_prix_de_vente.toString(),
          FRAISDEPORT: item.shipping_price.toString()
        })

        this.onBrandChange(false);
      } else {
        this.modifyItemForm.patchValue({
          STOCK_SOURCE_ID: item.stock_source,
          STOCK_SOURCE_LBL: item.stock_source_lbl,
          ARTICLE_ID: item.article_id,
          REFERENCE: item.article_reference,
          LABEL: item.article_desc,
          QTE: item.qte.toString(),
          PRIX_UNIT: item.unit_price.toString(),
          PRIX_TOTAL: item.total_price.toString(),
          AVAIL_STOCK: item.qte_stock.toString(),
          RETURN_STATUS: item.return_status,
          RETURN_DAYS: item.return_days?.toString(),
          ITEM_BRAND: item.brand_id ? item.brand_id : "",
          SERIALS_LIST: item.article_serials.join(",")
        })
      }

      if(this.numeroType == 'reception-numero-series' && item.achat_item_id !== undefined && item.achat_item_id !== null) {
        this.modifyItemForm.get('ACHAT_ITEM_ID')?.setValue(item.achat_item_id);

        const originQte = +item.achat_item_qte;
        const receivedQte = item.achat_item_qte_recieved || 0;
        let remainingQte = originQte - receivedQte;

        if(this.bonDocument.status == APP_CONSTANTS.RECEPTION_VALIDATION_STAT_YES) {
          remainingQte += +item.qte;
        }

        this.modifyItemForm.get('QTE_ORIGIN')?.setValue(originQte);
        this.modifyItemForm.get('QTE_RECEIVED')?.setValue(receivedQte);
        this.modifyItemForm.get('QTE_REMAINING')?.setValue(remainingQte);
      }

      if(this.numeroType == 'livraison-numero-series'){
        this.modifyItemForm.get('ASSURANCE')?.setValue(item.assurance)
      }

      if(this.numeroType == 'livraison-numero-series' || this.numeroType === 'transfer-de-stock-numero-series') {
        this.modifyItemForm.get('SELECTED_SERIALS_LIST')?.setValue(item.article_serials.join(","));
        this.modifyItemForm.get('SELECTED_SERIALS_LIST_BACKUP')?.setValue(item.article_serials.join(","));
      }
    }
  }

  calculatePrixTotal(){
    const formData = this.modifyItemForm.getRawValue();

    let totalPrix = 0;
    let totalprixdevente = 0;
    const QTE =  formData.QTE;
    if(QTE === null || QTE === '' || QTE === 0) {
      this.modifyItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_UNIT =  formData.PRIX_UNIT;
    if(PRIX_UNIT === null || PRIX_UNIT === '' || PRIX_UNIT === 0) {
      this.modifyItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
      return;
    }

    const PRIX_DE_VENTE = formData.PRIX_DE_VENTE;
    if(PRIX_DE_VENTE === null || PRIX_DE_VENTE === '' || PRIX_DE_VENTE === 0) {
      this.modifyItemForm.get("TOTAL_PRIX_DE_VENTE")?.setValue(totalprixdevente.toString());
      return;
    }

    totalPrix = parseInt(QTE) * parseFloat(PRIX_UNIT);
    totalprixdevente = parseInt(QTE) * parseFloat(PRIX_DE_VENTE);
    this.modifyItemForm.get("PRIX_TOTAL")?.setValue(totalPrix.toString());
    this.modifyItemForm.get("TOTAL_PRIX_DE_VENTE")?.setValue(totalprixdevente.toString());
  }

  changeQuantity(type: 'creation' | 'modification') {
    let oldListSerials = [];
    let selectedListSerials =[];

    if(this.numeroType == 'reception-numero-series') {
      const itemSerials = this.modifyItemForm.get("SERIALS_LIST")?.value;
      if(itemSerials !== "") {
        oldListSerials = itemSerials.split(",");
      }

      oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

      const achatItemId = this.modifyItemForm.get('ACHAT_ITEM_ID')?.value;
      let maximumDoableQte = undefined;
      if(achatItemId !== undefined && achatItemId !== null && achatItemId !== "") {
        maximumDoableQte = this.modifyItemForm.get('QTE_REMAINING')?.value;
      }

      this.processSerialNumPopup({
        oldListSerials,
        maxSerialsLimit: maximumDoableQte
      });
      return;
    }

    if(this.numeroType == 'transfer-de-stock-numero-series') {
      const itemSerials = this.modifyItemForm.get("SERIALS_LIST")?.value;
      let oldListSerials = this.commonService.formatSerialNumbers(itemSerials ? (typeof itemSerials === 'string'? itemSerials.split(","): itemSerials) :[]);

      const itemSelectedSerials = this.modifyItemForm.get('SELECTED_SERIALS_LIST')?.value;
      const selectedListSerials = itemSelectedSerials ? (typeof itemSelectedSerials === 'string' ? itemSelectedSerials.split(",") : itemSelectedSerials):[];

      const itemSelectedSerialsBackup = this.modifyItemForm.get('SELECTED_SERIALS_LIST_BACKUP')?.value;
      const selectedListSerialsBackup = itemSelectedSerialsBackup ? (typeof itemSelectedSerialsBackup === 'string' ? itemSelectedSerialsBackup.split(",") : itemSelectedSerialsBackup):[];

      const blockedInSerials = this.currentArticleDetail?.blocked_in_serials;
      const blockedOutSerials = this.currentArticleDetail?.blocked_out_serials;

      if(this.bonDocument.status == APP_CONSTANTS.TRANSFER_DE_STOCK_VALIDATION_STAT_YES) {
        selectedListSerialsBackup.forEach((row: string) => {
          const checkIfExists = oldListSerials.find(r => r.serial_no === row);
          if(checkIfExists === undefined || checkIfExists === null) {
            oldListSerials.unshift({
              serial_no: row,
              serial_tag: null,
              disabled: false,
            });
          }
        })
      }

      this.processSerialNumPopup({
        oldListSerials,
        selectedSerials: selectedListSerials,
        selectedSerialsBackup: selectedListSerialsBackup,
        block_in_serials: blockedInSerials,
        block_out_serials: blockedOutSerials
      });
      return;
    }

    if(this.numeroType == 'livraison-numero-series') {
      let oldListSerials = this.commonService.formatSerialNumbers(this.bonItemSerials);

      const itemSelectedSerials = this.modifyItemForm.get('SELECTED_SERIALS_LIST')?.value;
      const selectedListSerials = itemSelectedSerials ? (typeof itemSelectedSerials === 'string' ? itemSelectedSerials.split(",") : itemSelectedSerials):[];

      const itemSelectedSerialsBackup = this.modifyItemForm.get('SELECTED_SERIALS_LIST_BACKUP')?.value;
      const selectedListSerialsBackup = itemSelectedSerialsBackup ? (typeof itemSelectedSerialsBackup === 'string' ? itemSelectedSerialsBackup.split(",") : itemSelectedSerialsBackup):[];

      const blockedInSerials = this.currentArticleDetail?.blocked_in_serials;
      const blockedOutSerials = this.currentArticleDetail?.blocked_out_serials;

      if(this.bonDocument.status == APP_CONSTANTS.LIVRAISON_VALIDATION_STAT_YES) {
        selectedListSerialsBackup.forEach((row: string) => {
          const checkIfExists = oldListSerials.find(r => r.serial_no === row);
          if(checkIfExists === undefined || checkIfExists === null) {
            oldListSerials.unshift({
              serial_no: row,
              serial_tag: null,
              disabled: false,
            });
          }
        })
      }

      this.processSerialNumPopup({
        oldListSerials,
        selectedSerials: selectedListSerials,
        selectedSerialsBackup: selectedListSerialsBackup,
        block_in_serials: blockedInSerials,
        block_out_serials: blockedOutSerials
      });
      return;
    }

    if(this.numeroType == 'inventory-numero-series') {
      let listSerials = [],
        oldListSerials = [],
        removedListSerials = [];

      const itemSerials = this.modifyItemForm.get("SERIALS_LIST")?.value;
      if(itemSerials !== undefined && itemSerials !== null && itemSerials !== "") {
        listSerials = itemSerials.split(",");
      }

      listSerials = this.commonService.formatSerialNumbers(listSerials);

      const itemOldSerials = this.modifyItemForm.get("OLD_SERIALS_LIST")?.value;
      if(itemOldSerials !== undefined && itemOldSerials !== null && itemOldSerials !== "") {
        oldListSerials = itemOldSerials.split(",");
      }

      oldListSerials = this.commonService.formatSerialNumbers(oldListSerials);

      const itemRemovedSerials = this.modifyItemForm.get("REMOVED_SERIALS_LIST")?.value;
      if(itemRemovedSerials !== undefined && itemRemovedSerials !== null && itemRemovedSerials !== "") {
        removedListSerials = itemRemovedSerials.split(",");
      }

      removedListSerials = this.commonService.formatSerialNumbers(removedListSerials);

      this.processSerialNumPopup({
        newListSerials: listSerials,
        removeListSerials: removedListSerials,
        oldListSerials: oldListSerials
      });
    }
  }

  openUpSerialNumPopup(data?: {}): Promise<any> {
    const dialogRef = this.dialog.open(NumeroSerieComponent,
      {
        width: '50vw',
        data: {
          numeroType: this.numeroType,
          ...data
        }
      }
    );

    return lastValueFrom(dialogRef.afterClosed());
  }

  async processSerialNumPopup(data?: {}): Promise<any> {
    return new Promise(async (resolve, reject) => {
      const serialResult = await this.openUpSerialNumPopup(data);

      if(serialResult && serialResult.status === "success") {
        // After popup validation
        console.log(serialResult);
        if(serialResult.data && serialResult.data !== '') {
          if(this.numeroType === 'inventory-numero-series') {
            this.modifyItemForm.get('QTE')?.setValue(serialResult.data.newSerials.length.toString());
          } else {
            this.modifyItemForm.get('QTE')?.setValue(serialResult.data.length.toString());
          }


          this.unitPrixInputEl?.nativeElement.select();
          this.unitPrixInputEl?.nativeElement.focus();
          this.quantiteInputEl?.nativeElement.dispatchEvent(new Event('change'));

          if(this.numeroType == 'livraison-numero-series') {
            this.modifyItemForm.get('SELECTED_SERIALS_LIST')?.setValue(serialResult.data.join(','));

            resolve(serialResult);
            return;
          }

          if(this.numeroType == 'transfer-de-stock-numero-series') {
            this.modifyItemForm.get('SELECTED_SERIALS_LIST')?.setValue(serialResult.data.join(','));

            resolve(serialResult);
            return;
          }

          if(this.numeroType == 'reception-numero-series') {
            this.modifyItemForm.get('SERIALS_LIST')?.setValue(serialResult.data.join(','));

            resolve(serialResult);
            return;
          }

          if(this.numeroType == 'inventory-numero-series') {
            const { newSerials, removedSerials } = serialResult.data;
            this.newSerialLists = newSerials.join(',');

            this.modifyItemForm.get('QTE')?.setValue(newSerials.length.toString());
            this.modifyItemForm.get('SERIALS_LIST')?.setValue(newSerials.join(','));
            this.modifyItemForm.get('REMOVED_SERIALS_LIST')?.setValue(removedSerials.join(','));

            resolve(serialResult);
            return;
          }
        }
      }
    });
  }

  onChangeReturnStatus(event:any){
    if(event.value === -1){
      this.modifyItemForm.get('RETURN_DAYS')?.setValidators([Validators.required,Validators.min(1)])
      this.modifyItemForm.get('RETURN_DAYS')?.updateValueAndValidity();
    }else{
      this.modifyItemForm.get('RETURN_DAYS')?.clearValidators();
      this.modifyItemForm.get('RETURN_DAYS')?.updateValueAndValidity();
    }
  }

  async modifyItemFormSubmitter() {
    console.log(this.modifyItemForm);
    if(this.modifyItemForm.invalid ) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KMANDATORY_FIELDS'
        });
        return;
    }

    let formData = this.modifyItemForm.getRawValue();

    const user_detail = this.authService.getCurrentUser();
    const user_id = user_detail ? user_detail.USID : null;

    if(+formData.QTE < 1) {
      this.loadingService.hide();
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KVALIDATE_QTE_GREATER_ZERO'
      });
      return;
    }

    if(this.numeroType == 'achat-commande-numero-series') {
      let bonArticles: BonAchatCommandeItem[] = [...this.bonDocument.articles];
      bonArticles.map((row) => {
        if(row.item_id === this.currentItem.item_id) {
          row.qte = +formData.QTE;
          row.unit_price = +formData.PRIX_UNIT;
          row.total_price = +formData.PRIX_TOTAL;
          row.prix_de_vente = +formData.PRIX_DE_VENTE;
          row.total_prix_de_vente = +formData.TOTAL_PRIX_DE_VENTE;
          row.supplier_brand_id = formData.ITEM_SUPPLIER_BRAND;
          row.client_brand_id = formData.ITEM_CLIENT_BRAND;
          row.client_store_id = formData.ITEM_CLIENT_STORE;
          row.shipping_price = +formData.FRAISDEPORT;
          row.item_status = formData.ITEM_STATUS;
        }

        return row;
      })

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      const bonDocument: BonAchatCommande = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
        modified_userid: user_id
      };

      delete bonDocument.achatcommande_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'reception-numero-series') {
      const itemSerials = formData.SERIALS_LIST.split(',');
      let bonArticles: BonReceptionItem[] = [...this.bonDocument.articles];
      bonArticles.map((row) => {
        if(row.item_id === this.currentItem.item_id) {
          row.qte = +formData.QTE;
          row.unit_price = +formData.PRIX_UNIT;
          row.total_price = +formData.PRIX_TOTAL;
          row.article_serials = itemSerials;
          row.brand_id = formData.ITEM_BRAND;
        }

        return row;
      })

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      const bonDocument: BonReception = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
      };

      delete bonDocument.reception_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'livraison-numero-series') {
      const itemSerials = formData.SELECTED_SERIALS_LIST.split(',');
      let bonArticles: BonLivraisonItem[] = [...this.bonDocument.articles];
      bonArticles.map((row) => {
        if(row.item_id === this.currentItem.item_id) {
          row.qte = +formData.QTE;
          row.unit_price = +formData.PRIX_UNIT;
          row.total_price = +formData.PRIX_TOTAL;
          row.return_status = formData.RETURN_STATUS;
          row.return_days = +formData.RETURN_DAYS;
          row.article_serials = itemSerials;
          row.assurance = +formData.ASSURANCE;
        }
        return row;
      })

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      let bonTotalAssurance = bonArticles.reduce((acc,b) => acc + +b.assurance, 0);
      const bonDocument: BonLivraison = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
        total_assurance: bonTotalAssurance,
      };

      delete bonDocument.livraison_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'transfer-de-stock-numero-series') {
      const itemSerials = formData.SELECTED_SERIALS_LIST.split(',');
      let bonArticles: BonTransferDeStockItem[] = [...this.bonDocument.articles];
      bonArticles.map((row) => {
        if(row.item_id === this.currentItem.item_id) {
          row.qte = +formData.QTE;
          row.unit_price = +formData.PRIX_UNIT;
          row.total_price = +formData.PRIX_TOTAL;
          row.article_serials = itemSerials;
        }

        return row;
      })

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      const bonDocument: BonTransferDeStock = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
      };

      delete bonDocument.tds_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'inventory-numero-series') {
      let bonArticles: BonInventoryItem[] = [...this.bonDocument.articles];

      const itemSerialsNew = (formData.SERIALS_LIST !== '' && formData.SERIALS_LIST !== null) ? formData.SERIALS_LIST.split(',') : [];
      const itemSerialsRemoved = (formData.REMOVED_SERIALS_LIST !== '' && formData.REMOVED_SERIALS_LIST !== null) ? formData.REMOVED_SERIALS_LIST.split(',') : [];

      bonArticles = bonArticles.map((row) => {
        if(row.item_id === this.currentItem.item_id) {
          row.qte = +formData.QTE;
          row.qte_stock = +formData.SERIALS_LIST.length;
          row.unit_price = +formData.PRIX_UNIT;
          row.total_price = +formData.PRIX_TOTAL;
          row.article_serials = itemSerialsNew;
          row.removed_serials = itemSerialsRemoved;
        }

        return row;
      });

      const bonDocument: BonInventory = {
        ...this.bonDocument,
        articles: bonArticles,
      };

      delete bonDocument.inventory_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }
  }

  removeItem() {
    const user_detail = this.authService.getCurrentUser();
    const user_id = user_detail ? user_detail.USID : null;

    if(this.numeroType == 'achat-commande-numero-series') {
      let bonArticles: BonAchatCommandeItem[] = [...this.bonDocument.articles];
      bonArticles = bonArticles.filter((row) => row.item_id !== this.currentItem.item_id);

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      const bonDocument: BonAchatCommande = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
        modified_userid: user_id
      };

      delete bonDocument.achatcommande_id;

      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'reception-numero-series') {
      let bonArticles: BonReceptionItem[] = [...this.bonDocument.articles];
      bonArticles = bonArticles.filter((row) => row.item_id !== this.currentItem.item_id);

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      const bonDocument: BonReception = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
      };

      delete bonDocument.reception_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'livraison-numero-series') {
      let bonArticles: BonLivraisonItem[] = [...this.bonDocument.articles];
      bonArticles = bonArticles.filter((row) => row.item_id !== this.currentItem.item_id);

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      let bonTotalAssurance = bonArticles.reduce((acc,b) => acc + +b.assurance, 0);
      const bonDocument: BonLivraison = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix,
        total_assurance: bonTotalAssurance,
      };

      delete bonDocument.livraison_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'transfer-de-stock-numero-series') {
      let bonArticles: BonTransferDeStockItem[] = [...this.bonDocument.articles];
      bonArticles = bonArticles.filter((row) => row.item_id !== this.currentItem.item_id);

      let bonTotalPrix = bonArticles.reduce((acc,b) => acc + +b.total_price, 0);
      const bonDocument: BonTransferDeStock = {
        ...this.bonDocument,
        articles: bonArticles,
        total_price: bonTotalPrix
      };

      delete bonDocument.tds_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }

    if(this.numeroType == 'inventory-numero-series') {
      let bonArticles: BonInventoryItem[] = [...this.bonDocument.articles];
      bonArticles = bonArticles.filter((row) => row.item_id !== this.currentItem.item_id);
      const bonDocument: BonInventory = {
        ...this.bonDocument,
        articles: bonArticles,
      };

      delete bonDocument.inventory_id;

      console.log('BON DOCUMENT: ', bonDocument);
      this.dialogRef.close({ status: 'success', data: bonDocument })
      return;
    }
  }


}
