import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import {PageEvent} from '@angular/material/paginator';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { CommonService } from '../../../_services/common.service';
import { ContactService } from '../../../_services/contact.service';
import {
  BrandInfoList,
  ContactList,
  CustomHttpResponse,
  Store, StoreCategories,
  StoreClientCategories, StoreList,
  StoreSupplierCategories
} from "../../../_models";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {lastValueFrom} from "rxjs";
import {TranslateService} from "@ngx-translate/core";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {BrandList} from "../../../_models/brand";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-contact-list',
  templateUrl: './contact-list.component.html',
  styleUrls: ['./contact-list.component.scss']
})
export class ContactListComponent implements OnInit {

  @Input() displayType: "default" | "livrasion-contact-popup" | "reception-contact-popup" | "achat-commande-contact-popup" | "fusionner-popup" = "default";
  @Input() searchDefaultParams: any | undefined;
  @Input() mainStore: Store | undefined;
  get chosenContact(): any { return this._chosenContact }
  @Input() set chosenContact(chosenContact: any) {
    this.chosenContactChange.emit(this._chosenContact = chosenContact);
  }
  @Output() private chosenContactChange: EventEmitter<any> = new EventEmitter<any>();

  displayedColumns: string[] | undefined;
  displayedColumnsBrand: string[] = ['BR_LABEL', 'BR_TYPE', 'BR_TELE', 'BR_MAIL', 'BR_LOGO', 'BR_ACTIONS'];
  displayedColumnsContact: string[] = ['CT_LABEL', 'CT_TELE', 'CT_MAIL', 'CT_ACTIONS'];
  displayedColumnsStore: string[] = ['ST_LABEL', 'ST_BRAND', 'ST_CATEGORY', 'ST_TYPE', 'ST_HQ', 'ST_ACTIONS'];

  submitted: boolean = false;
  searchForm: FormGroup = this.formBuilder.group({
    FILTER_TYPE: ['', [Validators.required]], // contact, brand, store
    FILTER_KEYWORD: [''],

    FILTER_STORE_TYPE: [''], // if FILTER_TYPE is selected to "store" => client, supplier
    FILTER_STORE_BRAND: [''], // if FILTER_TYPE is selected to "store"
    FILTER_STORE_CATEGORY: [''], // if FILTER_TYPE is selected to "store"
  });

  brandList: BrandInfoList[] = [];
  storeCategoryList: StoreCategories[] = [];

  listData: any | undefined;
  imageURL: any;

  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  rolePermission:any;
  private _chosenContact: any = undefined;

  constructor(private formBuilder: FormBuilder,
              private commonService: CommonService,
              private dataSharingService: DataSharingService,
              private contactService: ContactService,
              private translateService: TranslateService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
  }

  ngOnInit(): void {
    this.checkForListCache();
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }

    await this._getBrandList();

    await this.loadingService.hide();

    if(this.displayType === 'livrasion-contact-popup') {
      this.displayedColumnsStore = [ 'CONTACT_CHOOSE', 'ST_LABEL', 'ST_BRAND', 'ST_CATEGORY', 'ST_TYPE', 'ST_HQ'];
      this.displayedColumns = this.displayedColumnsStore;

      const filterType = this.searchDefaultParams.FILTER_TYPE;
      const store_type = this.searchDefaultParams.FILTER_STORE_TYPE;

      this.searchForm.get('FILTER_TYPE')?.setValue(filterType);
      this.searchForm.get('FILTER_STORE_TYPE')?.setValue(store_type);
      this.searchForm.get('FILTER_KEYWORD')?.setValue(this.searchDefaultParams.FILTER_KEYWORD);
      this.searchForm.get('FILTER_TYPE')?.disable({ onlySelf: true });
      this.searchForm.get('FILTER_STORE_CATEGORY')?.setValue('');
      this.searchForm.updateValueAndValidity();

      await this.loadingService.show();
      this._getStoreCategoriesList(store_type);
      this.searchFormSubmitter();
    }

    if(this.displayType === 'reception-contact-popup') {
      this.displayedColumnsStore = [ 'CONTACT_CHOOSE', 'ST_LABEL', 'ST_BRAND', 'ST_CATEGORY', 'ST_TYPE', 'ST_HQ'];
      this.displayedColumns = this.displayedColumnsStore;

      const filterType = this.searchDefaultParams.FILTER_TYPE;
      const store_type = this.searchDefaultParams.FILTER_STORE_TYPE;

      this.searchForm.get('FILTER_TYPE')?.setValue(filterType);
      this.searchForm.get('FILTER_STORE_TYPE')?.setValue(store_type);
      this.searchForm.get('FILTER_KEYWORD')?.setValue(this.searchDefaultParams.FILTER_KEYWORD);
      this.searchForm.get('FILTER_TYPE')?.disable({ onlySelf: true });
      this.searchForm.get('FILTER_STORE_CATEGORY')?.setValue('');
      this.searchForm.updateValueAndValidity();

      await this.loadingService.show();
      this._getStoreCategoriesList(store_type);
      this.searchFormSubmitter();
    }

    if(this.displayType === 'achat-commande-contact-popup') {
      this.displayedColumnsStore = [ 'CONTACT_CHOOSE', 'ST_LABEL', 'ST_BRAND', 'ST_CATEGORY', 'ST_TYPE', 'ST_HQ'];
      this.displayedColumns = this.displayedColumnsStore;

      const filterType = this.searchDefaultParams.FILTER_TYPE;
      const store_type = this.searchDefaultParams.FILTER_STORE_TYPE;

      this.searchForm.get('FILTER_TYPE')?.setValue(filterType);
      this.searchForm.get('FILTER_STORE_TYPE')?.setValue(store_type);
      this.searchForm.get('FILTER_KEYWORD')?.setValue(this.searchDefaultParams.FILTER_KEYWORD);

      this.searchForm.get('FILTER_TYPE')?.disable({ onlySelf: true });
      this.searchForm.get('FILTER_STORE_TYPE')?.disable({ onlySelf: true });
      this.searchForm.get('FILTER_STORE_CATEGORY')?.setValue('');
      this.searchForm.updateValueAndValidity();

      await this.loadingService.show();
      this._getStoreCategoriesList(store_type);
      this.searchFormSubmitter();
    }

    if(this.displayType === 'fusionner-popup') {
      this.displayedColumnsStore = [ 'CONTACT_CHOOSE', 'ST_LABEL', 'ST_BRAND', 'ST_CATEGORY', 'ST_TYPE', 'ST_HQ'];
      this.displayedColumns = this.displayedColumnsStore;

      const filterType = this.searchDefaultParams.FILTER_TYPE;
      const store_type = this.searchDefaultParams.FILTER_STORE_TYPE;

      this.searchForm.get('FILTER_TYPE')?.setValue(filterType);
      this.searchForm.get('FILTER_STORE_TYPE')?.setValue(store_type);

      this.searchForm.get('FILTER_TYPE')?.disable({ onlySelf: true });
      this.searchForm.get('FILTER_STORE_TYPE')?.disable({ onlySelf: true });

      await this.loadingService.show();
      this._getStoreCategoriesList(store_type);
      this.searchFormSubmitter();
    }
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM);
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        FILTER_TYPE: formCache.FILTER_TYPE,
        FILTER_KEYWORD: formCache.FILTER_KEYWORD,

        FILTER_STORE_TYPE: formCache.FILTER_STORE_TYPE,
        FILTER_STORE_BRAND: formCache.FILTER_STORE_BRAND,
        FILTER_STORE_CATEGORY: formCache.FILTER_STORE_CATEGORY,
      });

      this._refreshDisplayColumns(formCache.FILTER_TYPE);

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
      if(dataCache !== undefined) {
        if(formCache.FILTER_TYPE === 'contact') {
          this.listData = dataCache.data as ContactList[];
        }

        if(formCache.FILTER_TYPE === 'store') {
          this.listData = dataCache.data as StoreList[];
        }

        if(formCache.FILTER_TYPE === 'brand') {
          this.listData = dataCache.data as BrandList[];
        }

        this.imageURL = dataCache.imageurl;

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }
  }

  get fr() {
    return this.searchForm.controls;
  }

  get filterType(): string {
    return this.searchForm.get('FILTER_TYPE')?.value;
  }

  storeStatus(status: number):boolean {
    return status === APP_CONSTANTS.STORE_HQ_STAT_YES;
  }

  documentStatus(item: StoreList) {
    const statusList = APP_CONSTANTS.STORE_HQ_STATUS;

    const statusData = statusList.find((r) => r.value == item.store_hq);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.store_hq) {
      case APP_CONSTANTS.STORE_HQ_STAT_YES:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.STORE_HQ_STAT_NO:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }

  _getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  _refreshDisplayColumns(type: 'contact' | 'brand' | 'store') {
    if(type === 'contact') {
      this.displayedColumns = this.displayedColumnsContact;
      return;
    }

    if(type === 'brand') {
      this.displayedColumns = this.displayedColumnsBrand;
      return;
    }

    if(type === 'store') {
      this.displayedColumns = this.displayedColumnsStore;
      return;
    }
  }

  onStoreTypeChange() {
    const store_type = this.searchForm.get('FILTER_STORE_TYPE')?.value;
    this.searchForm.get('FILTER_STORE_CATEGORY')?.setValue('');

    if(store_type === '') {
      return;
    }

    this.loadingService.show();
    this._getStoreCategoriesList(store_type);
  }

  _getStoreCategoriesList(store_type: 'supplier' | 'client'): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<StoreSupplierCategories[] | StoreClientCategories[]>) => {
          this.loadingService.hide();
          if(res.status === 'success') {
            const categoriesList = res.data;

            if(store_type === 'client') {
              this.storeCategoryList = (categoriesList as StoreClientCategories[]).map((r => {
                return { category_id: r.CCTID, category_label: r.CCTLBL };
              }));
            }

            if(store_type === 'supplier') {
              this.storeCategoryList = (categoriesList as StoreSupplierCategories[]).map((r => {
                return { category_id: r.SCTID, category_label: r.SCTLBL };
              }));
            }
            resolve(res);
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStoreCategories(store_type).subscribe(observer);
    });
  }

  onContactSelectionChange(e: MatCheckboxChange, row: any) {
    this.chosenContact = undefined;
    if(e.checked) {
      this.chosenContact = row;
    }
  }

  onTableRowClicked(row: any) {
    if(this.displayType === 'livrasion-contact-popup') {
      this.updateClientOnRowClick(row);
      return;
    }

    if(this.displayType === 'reception-contact-popup') {
      this.updateClientOnRowClick(row);
      return;
    }
  }

  updateClientOnRowClick(row: any) {
    if(this.chosenContact === row) {
      this.chosenContact = undefined;
    } else {
      this.chosenContact = row;
    }
  }


  searchFormSubmitter() {
    if(this.searchForm.invalid) {
      return;
    }

    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM, JSON.stringify(formData));

    this._refreshDisplayColumns(formData.FILTER_TYPE);

    let payload: any = {
      filter_type: "",
      keyword: "",
      store_type: "",
      store_brand: "",
      store_category: ""
    };

    if(formData.FILTER_TYPE === 'contact') {
      payload = {
        ...payload,
        filter_type: formData.FILTER_TYPE,
        keyword: formData.FILTER_KEYWORD
      };
    }

    if(formData.FILTER_TYPE === 'store') {
      payload = {
        ...payload,
        filter_type: formData.FILTER_TYPE,
        keyword: formData.FILTER_KEYWORD,
        store_type: formData.FILTER_STORE_TYPE,
        store_brand: formData.FILTER_STORE_BRAND,
        store_category: formData.FILTER_STORE_CATEGORY
      };
    }

    if(formData.FILTER_TYPE === 'brand') {
      payload = {
        ...payload,
        filter_type: formData.FILTER_TYPE,
        keyword: formData.FILTER_KEYWORD
      };
    }

    this.currentPage = 1;

    this.loadingService.show();
    this._getContactList(payload, this.currentPage, this.pageSize);
  }

  _getContactList(payload: { filter_type: string, keyword: string, store_type?: string, store_brand?: string, store_category?: string }, page: number = 1, pageSize = 10) {
    const observer = {
      next: (res: CustomHttpResponse<ContactList[] | StoreList[] | BrandList[]>) => {
        this.loadingService.hide();

        if(res.status === "success") {
          if(payload.filter_type === 'contact') {
            this.listData = res.data as ContactList[];
          }

          if(payload.filter_type === 'store') {
            this.listData = res.data as StoreList[];
          }

          if(payload.filter_type === 'brand') {
            this.listData = res.data as BrandList[];
          }

          this.imageURL = res.imageurl;

          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;

          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST, JSON.stringify(res));
        } else {
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
        this.loadingService.hide();
      }
    };

    this.contactService.getContactList(payload, page, pageSize).subscribe(observer);
  }

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();

    let payload: any = {
      filter_type: "",
      keyword: "",
      store_type: "",
      store_brand: "",
      store_category: ""
    };

    if(formData.FILTER_TYPE === 'contact') {
      payload = {
        ...payload,
        filter_type: formData.FILTER_TYPE,
        keyword: formData.FILTER_KEYWORD
      };
    }

    if(formData.FILTER_TYPE === 'store') {
      payload = {
        ...payload,
        filter_type: formData.FILTER_TYPE,
        keyword: formData.FILTER_KEYWORD,
        store_type: formData.FILTER_STORE_TYPE,
        store_brand: formData.FILTER_STORE_BRAND,
        store_category: formData.FILTER_STORE_CATEGORY
      };
    }

    if(formData.FILTER_TYPE === 'brand') {
      payload = {
        ...payload,
        filter_type: formData.FILTER_TYPE,
        keyword: formData.FILTER_KEYWORD
      };
    }

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;

    this._getContactList(payload, this.currentPage, this.pageSize);
  }

  __updateListCache(id: number, type: 'contact' | 'store' | 'brand') {
      const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
      if(listCache !== undefined) {
        let listContacts: any[] = listCache.data;
        if(type === 'contact') {
          listContacts = (listContacts as ContactList[]).filter(r => r.contact_id != id);
        } else if (type === 'store') {
          listContacts = (listContacts as StoreList[]).filter(r => r.store_id != id);
        } else if (type === 'brand') {
          listContacts = (listContacts as BrandList[]).filter(r => r.brand_id != id);
        }

        listCache.data = listContacts;

        this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST, JSON.stringify(listCache));

        this.checkForListCache();

        if(listContacts.length === 0) {
          this.searchFormSubmitter();
        }
      }
  }

  confirmRemoveItem(label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: label })}</h3>`;

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeItem(id: number, label: string, type: 'contact' | 'brand' | 'store', store_type: 'supplier' | 'client' = 'supplier') {
    const removeConfirmation = await this.confirmRemoveItem(label);

    if(removeConfirmation) {
      this.loadingService.show();

      if(type === 'contact') {
        if(!this.rolePermission['5_14']) {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KACCESS_DENIED'
          })
          return;
        }
        this._deleteContact(id);
      }

      if(type === 'brand') {
        if(!this.rolePermission['5_24']) {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KACCESS_DENIED'
          })
          return;
        }
        this._deleteBrand(id);
      }

      if(type === 'store') {
        if(!this.rolePermission['5_34']) {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KACCESS_DENIED'
          })
          return;
        }
        this._deleteStore(id, store_type);
      }

      this.loadingService.hide();
    }
  }

  async _deleteContact(contact_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          }).then(() => {
            this.__updateListCache(contact_id, 'contact');
          });

          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.contactService.deleteContact(contact_id).subscribe(observer);
  }

  async _deleteBrand(brand_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          }).then(() => {
            this.__updateListCache(brand_id, 'brand');
          });

          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.contactService.deleteBrand(brand_id).subscribe(observer);
  }

  async _deleteStore(store_id: number, type: string) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          }).then(() => {
            this.__updateListCache(store_id, 'store');
          });

          return;
        }

        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    const payload = {
      store_type: type,
      store_id: store_id
    };

    this.contactService.deleteStore(payload).subscribe(observer);
  }

}
