import { Component, OnInit } from '@angular/core';
import {FormBuilder, FormGroup, Validators} from "@angular/forms";
import {ArticlesList, CustomHttpResponse} from "../../../_models";
import {CommonService} from "../../../_services/common.service";
import {ContactService} from "../../../_services/contact.service";
import {MatDialog} from "@angular/material/dialog";
import {ActivatedRoute, Router} from "@angular/router";
import {TranslateService} from "@ngx-translate/core";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import {CreateContactAssocComponent} from "../create-contact-assoc/create-contact-assoc.component";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-create-store',
  templateUrl: './create-store.component.html',
  styleUrls: ['./create-store.component.scss']
})
export class CreateStoreComponent implements OnInit {

  filteredVillesFacturation: any = [];
  filteredVillesExpedition: any = [];

  filteredPostalCodesFacturation: any = [];
  filteredPostalCodesExpedition: any = [];

  submitted: boolean = false;

  brandId: number | undefined;

  brandList: any = [];
  articleList: ArticlesList[] = [];
  paysList: any = [];
  defriementList: any = [];

  _filteredArticles: ArticlesList[] = [];
  _filterArticleKey: string = "";

  createStoreForm: FormGroup = this.formBuilder.group({
    store_brand: ['', Validators.required],
    store_nom: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_NOM_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_NOM_MAX_LENGTH)]],
    store_category: ['', Validators.required],

    store_defriement: [''], // Only for clients

    store_delay_livraison: [''], // Only for supplier
    store_cout_commande: [''], // Only for supplier

    store_address: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_ADDRESS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_ADDRESS_MAX_LENGTH)]],
    store_code_postal: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_CP_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_CP_MAX_LENGTH)]],
    store_pays: ['FR', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_PAYS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_PAYS_MAX_LENGTH)]],
    store_ville: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_VILLE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_VILLE_MAX_LENGTH)]],

    storefac_address: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_ADDRESS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_ADDRESS_MAX_LENGTH)]],
    storefac_code_postal: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_CP_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_CP_MAX_LENGTH)]],
    storefac_ville: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_PAYS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_PAYS_MAX_LENGTH)]],
    storefac_pays: ['FR', [Validators.required, Validators.minLength(APP_CONSTANTS.STORE_VILLE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STORE_VILLE_MAX_LENGTH)]],
    store_articles: [[]],

    SAME_AS_SHIPPING: [false],
    store_headquarters: [false]
  });

  store_type: string | undefined;
  storeCategories: any;
  contactFormData: any;
  sameAsShipping = false;

  constructor(private commonService: CommonService,
              private dataSharingService: DataSharingService,
              private contactService: ContactService,
              private formBuilder: FormBuilder,
              private dialog: MatDialog,
              private route: ActivatedRoute,
              private router: Router,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService) {
  }

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getPaysList();
    await this.getArticleList();
    await this.getBrandList();
    await this.getDefriementList();

    this.route.queryParams.subscribe(params => {
      if(params !== undefined && params !== null && params.hasOwnProperty('brand_id')) {
        this.brandId = +params["brand_id"];
        if(this.brandId !== undefined && this.brandId !== null) {
          this.createStoreForm.get('store_brand')?.setValue(this.brandId);
          this._onBrandChange();
        }
      }
    });

    await this.loadingService.hide();
  }

  getPaysList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.paysList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListPays().subscribe(observer);
    });
  }

  getArticleList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<ArticlesList[]>) => {
          if(res.status === 'success') {
            this.articleList = res.data;
            this._filteredArticles = this.articleList;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListArticles().subscribe(observer);
    });
  }

  getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  getDefriementList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.defriementList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListDefriement().subscribe(observer);
    });
  }

  getStoreCategories(type: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.storeCategories = res.data;

            if(type === 'client') {
              this.storeCategories = this.storeCategories.map((row: any) => {
                return { category_id: row.CCTID, category_lbl: row.CCTLBL };
              });
            }

            if(type === 'supplier') {
              this.storeCategories = this.storeCategories.map((row: any) => {
                return { category_id: row.SCTID, category_lbl: row.SCTLBL };
              });
            }
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListStoreCategories(type).subscribe(observer);
    });
  }

  openUpContactCreationPopup() {
    const brand_id = this.createStoreForm.get('store_brand')?.value;

    if(this.contactFormData === undefined || this.contactFormData === null) {
      this.contactFormData = {
        CTNOMDU: '',
        CTTEL: '',
        CTMAIL: '',
        CTPOSTE: '',
        CTSTID: '',
        CTBRID: '',
      };
    }

    this.contactFormData['CTBRID'] = brand_id;

    const dialog = this.dialog.open(CreateContactAssocComponent, {
      disableClose: true,
      autoFocus: true,
      data: {
        oldFormData: this.contactFormData
      }
    });

    dialog.afterClosed().subscribe(
      (res) => {
        if(res !== undefined && res !== null && res.status === 'success') {
          this.contactFormData = res.data;
        }
      }
    )
  }

  createStoreFormSubmitter() {
    this.submitted = true;
    if(this.createStoreForm.invalid){
      return;
    }

    this.createStore();
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM);
  }

  createStore() {
    this.submitted = true;

    this.createStoreForm.updateValueAndValidity();
    if(this.createStoreForm.invalid){
      return;
    }

    const formData = this.createStoreForm.getRawValue();

    let articlesList = formData.store_articles ? [...formData.store_articles.filter((_r: any) => _r != null)] : [];
    let payload: any = {};

    if(this.store_type === 'client') {
      payload = {
        BRID : formData.store_brand,

        CSTNOM : formData.store_nom,
        CSTCCTID: formData.store_category,

        CSTEXPADDR : formData.store_address,
        CSTEXPCODPOS : formData.store_code_postal,
        CSTEXPPAYCOD : formData.store_pays,
        CSTEXPVILLE : formData.store_ville,

        CSTFACADDR : formData.storefac_address,
        CSTFACCODPOS : formData.storefac_code_postal,
        CSTFACVILLE : formData.storefac_ville,
        CSTFACPAYCOD :formData.storefac_pays,

        CSTDEFID : formData.store_defriement,
        CSTHQSTAT : formData.store_headquarters,

        CSTARTID : articlesList,
        IS_CONTACT: 0
      };
    }

    if(this.store_type === 'supplier') {
      payload = {
        BRID : formData.store_brand,

        SSTNOM : formData.store_nom,
        SSTCCTID: formData.store_category,

        SSTEXPADDR : formData.store_address,
        SSTEXPCODPOS : formData.store_code_postal,
        SSTEXPPAYCOD : formData.store_pays,
        SSTEXPVILLE : formData.store_ville,

        SSTFACADDR : formData.storefac_address,
        SSTFACCODPOS : formData.storefac_code_postal,
        SSTFACVILLE : formData.storefac_ville,
        SSTFACPAYCOD : formData.storefac_pays,

        SSTCOMM: formData.store_cout_commande,
        SSTDELIV: formData.store_delay_livraison,
        SSTHQSTAT : formData.store_headquarters,

        SSTARTID : articlesList,
        IS_CONTACT: 0
      };
    }

    if(this.contactFormData !== undefined && this.contactFormData !== null) {
      payload = {
        ...payload,
        CTNOMDU : this.contactFormData.CTNOMDU,
        CTTEL : this.contactFormData.CTTEL,
        CTMAIL : this.contactFormData.CTMAIL,
        CTPOSTE : this.contactFormData.CTPOSTE,
        IS_CONTACT: 1
      }
    }

    const observer = {
      next: (res: CustomHttpResponse<any>) => {
        if(res.status === 'success') {
          this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          }).then(() => {
            this.__resetListCache();
            this.router.navigate(['/contact/contact-list']);
          });
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        console.log(err);
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.contactService.createStore(payload).subscribe(observer);
  }

  get storeType(): boolean {
    return this.store_type !== undefined && this.store_type !== null && this.store_type !== '';
  }

  get isClient(): boolean {
    return (this.store_type !== undefined && this.store_type === 'client');
  }

  get isSupplier(): boolean {
    return (this.store_type !== undefined && this.store_type === 'supplier');
  }

  get expeditionCodePostal() {
    return this.createStoreForm.get("store_code_postal")?.value;
  }

  get expeditionVille() {
    return this.createStoreForm.get("store_ville")?.value;
  }

  get facturationCodePostal() {
    return this.createStoreForm.get("storefac_code_postal")?.value;
  }

  get facturationVille() {
    return this.createStoreForm.get("storefac_ville")?.value;
  }

  get expeditionPays() {
    return this.createStoreForm.get("store_pays")?.value;
  }

  get facturationPays() {
    return this.createStoreForm.get("storefac_pays")?.value;
  }

  get fr() {
    return this.createStoreForm.controls;
  }

  async _onBrandChange() {
    const brand = this.createStoreForm.get('store_brand')?.value;
    const brandDetail = this.brandList.find((r: any) => r.BRID == brand);

    if(brand !== '' && brand !== undefined && brand !== null) {
      const oldType = this.store_type;
      this.store_type = brandDetail.BRTYPE;

      if(oldType !== this.store_type) {
        await this.loadingService.show();
        this.getStoreCategories(this.store_type);
        await this.loadingService.hide();
      }
    } else  {
      // if Brand is empty
      this.store_type = undefined;
      this.storeCategories = [];
    }
  }

  _onSameBillingChange(status: boolean) {
    if (status) {
      this.createStoreForm.patchValue({
        storefac_address: this.createStoreForm.get('store_address')?.value,
        storefac_code_postal: this.createStoreForm.get('store_code_postal')?.value,
        storefac_pays: this.createStoreForm.get('store_pays')?.value,
        storefac_ville: this.createStoreForm.get('store_ville')?.value,
      });
    } else {
      this.createStoreForm.patchValue({
        storefac_address: "",
        storefac_code_postal: "",
        storefac_pays: "",
        storefac_ville: "",
      });
    }
  }

  _onArticleFilterChange(search: string) {
    this._filteredArticles = this.articleList.filter((row: ArticlesList) => row.article_label.toLowerCase().indexOf(search.toLowerCase()) > -1);
  }

  onCodePostalChanged(type: 'facturation' | 'expedition') {
    let postalCode = undefined,
      pays = undefined;

    if(type === 'facturation') {
      postalCode = this.createStoreForm.get('storefac_code_postal')?.value;
      pays = this.facturationPays;
    }

    if(type === 'expedition') {
      postalCode = this.createStoreForm.get('store_code_postal')?.value;
      pays = this.expeditionPays;
    }

    if(postalCode !== '' && postalCode != null) {
      const payload = {
        codpos: postalCode,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            if(type === 'facturation') {
              this.filteredPostalCodesFacturation = res.data;
            }

            if (type === 'expedition') {
              this.filteredPostalCodesExpedition = res.data;
            }
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  resetCodePostal(type: 'facturation' | 'expedition') {
    if(type === 'facturation') {
      this.createStoreForm.get('storefac_code_postal')?.reset()
    }

    if(type === 'expedition') {
      this.createStoreForm.get('store_code_postal')?.reset()
    }
  }

  onVilleChanged(type: 'facturation' | 'expedition') {
    let ville = undefined,
      pays = undefined;

    if(type === 'facturation') {
      ville = this.createStoreForm.get('storefac_ville')?.value;
      pays = this.facturationPays;
    }

    if(type === 'expedition') {
      ville = this.createStoreForm.get('store_ville')?.value;
      pays = this.expeditionPays;
    }

    if(ville !== '' && ville != null) {
      const payload = {
        ville,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            if(type === 'facturation') {
              this.filteredVillesFacturation = res.data;
            }

            if (type === 'expedition') {
              this.filteredVillesExpedition = res.data;
            }
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  resetVille(type: 'facturation' | 'expedition') {
    if(type =='facturation') {
      this.createStoreForm.get('storefac_ville')?.reset()
    }

    if(type == 'expedition') {
      this.createStoreForm.get('store_ville')?.reset()
    }
  }

  onCodeOptionSelected(event:any, type: 'facturation' | 'expedition') {
    if(type === 'facturation') {
      this.createStoreForm.get('storefac_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.createStoreForm.get('storefac_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }

    if(type === 'expedition') {
      this.createStoreForm.get('store_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.createStoreForm.get('store_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }
  }

  onVilleOptionSelected(event:any, type: 'facturation' | 'expedition') {
    if(type === 'facturation') {
      this.createStoreForm.get('storefac_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.createStoreForm.get('storefac_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }

    if(type === 'expedition') {
      this.createStoreForm.get('store_code_postal')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.createStoreForm.get('store_ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
    }
  }

}
