import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import {NgxSpinnerService} from "ngx-spinner";
import {ContactService} from "../../../_services/contact.service";
import {CustomHttpResponse} from "../../../_models";
import {Brand} from "../../../_models/brand";
import {DialogService} from "../../../_services/dailog.service";
import {TranslateService} from "@ngx-translate/core";

@Component({
  selector: 'app-view-brand',
  templateUrl: './view-brand.component.html',
  styleUrls: ['./view-brand.component.scss']
})
export class ViewBrandComponent implements OnInit {

  createBrandForm: FormGroup = this.formBuilder.group({
    BRLAB: [{ value: "", disabled: true }],
    BRTEL: [{ value: "", disabled: true }],
    BRMAIL: [{ value: "", disabled: true }],
    BRLOGO: [{ value: "", disabled: true }],
    BRSTOCKID: [{ value: "", disabled: true }],
    BRTYPE: [{ value: "", disabled: true }],
    BRCONTRACT: [{ value: "", disabled: true }],
  });

  brandId: number | undefined;
  brandDetail: Brand | undefined;
  imgURL: string | undefined;

  constructor(private contactService: ContactService,
              private formBuilder: FormBuilder,
              private route: ActivatedRoute,
              private router:Router,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
   }

   ngOnInit(): void {
    this.route.params.subscribe(params => {
      if(params.hasOwnProperty('brand_id')) {
        this.brandId = +params["brand_id"];

        this.loadDependencies();
      }
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getBrandDetail(this.brandId);
    this._preFillBrandDetails(this.brandDetail);

    await this.loadingService.hide();
  }

  getBrandDetail(id: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = {
        brand_id: id
      };

      const observer = {
        next: (res: CustomHttpResponse<Brand>) => {
          if(res.status === 'success') {
            this.imgURL = res.imageurl;
            this.brandDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getBrandDetail(payload).subscribe(observer);
    })
  }

  _preFillBrandDetails(brandDetail?: Brand) {
    if(brandDetail === undefined || brandDetail === null) {
      return;
    }

    this.createBrandForm.get('BRLAB')?.setValue(brandDetail.brand_lbl);
    this.createBrandForm.get('BRTEL')?.setValue(brandDetail.brand_tel);
    this.createBrandForm.get('BRMAIL')?.setValue(brandDetail.brand_mail);
    this.createBrandForm.get('BRTYPE')?.setValue(brandDetail.brand_type);
    this.createBrandForm.get('BRCONTRACT')?.setValue(brandDetail.brand_contract);
    this.createBrandForm.get('BRSTOCKID')?.setValue(brandDetail.stock_location_lbl);
  }

  goToStore(id: number) {
    this.router.navigate(['/contact/store/detail/', id, this.brandDetail?.brand_type]);
  }

}
