import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import {CustomHttpResponse, Contact} from "../../../_models";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {ContactService} from "../../../_services/contact.service";
import { TranslateService } from '@ngx-translate/core';

@Component({
  selector: 'app-view-contact',
  templateUrl: './view-contact.component.html',
  styleUrls: ['./view-contact.component.scss']
})
export class ViewContactComponent implements OnInit {

  createContactForm: FormGroup = this.formBuilder.group({
    CTNOMDU: [{ value: '', disabled: true }],
    CTTEL: [{ value: '', disabled: true }],
    CTMAIL: [{ value: '', disabled: true }],
    CTPOSTE: [{ value: '', disabled: true }],
    CTSTID: [{ value: '', disabled: true }],
    CTBRID: [{ value: '', disabled: true }],
  });

  contact_id: number | undefined;
  contactDetail: Contact | undefined;

  constructor(private contactService: ContactService,
              private formBuilder: FormBuilder,
              private route: ActivatedRoute,
              private router:Router,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
   }

  ngOnInit(): void {
    this.route.params.subscribe(params => {
      this.contact_id = +params["contact_id"];
      this.loadDependencies();
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getContactDetail(this.contact_id);
    this._preFillContactDetails(this.contactDetail);

    await this.loadingService.hide();
  }

  getContactDetail(id: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const payload = {
        CTID: id
      };

      const observer = {
        next: (res: CustomHttpResponse<Contact>) => {
          if(res.status === 'success') {
            this.contactDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.contactService.getContactDetail(payload).subscribe(observer);
    })
  }

  _preFillContactDetails(contactDetail?: Contact) {
    if(contactDetail === undefined || contactDetail === null) {
      return;
    }

    this.createContactForm.get('CTNOMDU')?.setValue(contactDetail.CTNOMDU);
    this.createContactForm.get('CTBRID')?.setValue(contactDetail.BRLAB);
    this.createContactForm.get('CTPOSTE')?.setValue(contactDetail.CTPOSTE);
    this.createContactForm.get('CTMAIL')?.setValue(contactDetail.CTMAIL);
    this.createContactForm.get('CTTEL')?.setValue(contactDetail.CTTEL);
  }

  goToBrand(id: number) {
    this.router.navigate(['/contact/brand/detail', id]);
  }

  goToStore(id: number, type: string) {
    this.router.navigate(['/contact/store/detail', id, type]);
  }

}
