import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup } from '@angular/forms';
import {MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import {NgxSpinnerService} from "ngx-spinner";
import {CustomHttpResponse, Store} from "../../../_models";
import {ContactService} from "../../../_services/contact.service";
import {DialogService} from "../../../_services/dailog.service";
import {FusionerStoreComponent} from "../fusioner-store/fusioner-store.component";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { TranslateService } from '@ngx-translate/core';
import {DataSharingService} from "../../../_services/data-sharing.service";

@Component({
  selector: 'app-view-store',
  templateUrl: './view-store.component.html',
  styleUrls: ['./view-store.component.scss']
})
export class ViewStoreComponent implements OnInit {

  createStoreForm: FormGroup = this.formBuilder.group({
    CSTNOM:  [{ value:'', disabled: true }],
    BRID:  [{ value:'', disabled: true }],
    /**Client */
    CSTDEFID:  [{ value:'', disabled: true }],
    CSTCCTID:  [{ value:'', disabled: true }],
    CSTARTID:  [{ value:'', disabled: true }],
    /** Address Facturation */
    CSTFACVILLE:  [{ value:'', disabled: true }],
    CSTFACCODPOS:  [{ value:'', disabled: true }],
    CSTFACPAYCOD: [{ value:'', disabled: true }],
    /** Address Expedition */
    CSTEXPVILLE:  [{ value:'', disabled: true }],
    CSTEXPCODPOS:  [{ value:'', disabled: true }],
    CSTEXPPAYCOD:  [{ value:'', disabled: true }],
    CSTEXPADDR:  [{ value:'', disabled: true }],
    CSTFACADDR: [{ value:'', disabled: true }],

    /** Supplier */
    SSTARTID: [{ value:'', disabled: true }],
    SSTNOM: [{ value:'', disabled: true }],
    SSTCCTID: [{ value:'', disabled: true }],
    SSTDELIV: [{ value:'', disabled: true }],
    SSTCOMM: [{ value:'', disabled: true }],
    SSTEXPADDR: [{ value:'', disabled: true }],
    SSTEXPCODPOS: [{ value:'', disabled: true }],
    SSTEXPVILLE: [{ value:'', disabled: true }],
    SSTEXPPAYCOD: [{ value:'', disabled: true }],
    SSTFACCODPOS: [{ value:'', disabled: true }],
    SSTFACVILLE: [{ value:'', disabled: true }],
    SSTFACADDR: [{ value:'', disabled: true }],
    SSTFACPAYCOD: [{ value:'', disabled: true }],
  });

  store_id: number | undefined;
  store_type: string | undefined;
  storeDetail: Store | undefined;
  imgURL: string | undefined;

  constructor(private formBuilder: FormBuilder,
              private contactService: ContactService,
              private dataSharingService: DataSharingService,
              private dialog: MatDialog,
              private route: ActivatedRoute,
              private router: Router,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
  }

  ngOnInit(): void {
    this.route.params.subscribe(params => {
      this.store_id = +params["store_id"];
      this.store_type = params["store_type"];

      this.loadDependencies();
    });
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this.getStoreDetail(this.store_id, this.store_type);
    this._preFillStoreDetails(this.storeDetail);

    await this.loadingService.hide();
  }

  getStoreDetail(id: any, type: any): Promise<any> {
    return new Promise<any>((resolve, reject) => {

      const payload = {
        store_id: id,
        store_type: type
      };

      const observer = {
        next: (res: CustomHttpResponse<Store>) => {
          if(res.status === 'success') {
            this.imgURL = res.imageurl;
            this.storeDetail = res.data;
            resolve(res);
          } else {
            this.loadingService.hide();
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
            reject(res);
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR',
          });
          reject(err);
        }
      }

      this.contactService.getStoreDetail(payload).subscribe(observer);
    })
  }

  _preFillStoreDetails(storeDetail?: Store) {
    if(storeDetail === undefined || storeDetail === null) {
      return;
    }

    if(this.store_type === 'client') {
      this.createStoreForm.get('CSTNOM')?.setValue(storeDetail.store_nom);
      this.createStoreForm.get('CSTDEFID')?.setValue(storeDetail.defriement_lbl);
      this.createStoreForm.get('CSTEXPADDR')?.setValue(storeDetail.store_address);
      this.createStoreForm.get('CSTEXPCODPOS')?.setValue(storeDetail.store_code_postal);
      this.createStoreForm.get('CSTEXPPAYCOD')?.setValue(storeDetail.store_pays);
      this.createStoreForm.get('CSTEXPVILLE')?.setValue(storeDetail.store_ville);
      this.createStoreForm.get('CSTFACADDR')?.setValue(storeDetail.storefac_address);
      this.createStoreForm.get('CSTFACCODPOS')?.setValue(storeDetail.storefac_code_postal);
      this.createStoreForm.get('CSTFACVILLE')?.setValue(storeDetail.storefac_ville);
      this.createStoreForm.get('CSTFACPAYCOD')?.setValue(storeDetail.storefac_pays);
      this.createStoreForm.get('BRID')?.setValue(storeDetail.store_brand);
      this.createStoreForm.get('CSTCCTID')?.setValue(storeDetail.store_category);
    }

    if(this.store_type === 'supplier') {
      this.createStoreForm.get('SSTNOM')?.setValue(storeDetail.store_nom);
      this.createStoreForm.get('SSTCCTID')?.setValue(storeDetail.store_category);
      this.createStoreForm.get('SSTDELIV')?.setValue(storeDetail.store_de_livraison);
      this.createStoreForm.get('SSTCOMM')?.setValue(storeDetail.store_de_commande);
      this.createStoreForm.get('SSTEXPADDR')?.setValue(storeDetail.store_address);
      this.createStoreForm.get('SSTEXPCODPOS')?.setValue(storeDetail.store_code_postal);
      this.createStoreForm.get('SSTEXPVILLE')?.setValue(storeDetail.store_ville);
      this.createStoreForm.get('SSTEXPPAYCOD')?.setValue(storeDetail.store_pays);
      this.createStoreForm.get('SSTFACADDR')?.setValue(storeDetail.storefac_address);
      this.createStoreForm.get('SSTFACCODPOS')?.setValue(storeDetail.storefac_code_postal);
      this.createStoreForm.get('SSTFACVILLE')?.setValue(storeDetail.storefac_ville);
      this.createStoreForm.get('SSTFACPAYCOD')?.setValue(storeDetail.storefac_pays);
      this.createStoreForm.get('BRID')?.setValue(storeDetail.store_brand);
    }
  }

  get isClient(): boolean {
    return (this.store_type !== undefined && this.store_type === 'client');
  }

  get isSupplier(): boolean {
    return (this.store_type !== undefined && this.store_type === 'supplier');
  }

  get fr() {
    return this.createStoreForm.controls;
  }

  storeStatus(status: number):boolean {
    return status === APP_CONSTANTS.STORE_HQ_STAT_YES;
  }

  goToArticle(id: number) {
    this.router.navigate(['/article/view-article', id]);
  }

  goToContact(id: number) {
    this.router.navigate(['/contact/detail', id]);
  }

  goToBrand(id: number) {
    this.router.navigate(['/contact/brand/detail', id]);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_CONTACT_LIST_FORM);
  }

  async openFusionerPopup() {
    const data = {
      currentStore: this.storeDetail,
    };

    const dialog = await this.dialog.open(FusionerStoreComponent, {
      data,
      maxHeight: "90vh",
      minWidth: "70vw"
    });

    dialog.afterClosed().subscribe((res) => {
      if(res !== undefined && res !== null && res.status === 'success') {
        const fusionnerType = res.fusionnerType;

        if(fusionnerType === APP_CONSTANTS.FUSIONNER_BOTH) {
          return;
        }

        if(fusionnerType === APP_CONSTANTS.FUSIONNER_Y_TO_X) {
          this.storeDetail = res.data;
          this._preFillStoreDetails(this.storeDetail);

          this.__resetListCache();
          return;
        }

        if(fusionnerType === APP_CONSTANTS.FUSIONNER_X_TO_Y) {
          this.storeDetail = res.data;

          const queryParams = {
            id: this.storeDetail?.store_id,
            type: this.storeDetail?.store_type
          };

          this.router.navigate(
            [],
            {
              relativeTo: this.route,
              queryParams: queryParams,
              queryParamsHandling: 'merge', // remove to replace all query params by provided
            });

          this._preFillStoreDetails(this.storeDetail);

          this.__resetListCache();
          return;
        }
      }
    });
  }

}
