import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { APP_CONSTANTS } from '../../../_helpers/constants';
import { CustomHttpResponse } from '../../../_models';
import { CommonService } from '../../../_services/common.service';
import { DialogService } from '../../../_services/dailog.service';
import { DepotTntService } from '../../../_services/depot-tnt.service';

@Component({
  selector: 'app-add-depot-tnt',
  templateUrl: './add-depot-tnt.component.html',
  styleUrls: ['./add-depot-tnt.component.scss']
})
export class AddDepotTntComponent implements OnInit {

  createDepotTntForm: FormGroup = this.formBuilder.group({
    label: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_LABEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_LABEL_MAX_LENGTH)]],
    address: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_ADDRESS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_ADDRESS_MAX_LENGTH)]],
    postal_code: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_POSTAL_CODE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_POSTAL_CODE_MAX_LENGTH)]],
    ville: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_VILLE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_VILLE_MAX_LENGTH)]],
    pays: ['FR', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_PAYS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_PAYS_MAX_LENGTH)]],
    status: ['', Validators.required]
  });
  submitted: boolean = false;
  statusList = APP_CONSTANTS.DEPOT_TNT_INFO_STATUS;
  paysList: any = [];
  filteredPostalCodes: any;
  filteredVilles: any =[];

  constructor(private formBuilder: FormBuilder,
              private translateService: TranslateService,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private commonService: CommonService,
              private depotTntService: DepotTntService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.getPaysList();
    await this.loadingService.hide();
  }

  getPaysList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.paysList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListPays().subscribe(observer);
    });
  }

  resetVille() {
      this.createDepotTntForm.get('ville')?.reset();
  }

  get fr() {
    return this.createDepotTntForm.controls;
  }

  get expeditionVille() {
    return this.createDepotTntForm.get("ville")?.value;
  }

  get expeditionCodePostal() {
    return this.createDepotTntForm.get("postal_code")?.value;
  }

  get facturationPays() {
    return this.createDepotTntForm.get("pays")?.value;
  }

  resetCodePostal() {
    this.createDepotTntForm.get('postal_code')?.reset()
  }

  onCodeOptionSelected(event:any) {
    this.createDepotTntForm.get('postal_code')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
    this.createDepotTntForm.get('ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
  }

  onVilleOptionSelected(event:any) {
      this.createDepotTntForm.get('postal_code')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.createDepotTntForm.get('ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
  }

  onVilleChanged() {
    let ville = undefined,
    pays = undefined;
    ville = this.createDepotTntForm.get('ville')?.value;
    pays = this.facturationPays;

    if(ville !== '' && ville != null) {
      const payload = {
        ville,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            this.filteredVilles = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  onCodePostalChanged() {
    let postalCode = undefined,
    pays = undefined;
    postalCode = this.createDepotTntForm.get('postal_code')?.value;
    pays = this.facturationPays;

    if(postalCode !== '' && postalCode != null) {
      const payload = {
        codpos: postalCode,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
              this.filteredPostalCodes = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };
      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST_FORM);
  }


  createDepotTntFormSubmitter() {
    this.submitted = true;
    if (this.createDepotTntForm.invalid) {
      return;
    }
    const payload = this.createDepotTntForm.getRawValue();
    this._createDepotTnt(payload);
  }

  async _createDepotTnt(payload: any) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.createDepotTntForm.reset();
          this.submitted = false;
          this.router.navigate(['/options/depot-tnt/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.depotTntService.createDepotTnt(payload).subscribe(observer);
  }
}
