import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { firstValueFrom } from 'rxjs';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { APP_CONSTANTS } from '../../../_helpers/constants';
import { CustomHttpResponse, DepotTNT } from '../../../_models';
import { CommonService } from '../../../_services/common.service';
import { DialogService } from '../../../_services/dailog.service';
import { DepotTntService } from '../../../_services/depot-tnt.service';

@Component({
  selector: 'app-edit-depot-tnt',
  templateUrl: './edit-depot-tnt.component.html',
  styleUrls: ['./edit-depot-tnt.component.scss']
})
export class EditDepotTntComponent implements OnInit {

  modifyDepotTntForm: FormGroup = this.formBuilder.group({
    label: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_LABEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_LABEL_MAX_LENGTH)]],
    address: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_ADDRESS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_ADDRESS_MAX_LENGTH)]],
    postal_code: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_POSTAL_CODE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_POSTAL_CODE_MAX_LENGTH)]],
    ville: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_VILLE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_VILLE_MAX_LENGTH)]],
    pays: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.DEPOT_TNT_PAYS_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.DEPOT_TNT_PAYS_MAX_LENGTH)]],
    status: ['', Validators.required]
  });
  submitted: boolean = false;
  statusList = APP_CONSTANTS.DEPOT_TNT_INFO_STATUS;
  paysList: any = [];
  filteredPostalCodes: any;
  filteredVilles: any =[];
  depot_tnt_id:number | undefined;
  readOnly:boolean = false;
  pageTitle: string='KDEPOT_TNT_MODIFY'
  editDepotTntInfo:any=[]
  constructor(private formBuilder: FormBuilder,
              private translateService: TranslateService,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private commonService: CommonService,
              private depotTntService: DepotTntService,
              private route: ActivatedRoute,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.depot_tnt_id = +params['depot_tnt_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        console.log(this.readOnly)
        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.getPaysList();
    await this._getDepotTntInfoDetail(this.depot_tnt_id);
    this.prefillFormDetail(this.editDepotTntInfo);
    await this.loadingService.hide();
  }

  _getDepotTntInfoDetail(depot_tnt_id:number | undefined): Promise<any>{
    return new Promise((resolve, reject) => {
      this.depotTntService.viewDepotTntDetail(depot_tnt_id).subscribe({
        next: (res: CustomHttpResponse<DepotTNT>) => {
          if(res.status === 'success') {
            this.editDepotTntInfo = res.data;
          }else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  prefillFormDetail(editDepotTntInfo: DepotTNT | undefined) {
    if(editDepotTntInfo === undefined || editDepotTntInfo === null) {
      return;
    }
    this.modifyDepotTntForm.patchValue({
      label: editDepotTntInfo.label,
      address: editDepotTntInfo.address,
      postal_code: editDepotTntInfo.postal_code,
      ville: editDepotTntInfo.ville,
      pays: editDepotTntInfo.pays,
      status: editDepotTntInfo.status
    });
  }

  _changeToDetailMode() {
    this.pageTitle = 'KDEPOT_TNT_DETAIL';
    this.modifyDepotTntForm.get('label')?.disable({ onlySelf: true });
    this.modifyDepotTntForm.get('address')?.disable({ onlySelf: true });
    this.modifyDepotTntForm.get('postal_code')?.disable({ onlySelf: true });
    this.modifyDepotTntForm.get('ville')?.disable({ onlySelf: true });
    this.modifyDepotTntForm.get('pays')?.disable({ onlySelf: true });
    this.modifyDepotTntForm.get('status')?.disable({ onlySelf: true });
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST);
    if(listCache !== undefined) {
      let listDepotTNT = listCache.data as DepotTNT[];
      listDepotTNT = listDepotTNT.map((row) => {
        if(row.id == this.depot_tnt_id) {
          row.id = this.depot_tnt_id;
          row.label = payload.label
          row.address = payload.address
          row.postal_code = payload.postal_code
          row.ville = payload.ville
          row.pays = payload.pays
          row.status = payload.status
        }
        return row;
      });

      listCache.data = listDepotTNT;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_DEPOT_TNT_LIST, JSON.stringify(listCache));
    }
  }

  getPaysList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<any[]>) => {
          if(res.status === 'success') {
            this.paysList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListPays().subscribe(observer);
    });
  }

  resetVille() {
      this.modifyDepotTntForm.get('ville')?.reset();
  }

  get fr() {
    return this.modifyDepotTntForm.controls;
  }

  get expeditionVille() {
    return this.modifyDepotTntForm.get("ville")?.value;
  }

  get expeditionCodePostal() {
    return this.modifyDepotTntForm.get("postal_code")?.value;
  }

  get facturationPays() {
    return this.modifyDepotTntForm.get("pays")?.value;
  }

  resetCodePostal() {
    this.modifyDepotTntForm.get('postal_code')?.reset()
  }

  onCodeOptionSelected(event:any) {
    this.modifyDepotTntForm.get('postal_code')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
    this.modifyDepotTntForm.get('ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
  }

  onVilleOptionSelected(event:any) {
      this.modifyDepotTntForm.get('postal_code')?.setValue(event.option.value.CPCODPOS, { onlySelf: true });
      this.modifyDepotTntForm.get('ville')?.setValue(event.option.value.CPVILLE, { onlySelf: true });
  }

  onVilleChanged() {
    let ville = undefined,
    pays = undefined;
    ville = this.modifyDepotTntForm.get('ville')?.value;
    pays = this.facturationPays;

    if(ville !== '' && ville != null) {
      const payload = {
        ville,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
            this.filteredVilles = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };

      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }

  onCodePostalChanged() {
    let postalCode = undefined,
    pays = undefined;
    postalCode = this.modifyDepotTntForm.get('postal_code')?.value;
    pays = this.facturationPays;

    if(postalCode !== '' && postalCode != null) {
      const payload = {
        codpos: postalCode,
        codpays: pays
      };

      const observer = {
        next: (res: CustomHttpResponse<any>) => {
          if(res.status === 'success') {
              this.filteredPostalCodes = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            });
          }
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
        }
      };
      this.commonService.getListCodPosVille(payload).subscribe(observer);
    }
  }


  modifyDepotTntFormSubmitter() {
    this.submitted = true;
    if (this.modifyDepotTntForm.invalid) {
      return;
    }
    const payload = this.modifyDepotTntForm.getRawValue();
    this._createDepotTnt(payload);
  }

  async _createDepotTnt(payload: any) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.modifyDepotTntForm.reset();
          this.submitted = false;
          this.router.navigate(['/options/depot-tnt/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.depotTntService.modifyDepotTnt(payload, this.depot_tnt_id).subscribe(observer);
  }
}
