import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators, FormArray } from '@angular/forms';
import {DatePipe, Location} from '@angular/common';
import { AuthService } from '../../../_services/auth.service';
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { CustomHttpResponse, Fabriquant } from 'src/app/_models';
import { NgxSpinnerService } from 'ngx-spinner';
import { DialogService } from 'src/app/_services/dailog.service';
import { FabriquantService } from 'src/app/_services/fabriquant.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-create-fabriquant',
  templateUrl: './create-fabriquant.component.html',
  styleUrls: ['./create-fabriquant.component.scss']
})
export class CreateFabriquantComponent implements OnInit {

  createFabForm:FormGroup = this.formBuilder.group({
    MNFLBL: new FormControl('', [Validators.required, Validators.minLength(APP_CONSTANTS.FABRIQUANT_LABEL_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.FABRIQUANT_LABEL_MAX_LENGTH)]),
    MNFSTAT: new FormControl('', [Validators.required]),
  });
  Submitted: boolean = false;
  loading:boolean = false;
  statusList = APP_CONSTANTS.FABRIQUANT_STATUS;

  constructor(private formBuilder: FormBuilder,
              private translateService:TranslateService,
              private router: Router,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private fabriquantService:FabriquantService,
              private dataSharingService:DataSharingService) {}

  ngOnInit(): void {
  }

  get fr() {
    return this.createFabForm.controls;
  }

  createLigneFormSubmitter() {
    this.Submitted = true;
    if(this.createFabForm.invalid) {
      return;
    }

    const formData = this.createFabForm.getRawValue();
    this._addLigneInfo(formData);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST_FORM);
  }

  async _addLigneInfo(payload: Fabriquant) {
    const observer = {
      next: async (res: CustomHttpResponse<Fabriquant>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/options/fabriquant/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.fabriquantService.createFabriquant(payload).subscribe(observer);
  }


}
