import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import { AuthService } from '../../../_services/auth.service';
import { TranslateService } from '@ngx-translate/core';
import { Router, ActivatedRoute } from '@angular/router';
import { firstValueFrom } from 'rxjs';
import { NgxSpinnerService } from 'ngx-spinner';
import { CustomHttpResponse, Fabriquant } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { FabriquantService } from 'src/app/_services/fabriquant.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-edit-fabriquant',
  templateUrl: './edit-fabriquant.component.html',
  styleUrls: ['./edit-fabriquant.component.scss']
})
export class EditFabriquantComponent implements OnInit {

  editFabForm:FormGroup = this.formBuilder.group({
    MNFLBL: new FormControl('', [Validators.required, Validators.minLength(APP_CONSTANTS.FABRIQUANT_LABEL_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.FABRIQUANT_LABEL_MAX_LENGTH)]),
    MNFSTAT: new FormControl('', [Validators.required]),
  });
  fab_id:number | undefined;
  readOnly:boolean=false;
  fabriquantInfo:any=[];
  pageTitle: string = 'KMODIFY_FABRIQUANT';
  statusList = APP_CONSTANTS.FABRIQUANT_STATUS;
  Submitted: boolean = false;
  loading:boolean = false;

  constructor(private formBuilder: FormBuilder,
              private translateService:TranslateService,
              private loadingService: NgxSpinnerService,
              private router: Router,
              private dialogService: DialogService,
              private fabriquantService:FabriquantService,
              private route: ActivatedRoute,
              private dataSharingService:DataSharingService) {}

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.fab_id = +params['fab_id'];
      console.log(this.fab_id)
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];

        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }

  get fr() {
    return this.editFabForm.controls;
  }


  async loadDependencies() {
    await this.loadingService.show();
    await this._getFabriquantDetail(this.fab_id);
    this._prefillFormData();
    await this.loadingService.hide();
  }

  _getFabriquantDetail(fab_id: number | undefined): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      if(fab_id === undefined || fab_id === null) {
        reject();
        return;
      }

      const observer = {
        next: (res: CustomHttpResponse<Fabriquant>) => {
          if(res.status === "success") {
            this.fabriquantInfo = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
          messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/options/fabriquant/list']);
              }
            );
          }
          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      };
      this.fabriquantService.getFabriquantDetailById(fab_id).subscribe(observer);
    })
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST);
    if(listCache !== undefined) {
      let listFabriquant = listCache.data as Fabriquant[];
      listFabriquant = listFabriquant.map((row) => {
        console.log(row)
        console.log(payload)
        if(row.id == this.fab_id) {
          row.id = this.fab_id;
          row.label = payload.MNFLBL;
          row.status = payload.MNFSTAT;
        }
        return row;
      });

      listCache.data = listFabriquant;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST, JSON.stringify(listCache));
    }
  }


  _prefillFormData() {
    this.editFabForm.patchValue({
      MNFLBL: this.fabriquantInfo?.MNFLBL,
      MNFSTAT: this.fabriquantInfo?.MNFSTAT,
    })
  }

  _changeToDetailMode() {
    this.pageTitle = 'KDETAIL_FABRIQUANT';
    this.editFabForm.get('MNFLBL')?.disable({ onlySelf: true });
    this.editFabForm.get('MNFSTAT')?.disable({ onlySelf: true });
  }

  modifyFabriquantFormSubmitter() {
    this.Submitted = true;
    if(this.editFabForm.invalid) {
      return;
    }

    const formData = this.editFabForm.getRawValue();

    this._modifyFabInfo(formData);
  }

  async _modifyFabInfo(payload: Fabriquant) {
    if(this.fab_id === undefined || this.fab_id === null) {
      return;
    }

    const observer = {
      next: async (res: CustomHttpResponse<Fabriquant>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/options/fabriquant/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
          messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.fabriquantService.modifyFabriquantById(this.fab_id, payload).subscribe(observer);
  }


}
