import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl } from '@angular/forms';
import {PageEvent} from '@angular/material/paginator';
import { MatDialog } from '@angular/material/dialog';
import { TranslateService } from '@ngx-translate/core';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { DialogService } from 'src/app/_services/dailog.service';
import { CustomHttpResponse, Fabriquant } from 'src/app/_models';
import { NgxSpinnerService } from 'ngx-spinner';
import { FabriquantService } from 'src/app/_services/fabriquant.service';
import { lastValueFrom } from 'rxjs';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-fabriquant-list',
  templateUrl: './fabriquant-list.component.html',
  styleUrls: ['./fabriquant-list.component.scss']
})
export class FabriquantListComponent implements OnInit {

  fabriquantList:Fabriquant[]=[];
  Submitted: boolean = false;
  loading:boolean = false;
  searchForm:FormGroup = this.formBuilder.group({
    keyword: new FormControl(''),
  });
  isLoading = false;
  displayedColumnsFabriquant: string[] = ['MNFLBL' ,'MNFSTAT' ,'actions'];
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  rolePermission:any;

  constructor(private formBuilder: FormBuilder,
              public dialog: MatDialog,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService,
              private dialogService: DialogService,
              private fabriquantService:FabriquantService,
              private dataSharingService:DataSharingService) {}

  ngOnInit(): void {
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST);
      if(dataCache !== undefined) {

        this.fabriquantList = dataCache.data as Fabriquant[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
    this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    await this.loadingService.hide();
  }

  get fr() {
    return this.searchForm.controls;
  }

  documentStatus(item: Fabriquant) {
    const statusList = APP_CONSTANTS.FABRIQUANT_STATUS;

    const statusData = statusList.find((r) => r.value == item.status);
    let result = "";

    if(statusData == undefined) {
      return result;
    }

    switch (item.status) {
      case APP_CONSTANTS.FABRIQUANT_ACTIVE:
        result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
      case APP_CONSTANTS.FABRIQUANT_IN_ACTIVE:
        result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
        break;
    }

    return result;
  }
  
  searchFormSubmitter(){
    this.Submitted = true;
    if(this.searchForm.invalid){
      return;
    }
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword
    };
    this.currentPage = 1;
    this._searchFabriquantInfo(payload, this.currentPage, this.pageSize);
  }

  _searchFabriquantInfo(data: { keyword: string }, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<Fabriquant[]>) => {
        if(res.status === "success") {
          this.fabriquantList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.fabriquantService.searchFabriquantInfo(data, page, pageSize).subscribe( observer);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST);
    if(listCache !== undefined) {
      let listContacts: any[] = listCache.data;
      listContacts = (listContacts as Fabriquant[]).filter(r => r.id != id);

      listCache.data = listContacts;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_FABRIQUANT_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listContacts.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword
    };

    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchFabriquantInfo(payload, this.currentPage, this.pageSize);
  }

  confirmRemoveFabriquant(fab_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: fab_label })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeFabriquant(fab_id: number, fab_label:string) {
    if(!this.rolePermission['9_24']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveFabriquant(fab_label);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteFabriquantInfo(fab_id);
    }

  }

  async _deleteFabriquantInfo(fab_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(fab_id);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.fabriquantService.deleteFabriquantById(fab_id).subscribe(observer);
  }
}
