import {Component, Inject, OnInit, ViewChild} from '@angular/core';
import {FormBuilder, FormControl, FormGroup, NgForm, Validators} from "@angular/forms";
import {MAT_DIALOG_DATA, MatDialogRef} from "@angular/material/dialog";
import {debounceTime, lastValueFrom, of, startWith, switchMap} from 'rxjs';
import {CommonService} from "../../../_services/common.service";
import {DialogService} from "../../../_services/dailog.service";
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { TranslateService } from '@ngx-translate/core';
import {MatCheckboxChange} from "@angular/material/checkbox";
import {BlockSerialList, NumeroSerialList} from "../../../_models";

@Component({
  selector: 'app-numero-serie',
  templateUrl: './numero-serie.component.html',
  styleUrls: ['./numero-serie.component.scss']
})
export class NumeroSerieComponent implements OnInit {


  @ViewChild('serialFormEl') private serialFormEl: NgForm | undefined;

  serialsForm: FormGroup = this.formBuilder.group({
    SERIAL: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.SERIAL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.SERIAL_MAX_LENGTH)]]
  });

  listSerials: NumeroSerialList[] = [];
  newSerialsList: NumeroSerialList[] = [];
  removeSerialsList: NumeroSerialList[] = [];
  previousSerialsList: NumeroSerialList[] = [];

  blockedInSerialsList: BlockSerialList[] = [];
  blockedOutSerialsList: BlockSerialList[] = [];

  submitted: boolean = false;
  serialList: NumeroSerialList[] = [];
  selectedOptions: any;
  selectedOptionsBackup: any;

  search = new FormControl();
  occasionFilter = new FormControl(false);

  readonly: boolean = false;
  numeroType: any;
  reportingDocumentType: string | undefined;

  // Input Mask
  serialMask = APP_CONSTANTS.SERIAL_MASK;

  maxSerialsLimit: number | undefined;

  constructor(@Inject(MAT_DIALOG_DATA) public data: any,
              public dialogRef: MatDialogRef<NumeroSerieComponent>,
              private formBuilder: FormBuilder,
              private commonService: CommonService,
              private dialogService: DialogService,
              private translateService: TranslateService) {
  }

  ngOnInit(): void {
    this.submitted = false;
    this.numeroType = this.data.numeroType;

    if(this.data && this.data.hasOwnProperty('readonly')) {
      this.readonly = this.data.readonly;
    }

    if(this.data && this.data.hasOwnProperty('block_in_serials')) {
      const blockedInSerials = this.data.block_in_serials || "";
      this.blockedInSerialsList = this.commonService.formatBlockedSerialNumbers(blockedInSerials);
    }

    if(this.data && this.data.hasOwnProperty('block_out_serials')) {
      const blockedOutSerials = this.data.block_out_serials || "";
      this.blockedOutSerialsList = this.commonService.formatBlockedSerialNumbers(blockedOutSerials);
    }

    if(this.data && this.data.hasOwnProperty('maxSerialsLimit')) {
      this.maxSerialsLimit = this.data['maxSerialsLimit'];
    }

    if(this.numeroType === "livraison-numero-series" || this.numeroType === "transfer-de-stock-numero-series") {
      if(this.data && this.data.oldListSerials) {
        this.listSerials = this.data.oldListSerials as NumeroSerialList[];

        if(this.blockedOutSerialsList.length > 0) {
          this.listSerials = this.listSerials.map((row) => {

            const blockedExist = this.blockedOutSerialsList.find((_r) => _r.serial_no == row.serial_no);

            if(blockedExist) {
              row.disabled = true;
              row.document_no = blockedExist.document_no;
              row.document_type = blockedExist.document_type;
            }

            return row;
          })
        }

        this.serialList = this.listSerials;
      }

      if(this.data && this.data.hasOwnProperty('selectedSerials')) {
        this.selectedOptions = this.data.selectedSerials;
      }

      if(this.data && this.data.hasOwnProperty('selectedSerialsBackup')) {
        this.selectedOptionsBackup = this.data.selectedSerialsBackup;

        this.listSerials = this.listSerials.map((row) => {
          if(this.selectedOptionsBackup != undefined && this.selectedOptionsBackup.length > 0 && this.selectedOptionsBackup.indexOf(row.serial_no) > -1) {
            row.disabled = false;
          }

          return row;
        })
      }
    }

    if(this.numeroType === "reception-numero-series" || this.numeroType === "achat-commande-numero-series") {
      if(this.data && this.data.oldListSerials) {
        this.listSerials = this.data.oldListSerials as NumeroSerialList[];
        this.previousSerialsList = [...this.listSerials];
      }
    }

    if (this.numeroType === "inventory-numero-series" || this.numeroType === "inventory-tournant-numero-series") {
      if(this.data && this.data.oldListSerials) {
        this.previousSerialsList = this.data.oldListSerials as NumeroSerialList[];

        if(this.blockedOutSerialsList.length > 0) {
          this.previousSerialsList = this.previousSerialsList.map((row) => {

            const blockedExist = this.blockedOutSerialsList.find((_r) => _r.serial_no == row.serial_no);

            if(blockedExist) {
              row.disabled = true;
              row.document_no = blockedExist.document_no;
              row.document_type = blockedExist.document_type;
            }

            return row;
          })
        }
      }

      if(this.data && this.data.newListSerials) {
        this.listSerials = this.data.newListSerials as NumeroSerialList[];
        this.newSerialsList = [...this.listSerials];
      }

      if(this.data && this.data.removeListSerials) {
        this.removeSerialsList = this.data.removeListSerials as NumeroSerialList[];

        this.previousSerialsList = this.previousSerialsList.filter((r) => {
          const checkIfExists = this.removeSerialsList.find((cr) => cr.serial_no == r.serial_no);
          if (checkIfExists == undefined) {
            return true;
          }

          return false;
        });
      }
    }

    if (this.numeroType === 'reporting-numero-series') {
      if(this.data && this.data.reportingDocumentType) {
        this.reportingDocumentType = this.data.reportingDocumentType;
      }

      if(this.reportingDocumentType === 'inventory' || this.reportingDocumentType === 'tournant') {
        if(this.data && this.data.newListSerials) {
          this.listSerials = [...this.data.newListSerials] as NumeroSerialList[];
          this.newSerialsList = [...this.data.newListSerials] as NumeroSerialList[];
        }

        if(this.data && this.data.removeListSerials) {
          this.removeSerialsList = [...this.data.removeListSerials] as NumeroSerialList[];
        }

        this.previousSerialsList = this.data.oldListSerials as NumeroSerialList[];
      } else if (this.reportingDocumentType === 'livraison') {
        if(this.data && this.data.newListSerials) {
          this.listSerials = this.data.newListSerials as NumeroSerialList[];
          this.serialList = [...this.listSerials];
          this.selectedOptions = [...this.serialList].map(r => r.serial_no);
        }
      } else if (this.reportingDocumentType === 'reception') {
        if(this.data && this.data.newListSerials) {
          this.listSerials = this.data.newListSerials as NumeroSerialList[];
          this.previousSerialsList = [...this.listSerials];
        }
      } else if (this.reportingDocumentType === 'tds') {
        if(this.data && this.data.newListSerials) {
          this.listSerials = this.data.newListSerials as NumeroSerialList[];
          this.serialList = [...this.listSerials];
          this.selectedOptions = [...this.serialList].map(r => r.serial_no);
        }
      } else {
        // Default Type
        if(this.data && this.data.newListSerials) {
          this.listSerials = this.data.newListSerials as NumeroSerialList[];
          this.previousSerialsList = [...this.listSerials];
        }
      }
    }
  }

  $search = this.search.valueChanges.pipe(
    startWith(null),
    debounceTime(200),
    switchMap((res: string) => {
      if (!res) {
        if(this.occasionFilter.value) {
          return of(this.serialList.filter((x: any) => x.serial_tag != null));
        }

        return of(this.serialList);
      }

      res = res.toLowerCase();

      return of(
        this.serialList.filter((x:any) => {
          if(this.occasionFilter.value) {
            return x.serial_no.toLowerCase().indexOf(res) >= 0 && x.serial_tag != null;
          }

          return x.serial_no.toLowerCase().indexOf(res) >= 0;
        })
      );
    })
  );

  onNgModelChange(event:any) {
    this.selectedOptions = event;
  }

  onOccasionCheckBoxChange() {
    // Triggering the Search Input to filter the serials list by Occasion filter
    this.search.setValue('');
  }

  get fr() {
    return this.serialsForm.controls;
  }

  get serialInput() {
    return this.serialsForm.get('SERIAL') as FormControl;
  }

  addSerialToStack() {
    this.submitted = true;
    if(this.serialsForm.invalid) {
      return;
    }

    const serialData = this.serialInput.value;

    if(this.listSerials.find(_r => _r.serial_no.toUpperCase() === serialData.toUpperCase())) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: `<h3>${this.translateService.instant('KSERIAL_NOT_UNIQUE', { SERIAL:serialData })}</h3>`

      });
      return;
    }

    if(this.numeroType === "inventory-numero-series" || this.numeroType === "inventory-tournant-numero-series") {
      if(this.previousSerialsList.find((_r) => _r.serial_no.toUpperCase() === serialData.toUpperCase() && !_r.disabled)) {
        this.previousSerialsList = this.previousSerialsList.map((row) => {
          if(row.serial_no.toUpperCase() === serialData.toUpperCase()) {
            row.verified = true;
          }

          return row;
        })

        this.serialFormEl?.resetForm();
        this.submitted = false;
        return;
      }
    }

    this.listSerials.push({
      serial_no: serialData.toUpperCase(),
      serial_tag: null,
      verified: false,
      disabled: false
    });

    this.serialFormEl?.resetForm();
    this.submitted = false;
  }

  removeSerial(index: number, type?: 'new' | 'old' | 'removed') {
    if(this.numeroType === "inventory-numero-series" || this.numeroType === "inventory-tournant-numero-series") {
      if(type === 'new') {
        this.listSerials = this.listSerials.filter((r, i) => i !== index);
      }

      if(type === 'old') {
        this.removeSerialsList = [
          ...this.removeSerialsList,
          this.previousSerialsList[index]
        ];

        this.previousSerialsList = this.previousSerialsList.filter((r, i) => i !== index);
      }

      if(type === 'removed') {
        const undoItem = this.removeSerialsList[index];
        this.removeSerialsList = this.removeSerialsList.filter((r,i) => i !== index);
        this.previousSerialsList.unshift({
          serial_no: undoItem.serial_no,
          serial_tag: undoItem.serial_tag,
          verified: false,
          disabled: false
        });
      }

      return;
    }

    this.listSerials = this.listSerials.filter((r, i) => i !== index);
  }

  _checkForSerialDuplicates(serialsList: NumeroSerialList[]): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const SNUMBERS = [...serialsList].map(r => r.serial_no);
      const data = {
        SNUMBERS
      };

      this.commonService.checkSerialsDuplication(data).subscribe(
        (resp: any) => {
          if(resp.status === "failure") {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: resp.message,
              messageTranslateParam: resp.data
            });

            resolve({ error: true });
          }

          resolve({ error: false });
        },
        err => {
          reject(err);
        }
      )
    })
  }

  // Only if(this.numeroType == "inventory-numero-series" || this.numeroType == "inventory-tournant-numero-series")
  confirmRemoveUnVerifiedSerials(): Promise<any> {
    const message = this.translateService.instant('KMOVE_UNVERIFIED_TO_SUPPRIMER');

    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KCONFIRMATION_NEEDED',
      message: message
    }, false));
  }

  // Only if(this.numeroType == "inventory-numero-series" || this.numeroType == "inventory-tournant-numero-series")
  get isBothVerifiedUnVerifiedSerialsAvail(): boolean {
    return this.previousSerialsList.some(r => r.verified) && this.previousSerialsList.some(r => !r.verified);
  }

  // Only if(this.numeroType == "inventory-numero-series" || this.numeroType == "inventory-tournant-numero-series")
  async moveUnverifiedToSupprimer(confirmationNeeded: boolean = false) {
    if(this.isBothVerifiedUnVerifiedSerialsAvail) {
      let confirmation: boolean;
      if(confirmationNeeded) {
        confirmation = await this.confirmRemoveUnVerifiedSerials();

        if(confirmation === undefined || confirmation === null) {
          // If Confirmationn popup, closed without confirmation
          return;
        }
      } else {
        confirmation = true;
      }

      if(confirmation) {
        let unVerifiedSerialList = [...this.previousSerialsList];
        unVerifiedSerialList = unVerifiedSerialList.filter((r) => !r.verified);

        this.removeSerialsList = [
          ...this.removeSerialsList,
          ...unVerifiedSerialList
        ];

        this.previousSerialsList = this.previousSerialsList.filter((r) => r.verified);
      }
    }
  }

  async submitSerials() {
    if(this.maxSerialsLimit !== undefined) {
      if(this.listSerials.length > this.maxSerialsLimit) {
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KVALIDATE_MAX_LIMIT_REACHED',
          messageTranslateParam: { MAX_LENGTH: this.maxSerialsLimit, ACTUAL_LENGTH: this.listSerials.length }
        });
        return;
      }
    }

    if(this.numeroType == "reception-numero-series" || this.numeroType === "achat-commande-numero-series") {
      let newlyEnteredSerials = this.listSerials.filter(item => {
        if(this.previousSerialsList.map(r => r.serial_no).indexOf(item.serial_no) < 0) {
          return true;
        }

        return false;
      });

      if(newlyEnteredSerials !== undefined && newlyEnteredSerials !== null && newlyEnteredSerials.length > 0) {
        const checkDuplicate = await this._checkForSerialDuplicates(newlyEnteredSerials);
        if(checkDuplicate.hasOwnProperty('error') && checkDuplicate.error === true) {
          return;
        }

        // Check inside the Blocked-In serials for duplicates
        const foundBlockedDuplicates = this.blockedInSerialsList.filter(row => {
          if (newlyEnteredSerials.map(r => r.serial_no).indexOf(row.serial_no) > -1) {
            return true;
          }
          return false;
        });

        if(foundBlockedDuplicates !== undefined && foundBlockedDuplicates !== null && foundBlockedDuplicates.length > 0) {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: "SERIAL_NO_ALREADY_EXISTS",
            messageTranslateParam: {
              SNUMBERS: `${this.translateService.instant('KCURRENT_DOCUMENT')} (${foundBlockedDuplicates.map(r => r.serial_no).join(", ")})`
            }
          });

          return;
        }
      }

      const listSerialNumbers = this.listSerials.map(r => r.serial_no);
      this.dialogRef.close({ status: 'success', data: listSerialNumbers });
      return;
    }

    if(this.numeroType == "inventory-numero-series" || this.numeroType == "inventory-tournant-numero-series") {
      let newlyEnteredSerials = [...this.listSerials];

      const newSerialsListBackup = this.newSerialsList.map(r => r.serial_no);
      newlyEnteredSerials = newlyEnteredSerials.filter(r => newSerialsListBackup.indexOf(r.serial_no) == -1);

      if(newlyEnteredSerials !== undefined && newlyEnteredSerials !== null && newlyEnteredSerials.length > 0) {
        const checkDuplicate = await this._checkForSerialDuplicates(newlyEnteredSerials);
        if(checkDuplicate.hasOwnProperty('error') && checkDuplicate.error === true) {
          return;
        }
      }

      await this.moveUnverifiedToSupprimer(true);

      const listSerialNumbers = this.listSerials.map(r => r.serial_no);
      const removedSerialNumbers = this.removeSerialsList.map(r => r.serial_no);

      const data = {
        newSerials: listSerialNumbers,
        removedSerials: removedSerialNumbers
      }

      this.dialogRef.close({ status: 'success', data })
      return;
    }

    if(this.numeroType == "livraison-numero-series" || this.numeroType === "transfer-de-stock-numero-series") {
      this.dialogRef.close({ status: 'success', data: this.selectedOptions })
      return;
    }
  }

}
