import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import {Router} from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {LigneInfo} from "../../../_models/ligne";
import {LigneService} from "../../../_services/ligne.service";
import {NgxSpinnerService} from "ngx-spinner";
import {DialogService} from "../../../_services/dailog.service";
import {CustomHttpResponse} from "../../../_models";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-create-ligne',
  templateUrl: './create-ligne.component.html',
  styleUrls: ['./create-ligne.component.scss']
})
export class CreateLigneComponent implements OnInit {

  submitted: boolean = false;
  createLigneForm: FormGroup = this.formBuilder.group({
    LGISTAT: new FormControl('', [Validators.required]),
    LGILBL: new FormControl('', [Validators.required, Validators.maxLength(APP_CONSTANTS.LIGNE_LABEL_MAX_LENGTH), Validators.minLength(APP_CONSTANTS.LIGNE_LABEL_MIN_LENGTH)]),
  });

  ligne_id: number | undefined;
  ligneDetail: LigneInfo | undefined;

  statusList = APP_CONSTANTS.LIGNE_INFO_STATUS;

  constructor(private ligneService: LigneService,
              private formBuilder: FormBuilder,
              private router: Router,
              private translateService: TranslateService,
              private loadingService: NgxSpinnerService,
              private dialogService: DialogService,
              private dataSharingService:DataSharingService) {
  }

  ngOnInit(): void {
  }

  get fr() {
    return this.createLigneForm.controls;
  }

  createLigneFormSubmitter() {
    this.submitted = true;
    if(this.createLigneForm.invalid) {
      return;
    }

    const formData = this.createLigneForm.getRawValue();
    const payload = {
      LGILBL: formData.LGILBL,
      LGISTAT: formData.LGISTAT
    };

    this._addLigneInfo(payload);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIGNE_LIST_FORM);
  }

  async _addLigneInfo(payload: { LGILBL: string, LGISTAT: number }) {
    const observer = {
      next: async (res: CustomHttpResponse<LigneInfo>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/options/ligne/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.ligneService.createLigne(payload).subscribe(observer);
  }

}
