import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { CustomHttpResponse, LivraisonPrestation } from 'src/app/_models';
import { LivraisonPrestationService } from 'src/app/_services/livraison-prestation.service';

@Component({
  selector: 'app-add-livraison-prestation',
  templateUrl: './add-livraison-prestation.component.html',
  styleUrls: ['./add-livraison-prestation.component.scss']
})
export class AddLivraisonPrestationComponent implements OnInit {


  createLivraisonPrestationForm:FormGroup = this.formBuilder.group({
    prestation_label: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.LIVRAISON_PRESTATION_LABEL_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.LIVRAISON_PRESTATION_LABEL_MAX_LENGTH)]],
    prestation_status: ['', [Validators.required]],
    return_pending_status: [false]
  });
  Submitted: boolean = false;
  loading:boolean = false;
  statusList = APP_CONSTANTS.LIVRAISON_PRESTATION_STATUS;

  constructor(private formBuilder: FormBuilder,
              private router: Router,
              private loadingService: NgxSpinnerService,
              public livraisonPrestationService: LivraisonPrestationService,
              private dialogService: DialogService,
              private dataSharingService:DataSharingService) {}

  get fr() {
    return this.createLivraisonPrestationForm.controls;
  }

  ngOnInit(): void {
  }

  createLivraisonPrestation(){
    this.Submitted = true;
    if(this.createLivraisonPrestationForm.invalid){
      return;
    }
    const formData = this.createLivraisonPrestationForm.getRawValue();
    formData.return_pending_status=formData.return_pending_status? -1 : 0;
    this._addLivraisonPrestation(formData);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST_FORM);
  }

  async _addLivraisonPrestation(payload:LivraisonPrestation){
    const observer = {
      next: async (res: CustomHttpResponse<LivraisonPrestation>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/livraison-prestation/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.livraisonPrestationService.createLivraisonPrestation(payload).subscribe(observer);
  }
}
