import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse, LivraisonPrestation } from 'src/app/_models';
import { AuthService } from 'src/app/_services/auth.service';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import {firstValueFrom} from "rxjs";
import { LivraisonPrestationService } from 'src/app/_services/livraison-prestation.service';
@Component({
  selector: 'app-edit-livraison-prestation',
  templateUrl: './edit-livraison-prestation.component.html',
  styleUrls: ['./edit-livraison-prestation.component.scss']
})
export class EditLivraisonPrestationComponent implements OnInit {

  modifyLivraisonPrestationForm:FormGroup = this.formBuilder.group({
    prestation_label: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.LIVRAISON_PRESTATION_LABEL_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.LIVRAISON_PRESTATION_LABEL_MAX_LENGTH)]],
    prestation_status: ['', [Validators.required]],
    return_pending_status: [false]
  });

  prestation_id:any;
  readOnly: boolean = false;
  pageTitle: string = 'KLIVRAISON_PRESTATION_MODIFY';
  livraisonPrestationInfo: any=[];
  statusList = APP_CONSTANTS.LIVRAISON_PRESTATION_STATUS;
  Submitted: boolean = false;
  label:any;

  constructor(private formBuilder: FormBuilder,
              private dialogService: DialogService,
              private router: Router,
              private route: ActivatedRoute,
              public livraisonPrestationService: LivraisonPrestationService,
              private loadingService: NgxSpinnerService,
              private dataSharingService:DataSharingService) {}

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.prestation_id = +params['prestation_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];

        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }
  async loadDependencies() {
    await this.loadingService.show();
    await this._getLivraisonPrestationDetail(this.prestation_id);
    this._prefillFormData();
    await this.loadingService.hide();
  }
  _getLivraisonPrestationDetail(prestation_id: number | undefined): Promise<LivraisonPrestation> {
    return new Promise<any>((resolve, reject) => {
      if(prestation_id === undefined || prestation_id === null) {
        reject();
        return;
      }
      const observer = {
        next: (res: CustomHttpResponse<LivraisonPrestation>) => {
          if(res.status === "success") {
            this.livraisonPrestationInfo = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/livraison-prestation/list']);
              }
            );
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      };
      this.livraisonPrestationService.getLivraisonPrestationById(prestation_id).subscribe(observer);
    })
  }
  _changeToDetailMode() {
    this.pageTitle = 'KLIVRAISON_PRESTATION_VIEW';

    this.modifyLivraisonPrestationForm.get('prestation_label')?.disable({ onlySelf: true });
    this.modifyLivraisonPrestationForm.get('prestation_status')?.disable({ onlySelf: true });
    this.modifyLivraisonPrestationForm.get('return_pending_status')?.disable({ onlySelf: true });
  }

  _prefillFormData() {
    this.modifyLivraisonPrestationForm.patchValue({
      prestation_label: this.livraisonPrestationInfo?.prestation_label,
      prestation_status: this.livraisonPrestationInfo?.prestation_status,
      return_pending_status: this.livraisonPrestationInfo?.return_pending_status == -1? true: false
    })
  }

  get fr() {
    return this.modifyLivraisonPrestationForm.controls;
  }


  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST);
    if(listCache !== undefined) {
      let listLivraisonPrestation = listCache.data as LivraisonPrestation[];
      listLivraisonPrestation = listLivraisonPrestation.map((row) => {
        if(row.prestation_id == this.prestation_id) {
          row.prestation_id = this.prestation_id;
          row.prestation_label = payload.prestation_label;
          row.prestation_status = payload.prestation_status;
        }
        return row;
      });

      listCache.data = listLivraisonPrestation;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST, JSON.stringify(listCache));
    }
  }

  modifyLivraisonPrestation(){
    this.Submitted = true;
    if(this.modifyLivraisonPrestationForm.invalid){
      return;
    }
    const formData = this.modifyLivraisonPrestationForm.getRawValue();
    formData.return_pending_status=formData.return_pending_status? -1 : 0;
    this._modifyLivraisonPrestation(this.prestation_id, formData);
  }

  async _modifyLivraisonPrestation(prestation_id:number, payload:LivraisonPrestation){
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/livraison-prestation/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.livraisonPrestationService.modifyLivraisonPrestation(prestation_id, payload).subscribe(observer);
  }

}
