import { Component, OnInit } from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse, LivraisonPrestation } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import {lastValueFrom} from "rxjs";
import { FormBuilder, FormGroup } from '@angular/forms';
import { PageEvent } from '@angular/material/paginator';
import { LivraisonPrestationService } from 'src/app/_services/livraison-prestation.service';

@Component({
  selector: 'app-livraison-prestation-list',
  templateUrl: './livraison-prestation-list.component.html',
  styleUrls: ['./livraison-prestation-list.component.scss']
})
export class LivraisonPrestationListComponent implements OnInit {


  prestationList:LivraisonPrestation[]=[];
  isLoading = false;
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  displayedColumnsPrestation:string[] = ['PRESTATION_LABEL', 'PRESTATION_STATUS','PRESTATION_PENDING_STATUS','PRESTATION_ACTION'];
  searchForm: FormGroup = this.formBuilder.group({
    keyword: [''],
  });
  rolePermission:any;

  constructor(public dialog: MatDialog,
              public dataSharingService:DataSharingService,
              private dialogService: DialogService,
              private livraisonPrestationService:LivraisonPrestationService,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService) {}

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST);
      if(dataCache !== undefined) {

        this.prestationList = dataCache.data as LivraisonPrestation[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.loadingService.hide();
  }

  documentStatus(item: LivraisonPrestation, field: string) {
    let result = "";
    if(field === 'LIVRAISON_PRESTATION'){
      const statusList = APP_CONSTANTS.LIVRAISON_PRESTATION_STATUS;
      const statusData = statusList.find((r) => r.value == item.prestation_status);
      if(statusData == undefined) {
        return result;
      }
  
      switch (item.prestation_status) {
        case APP_CONSTANTS.LIVRAISON_PRESTATION_ACTIVE:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.LIVRAISON_PRESTATION_IN_ACTIVE:
          result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    if(field === 'PRESTATION_PENDING_STATUS'){
      const statusList = APP_CONSTANTS.LIVRAISON_PRESTATION_PENDING_STATUS;
      const statusData = statusList.find((r) => r.value == item.return_pending_status);

      if(statusData == undefined) {
        return result;
      }

      switch (item.return_pending_status) {
        case APP_CONSTANTS.LIVRAISON_PRESTATION_PENDING_YES:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.LIVRAISON_PRESTATION_PENDING_NO:
          result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    
    return result;
  }

  searchFormSubmitter(){
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword,
    };
    this.currentPage = 1;
    this._searchLivraisonPrestationInfo(payload, this.currentPage, this.pageSize);
  }

  _searchLivraisonPrestationInfo(data: { keyword: string}, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<LivraisonPrestation[]>) => {
        if(res.status === "success") {
          this.prestationList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.livraisonPrestationService.searchLivraisonPrestation(data, page, pageSize).subscribe( observer);
  }

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword,
      date_created: ""
    };
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchLivraisonPrestationInfo(payload, this.currentPage, this.pageSize);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST);
    if(listCache !== undefined) {
      let listLivraisonPrestation: any[] = listCache.data;
      listLivraisonPrestation = (listLivraisonPrestation as LivraisonPrestation[]).filter(r => r.prestation_id != id);

      listCache.data = listLivraisonPrestation;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_LIVRAISON_PRESTATION_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listLivraisonPrestation.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  confirmRemoveLivraisonPrestation(prestation_label: string): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: prestation_label })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeLivraisonPrestation(prestation_id: number, prestation_label:string) {
    if(!this.rolePermission['9_114']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveLivraisonPrestation(prestation_label);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteLivraisonPrestationById(prestation_id);
    }
  }

  async _deleteLivraisonPrestationById(prestation_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(prestation_id)
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.livraisonPrestationService.deleteLivraisonPrestationById(prestation_id).subscribe(observer);
  }


}
