import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { ServiceExpeditionService } from 'src/app/_services/service-expedition.service';

@Component({
  selector: 'app-add-service-expedition',
  templateUrl: './add-service-expedition.component.html',
  styleUrls: ['./add-service-expedition.component.scss']
})
export class AddServiceExpeditionComponent implements OnInit {

  createServiceExpeditionForm: FormGroup = this.formBuilder.group({
    label: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.SERVICE_EXPEDITION_LEBAL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.SERVICE_EXPEDITION_LEBAL_MAX_LENGTH)]],
    code: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.SERVICE_EXPEDITION_CODE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.SERVICE_EXPEDITION_CODE_MAX_LENGTH)]],
    status: ['', Validators.required]
  });
  submitted: boolean = false;
  statusList = APP_CONSTANTS.SERVICE_EXPEDITION_INFO_STATUS;

  constructor(private formBuilder: FormBuilder,
              public serviceExpeditionService: ServiceExpeditionService,
              private translateService: TranslateService,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
  }

  get fr() {
    return this.createServiceExpeditionForm.controls;
  }

  createServiceExpeditionFormSubmitter() {
    this.submitted = true;
    if (this.createServiceExpeditionForm.invalid) {
      return;
    }
    const payload = this.createServiceExpeditionForm.getRawValue();
    this._createServiceExpedition(payload);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_SERVICE_EXPEDITION_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_SERVICE_EXPEDITION_LIST_FORM);
  }

  async _createServiceExpedition(payload: any) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.createServiceExpeditionForm.reset();
          this.submitted = false;
          this.router.navigate(['/options/service-expedition/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.serviceExpeditionService.createServiceExpedition(payload).subscribe(observer);
  }

}
