import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { firstValueFrom } from 'rxjs';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse, ServiceExpedition } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { ServiceExpeditionService } from 'src/app/_services/service-expedition.service';

@Component({
  selector: 'app-edit-service-expedition',
  templateUrl: './edit-service-expedition.component.html',
  styleUrls: ['./edit-service-expedition.component.scss']
})
export class EditServiceExpeditionComponent implements OnInit {


  modifyServiceExpeditionForm: FormGroup = this.formBuilder.group({
    label: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.SERVICE_EXPEDITION_LEBAL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.SERVICE_EXPEDITION_LEBAL_MAX_LENGTH)]],
    code: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.SERVICE_EXPEDITION_CODE_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.SERVICE_EXPEDITION_CODE_MAX_LENGTH)]],
    status: ['', Validators.required]
  });
  service_expedition_id!: number;
  submitted: boolean = false;
  readOnly: boolean =false;
  pageTitle: string ='KSERVICE_EXP_MODIFY';
  statusList = APP_CONSTANTS.SERVICE_EXPEDITION_INFO_STATUS;
  editServiceExpeditionInfo:any=[];

  constructor(private formBuilder: FormBuilder,
              public serviceExpeditionService: ServiceExpeditionService,
              private translateService: TranslateService,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private route: ActivatedRoute,
              private dataSharingService: DataSharingService) {
  }

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.service_expedition_id = +params['service_expedition_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        console.log(this.readOnly)
        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this._getUserRoleInfoDetail(this.service_expedition_id);
    this.prefillFormDetail(this.editServiceExpeditionInfo);
    await this.loadingService.hide();
  }

  _getUserRoleInfoDetail(service_expedition_id:number): Promise<any>{
    return new Promise((resolve, reject) => {
      this.serviceExpeditionService.viewServiceExeditionDetail(service_expedition_id).subscribe({
        next: (res: CustomHttpResponse<ServiceExpedition>) => {
          if(res.status === 'success') {
            this.editServiceExpeditionInfo = res.data;
          }else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }
          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  prefillFormDetail(editServiceExpeditionInfo: ServiceExpedition | undefined) {
    if(editServiceExpeditionInfo === undefined || editServiceExpeditionInfo === null) {
      return;
    }
    this.modifyServiceExpeditionForm.patchValue({
      label: editServiceExpeditionInfo.label,
      code: editServiceExpeditionInfo.code,
      status: editServiceExpeditionInfo.status
    });
  }

  _changeToDetailMode() {
    this.pageTitle = 'KSERVICE_EXP_DETAIL';
    this.modifyServiceExpeditionForm.get('label')?.disable({ onlySelf: true });
    this.modifyServiceExpeditionForm.get('code')?.disable({ onlySelf: true });
    this.modifyServiceExpeditionForm.get('status')?.disable({ onlySelf: true });
  }

  get fr() {
    return this.modifyServiceExpeditionForm.controls;
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_SERVICE_EXPEDITION_LIST);
    if(listCache !== undefined) {
      let listLigneInfo = listCache.data as ServiceExpedition[];
      listLigneInfo = listLigneInfo.map((row) => {
        if(row.id == this.service_expedition_id) {
          row.id = this.service_expedition_id;
          row.label = payload.label;
          row.code = payload.code;
          row.status = payload.status;
        }
        return row;
      });

      listCache.data = listLigneInfo;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_SERVICE_EXPEDITION_LIST, JSON.stringify(listCache));
    }
  }

  modifyServiceExpeditionFormSubmitter() {
    this.submitted = true;
    if (this.modifyServiceExpeditionForm.invalid) {
      return;
    }
    const payload = this.modifyServiceExpeditionForm.getRawValue();
    this._modifyServiceExpedition(payload);
  }

  async _modifyServiceExpedition(payload: ServiceExpedition) {
    const observer = {
      next: async (res: CustomHttpResponse<ServiceExpedition>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.modifyServiceExpeditionForm.reset();
          this.submitted = false;
          this.router.navigate(['/options/service-expedition/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }
    this.serviceExpeditionService.modifyServiceExpedition(payload,this.service_expedition_id).subscribe(observer);
  }

}
