import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { TranslateService } from '@ngx-translate/core';
import { Router} from '@angular/router';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import {BrandInfoList, CustomHttpResponse} from "../../../_models";
import {StockLocationService} from "../../../_services/stock-location.service";
import {CommonService} from "../../../_services/common.service";
import {DialogService} from "../../../_services/dailog.service";
import {NgxSpinnerService} from "ngx-spinner";
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-add-storage-location',
  templateUrl: './add-storage-location.component.html',
  styleUrls: ['./add-storage-location.component.scss']
})
export class AddStorageLocationComponent implements OnInit {

  createStorageForm: FormGroup = this.formBuilder.group({
    STLLBL: ['', [Validators.required, Validators.minLength(APP_CONSTANTS.STOCK_LOCATION_LABEL_MIN_LENGTH), Validators.maxLength(APP_CONSTANTS.STOCK_LOCATION_LABEL_MAX_LENGTH)]],
    STLBRID: [''],
    STLSTAT: ['', Validators.required]
  });

  submitted: boolean = false;
  brandList: BrandInfoList[] = [];
  statusList = APP_CONSTANTS.STOCK_LOCATION_STATUS;

  constructor(private formBuilder: FormBuilder,
              public stockLocationService: StockLocationService,
              private commonService: CommonService,
              private translateService: TranslateService,
              private router: Router,
              private dialogService: DialogService,
              private loadingService: NgxSpinnerService,
              private dataSharingService:DataSharingService) {
  }

  ngOnInit(): void {
    this.loadDependencies();
  }

  async loadDependencies() {
    await this.loadingService.show();

    await this._getBrandList();

    await this.loadingService.hide();
  }

  get fr() {
    return this.createStorageForm.controls;
  }

  _getBrandList(): Promise<any> {
    return new Promise<any>((resolve, reject) => {
      const observer = {
        next: (res: CustomHttpResponse<BrandInfoList[]>) => {
          if(res.status === 'success') {
            this.brandList = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          console.log(err);
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: 'KNETWORK_CONNECTION_ERROR'
          });
          reject(err);
        }
      }

      this.commonService.getListBrands().subscribe(observer);
    });
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATION_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_STOCK_LOCATIONT_LIST_FORM);
  }

  createFormSubmitter() {
    this.submitted = true;
    if (this.createStorageForm.invalid) {
      return;
    }

    const formData = this.createStorageForm.getRawValue();
    const payload = {
      brand_id: formData.STLBRID,
      stock_location_lbl: formData.STLLBL,
      stock_location_status: formData.STLSTAT
    };

    this._createStockLocation(payload);
  }

  async _createStockLocation(payload: any) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.createStorageForm.reset();
          this.submitted = false;
          this.router.navigate(['/options/storage-location/list']);
        } else {
          this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
        this.dialogService.displayAlert({
          title: 'KERROR',
          message: 'KNETWORK_CONNECTION_ERROR'
        });
      }
    }

    this.stockLocationService.createStockLocation(payload).subscribe(observer);
  }

}
