import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators } from '@angular/forms';
import { TypeDeMaterialService } from '../../../_services/type-de-material.service';
import { TranslateService } from '@ngx-translate/core';
import { Router } from '@angular/router';
import { CustomHttpResponse, TypeDeMaterial } from 'src/app/_models';
import { NgxSpinnerService } from 'ngx-spinner';
import { DialogService } from 'src/app/_services/dailog.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-add-type-de-material',
  templateUrl: './add-type-de-material.component.html',
  styleUrls: ['./add-type-de-material.component.scss']
})
export class AddTypeDeMaterialComponent implements OnInit {

  createTypeForm:FormGroup = this.formBuilder.group({
    materialtype_label: new FormControl('', [Validators.required, Validators.minLength(APP_CONSTANTS.TYPE_DE_MATERIAL_LABEL_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.TYPE_DE_MATERIAL_LABEL_MAX_LENGTH)]),
    materialtype_status: new FormControl('', [Validators.required]),
    materialtype_activable_status: new FormControl(false)
  });
  Submitted: boolean = false;
  loading:boolean = false;
  statusList = APP_CONSTANTS.TYPE_DE_MATERIAL_STATUS;

  constructor(private formBuilder: FormBuilder,
              private translateService:TranslateService,
              private router: Router,
              private loadingService: NgxSpinnerService,
              public typeDeMaterialService:TypeDeMaterialService,
              private dialogService: DialogService,
              private dataSharingService:DataSharingService) {}

  get fr() {
    return this.createTypeForm.controls;
  }

  ngOnInit(): void {
  }

  createType(){
    this.Submitted = true;
    if(this.createTypeForm.invalid){
      return;
    }
    const formData = this.createTypeForm.getRawValue();
    formData.materialtype_activable_status=formData.materialtype_activable_status? -1 : 0;
    this._addTypeDeMaterial(formData);
  }

  __resetListCache() {
    this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST);
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST_FORM);
  }

  async _addTypeDeMaterial(payload:TypeDeMaterial){
    const observer = {
      next: async (res: CustomHttpResponse<TypeDeMaterial>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__resetListCache();
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/options/type-de-material/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };

    this.typeDeMaterialService.createTypeDeMaterial(payload).subscribe(observer);
  }
}
