import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, FormControl, Validators, FormArray } from '@angular/forms';
import { Location} from '@angular/common';
import { AuthService } from '../../../_services/auth.service';
import { TranslateService } from '@ngx-translate/core';
import {firstValueFrom} from "rxjs";
import { Router, ActivatedRoute } from '@angular/router';
import { TypeDeMaterialService } from '../../../_services/type-de-material.service';
import { NgxSpinnerService } from 'ngx-spinner';
import { CustomHttpResponse, TypeDeMaterial } from 'src/app/_models';
import { DialogService } from 'src/app/_services/dailog.service';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { DataSharingService } from 'src/app/_services/data-sharing.service';

@Component({
  selector: 'app-edit-type-de-material',
  templateUrl: './edit-type-de-material.component.html',
  styleUrls: ['./edit-type-de-material.component.scss']
})
export class EditTypeDeMaterialComponent implements OnInit {

  modifyTypeForm:FormGroup = this.formBuilder.group({
    materialtype_label: new FormControl('', [Validators.required, Validators.minLength(APP_CONSTANTS.TYPE_DE_MATERIAL_LABEL_MIN_LENGTH),Validators.maxLength(APP_CONSTANTS.TYPE_DE_MATERIAL_LABEL_MAX_LENGTH)]),
    materialtype_status: new FormControl('', [Validators.required]),
    materialtype_activable_status: new FormControl('')
  });
  material_id:any;
  readOnly: boolean = false;
  pageTitle: string = 'KMODIFY_TYPE_MATERIAL';
  typeDeMaterialInfo: any=[];
  statusList = APP_CONSTANTS.TYPE_DE_MATERIAL_STATUS;
  Submitted: boolean = false;
  label:any;

  constructor(private location: Location,
              private formBuilder: FormBuilder,
              private authService:AuthService,
              private dialogService: DialogService,
              private translateService:TranslateService,
              private router: Router,
              private route: ActivatedRoute,
              public typeDeMaterialService:TypeDeMaterialService,
              private loadingService: NgxSpinnerService,
              private dataSharingService:DataSharingService) {}

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.material_id = +params['material_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];

        if(this.readOnly) {
          this._changeToDetailMode();
        }
      }
    })
  }
  async loadDependencies() {
    await this.loadingService.show();
    await this._getTypeDeMaterialDetail(this.material_id);
    this._prefillFormData();
    await this.loadingService.hide();
  }
  _getTypeDeMaterialDetail(material_id: number | undefined): Promise<TypeDeMaterial> {
    return new Promise<any>((resolve, reject) => {
      if(material_id === undefined || material_id === null) {
        reject();
        return;
      }
      const observer = {
        next: (res: CustomHttpResponse<TypeDeMaterial>) => {
          if(res.status === "success") {
            this.typeDeMaterialInfo = res.data;
          } else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            }).then(
              () => {
                this.router.navigate(['/options/type-de-material/list']);
              }
            );
          }

          resolve(res);
        },
        error: (err: any) => {
          this.loadingService.hide();
          reject(err);
        }
      };
      this.typeDeMaterialService.getTypeDeMaterialById(material_id).subscribe(observer);
    })
  }
  _changeToDetailMode() {
    this.pageTitle = 'KDETAIL_TYPE_MATERIAL';

    this.modifyTypeForm.get('materialtype_label')?.disable({ onlySelf: true });
    this.modifyTypeForm.get('materialtype_status')?.disable({ onlySelf: true });
    this.modifyTypeForm.get('materialtype_activable_status')?.disable({ onlySelf: true });
  }

  _prefillFormData() {
    this.modifyTypeForm.patchValue({
      materialtype_label: this.typeDeMaterialInfo?.materialtype_label,
      materialtype_status: this.typeDeMaterialInfo?.materialtype_status,
      materialtype_activable_status: this.typeDeMaterialInfo?.materialtype_activable_status == -1? true: false
    })
  }

  get fr() {
    return this.modifyTypeForm.controls;
  }


  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST);
    if(listCache !== undefined) {
      let listTypeDeMaterial = listCache.data as TypeDeMaterial[];
      listTypeDeMaterial = listTypeDeMaterial.map((row) => {
        if(row.materialtype_id == this.material_id) {
          row.materialtype_id = this.material_id;
          row.materialtype_label = payload.materialtype_label;
          row.materialtype_status = payload.materialtype_status;
        }
        return row;
      });

      listCache.data = listTypeDeMaterial;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST, JSON.stringify(listCache));
    }
  }

  modifyMaterialInfo(){
    this.Submitted = true;
    if(this.modifyTypeForm.invalid){
      return;
    }
    const formData = this.modifyTypeForm.getRawValue();
    formData.materialtype_activable_status=formData.materialtype_activable_status? -1 : 0;
    this._modifyTypeDeMaterial(this.material_id, formData);
  }

  async _modifyTypeDeMaterial(material_id:number, payload:TypeDeMaterial){
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        this.loadingService.hide();
        if(res.status === 'success') {
          this.__updateListCache(payload);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          });
          this.router.navigate(['/options/type-de-material/list']);
        } else {
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          });
        }
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.typeDeMaterialService.modifyTypeDeMaterial(material_id, payload).subscribe(observer);
  }

}
