import { Component, OnInit } from '@angular/core';
import { PageEvent } from '@angular/material/paginator';
import { MatDialog } from '@angular/material/dialog';
import { DataSharingService} from '../../../_services/data-sharing.service';
import { TypeDeMaterialService} from '../../../_services/type-de-material.service';
import { FormBuilder, FormControl, FormGroup } from '@angular/forms';
import {NgxSpinnerService} from "ngx-spinner";
import { TranslateService } from '@ngx-translate/core';
import {APP_CONSTANTS} from "../../../_helpers/constants";
import { DialogService } from 'src/app/_services/dailog.service';
import { lastValueFrom } from 'rxjs';
import { CustomHttpResponse, TypeDeMaterial } from 'src/app/_models';

@Component({
  selector: 'app-type-de-material-list',
  templateUrl: './type-de-material-list.component.html',
  styleUrls: ['./type-de-material-list.component.scss']
})
export class TypeDeMaterialListComponent implements OnInit {

  typeDeMaterialList:TypeDeMaterial[]=[];
  isLoading = false;
  totalRows = 0;
  pageSize = 10;
  currentPage = 1;
  pageSizeOptions: number[] = [10, 25, 100];
  displayedColumnsTypeDeMaterial:string[] = ['MATERIAL_LABEL', 'MATERIAL_STATUS','MATERIAL_ACTIVABLE_STATUS','MATERIAL_ACTION'];
  searchForm:FormGroup = this.formBuilder.group({
    keyword: new FormControl('', ),
  });
  rolePermission:any;

  constructor(public dialog: MatDialog,
              public dataSharingService:DataSharingService,
              private dialogService: DialogService,
              private typeDeMaterialService:TypeDeMaterialService,
              private formBuilder: FormBuilder,
              private loadingService: NgxSpinnerService,
              private translateService:TranslateService) {}

  ngOnInit(): void {
    const user_role_permission = localStorage.getItem(APP_CONSTANTS.USER_ROLE_PERMISSION_KEY);
    if(user_role_permission != null) {
      this.rolePermission = JSON.parse(user_role_permission);
    }
    this.checkForListCache();
    this.loadDependencies();
  }

  checkForListCache() {
    // Check if Old data cache is available
    const formCache = this.dataSharingService.getListFormDataFromCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST_FORM);
    console.log(formCache)
    if(formCache !== undefined) {
      this.searchForm.patchValue({
        keyword: formCache.keyword
      });

      const dataCache = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST);
      if(dataCache !== undefined) {

        this.typeDeMaterialList = dataCache.data as TypeDeMaterial[];

        this.currentPage = dataCache.currentPage ? dataCache.currentPage - 1 : this.currentPage;
        this.totalRows = dataCache.totalRows ? dataCache.totalRows : this.totalRows;
      }
    }else{
      this.searchFormSubmitter();
    }
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this.loadingService.hide();
  }

  documentStatus(item: TypeDeMaterial, field: string) {
    let result = "";
    if(field === 'MATERIAL_STATUS'){
      const statusList = APP_CONSTANTS.TYPE_DE_MATERIAL_STATUS;
      const statusData = statusList.find((r) => r.value == item.materialtype_status);
      if(statusData == undefined) {
        return result;
      }
  
      switch (item.materialtype_status) {
        case APP_CONSTANTS.TYPE_DE_MATERIAL_ACTIVE:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.TYPE_DE_MATERIAL_IN_ACTIVE:
          result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    if(field === 'MATERIAL_ACTIVABLE_STATUS'){
      const statusList = APP_CONSTANTS.TYPE_DE_MATERIAL_ACTIVE_STATUS;
      const statusData = statusList.find((r) => r.value == item.materialtype_activable_status);

      if(statusData == undefined) {
        return result;
      }

      switch (item.materialtype_activable_status) {
        case APP_CONSTANTS.TYPE_DE_MATERIAL_ACTIVE_YES:
          result = `<span class='badge bg-success'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
        case APP_CONSTANTS.TYPE_DE_MATERIAL_ACTIVE_NO:
          result = `<span class='badge bg-danger'>${this.translateService.instant(statusData?.label)}</span>`;
          break;
      }
    }
    
    return result;
  }

  searchFormSubmitter(){
    const formData = this.searchForm.getRawValue();
    this.dataSharingService.setListFormDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST_FORM, JSON.stringify(formData));
    const payload = {
      keyword: formData.keyword,
    };
    this.currentPage = 1;
    this._searchTypeDeMaterialInfo(payload, this.currentPage, this.pageSize);
  }

  _searchTypeDeMaterialInfo(data: { keyword: string}, page: number, pageSize: number) {
    const observer = {
      next: (res: CustomHttpResponse<TypeDeMaterial[]>) => {
        if(res.status === "success") {
          this.typeDeMaterialList = res.data;
          this.currentPage = res.currentPage ? res.currentPage - 1 : this.currentPage;
          this.totalRows = res.totalRows ? res.totalRows : this.totalRows;
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST, JSON.stringify(res));
        }else{
          this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST);
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.typeDeMaterialService.searchTypeDeMaterial(data, page, pageSize).subscribe( observer);
  }

  pageChanged(event: PageEvent) {
    const formData = this.searchForm.getRawValue();
    const payload = {
      keyword: formData.keyword,
      date_created: ""
    };
    this.pageSize = event.pageSize;
    this.currentPage = event.pageIndex + 1;
    this._searchTypeDeMaterialInfo(payload, this.currentPage, this.pageSize);
  }

  __updateListCache(id: number) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST);
    if(listCache !== undefined) {
      let listTypeDeMaterial: any[] = listCache.data;
      listTypeDeMaterial = (listTypeDeMaterial as TypeDeMaterial[]).filter(r => r.materialtype_id != id);

      listCache.data = listTypeDeMaterial;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_TYPE_DE_MATERIAL_LIST, JSON.stringify(listCache));

      this.checkForListCache();

      if(listTypeDeMaterial.length === 0) {
        this.searchFormSubmitter();
      }
    }
}

  confirmRemoveTypeDeMaterial(material_id: number): Promise<any> {
    const message = `<h3>${this.translateService.instant('KYOURE_GOING_REMOVE_MODULE_CONFIRM', { ITEM_VALUE: material_id })}</h3>`;
    return lastValueFrom(this.dialogService.confirmDialog({
      title: 'KARE_YOU_SURE_WANT_CONTINUE',
      message: message
    }));
  }

  async removeTypeDeMaterial(material_id: number) {
    if(!this.rolePermission['9_44']) {
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KACCESS_DENIED'
      })
      return;
    }
    const removeConfirmation = await this.confirmRemoveTypeDeMaterial(material_id);
    if(removeConfirmation) {
      this.loadingService.show();
      this._deleteTypeDeMaterialById(material_id);
    }
  }

  async _deleteTypeDeMaterialById(material_id: number) {
    const observer = {
      next: async (res: CustomHttpResponse<any>) => {
        await this.loadingService.hide();
        if(res.status === "success") {
          this.__updateListCache(material_id)
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          return;
        }
        await this.dialogService.displayAlert({
          title: 'KERROR',
          message: res.message,
          messageTranslateParam: res.data
        });
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.typeDeMaterialService.deleteTypeDeMaterialById(material_id).subscribe(observer);
  }

}
