import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';
import { NgxSpinnerService } from 'ngx-spinner';
import { TreeviewConfig, TreeviewItem } from 'ngx-treeview';
import { firstValueFrom } from 'rxjs';
import { APP_CONSTANTS } from 'src/app/_helpers/constants';
import { CustomHttpResponse } from 'src/app/_models';
import { UserPermission, UserRole } from 'src/app/_models/user-role';
import { DialogService } from 'src/app/_services/dailog.service';
import { DataSharingService } from 'src/app/_services/data-sharing.service';
import { UserRolesService } from 'src/app/_services/user-roles.service';

@Component({
  selector: 'app-edit-role',
  templateUrl: './edit-role.component.html',
  styleUrls: ['./edit-role.component.scss']
})
export class EditRoleComponent implements OnInit {


  modifyRoleForm:FormGroup = this.formBuilder.group({
    role_name: ['', [Validators.required]],
    role_status: ['', [Validators.required]]
  });
  submitted: boolean = false;
  loading:boolean = false;
  userPermissionlist: UserPermission[]=[];
  editRoleFormInfo:any=[];
  allComplete: boolean[] = [];
  collectPermissionData: any[]=[];
  role_id!: number;
  readOnly: boolean = false;
  pageTitle: string = 'KMODIFY_ROLE';
  disabled:boolean = false;
  statusList = APP_CONSTANTS.USER_ROLE_INFO_STATUS;
  roleNameMask = APP_CONSTANTS.ROLE_NAME_MASK;

  dropdownEnabled = true;
  items: TreeviewItem[] =[];
  values: number[] | undefined;
  config = TreeviewConfig.create({
      hasAllCheckBox: false,
      hasFilter: false,
      hasCollapseExpand: false,
      decoupleChildFromParent: false
  });
  checkedValueList:any =[];

  constructor(private formBuilder: FormBuilder,
    private router: Router,
    private loadingService: NgxSpinnerService,
    private dialogService: DialogService,
    private userRolesService: UserRolesService,
    private route: ActivatedRoute,
    private translateService: TranslateService,
    private dataSharingService: DataSharingService
    ) {}

  ngOnInit(): void {
    firstValueFrom(this.route.params).then((params) => {
      this.role_id = +params['role_id'];
      this.loadDependencies();
    })

    firstValueFrom(this.route.data).then((data) => {
      if(data !== undefined && data !== null && data.hasOwnProperty('readOnly')) {
        this.readOnly = data['readOnly'];
        console.log(this.readOnly)
        if(this.readOnly) {
          this._changeToDetailMode();
          this.disabled=true;
        }
      }
    })
  }

  async loadDependencies() {
    await this.loadingService.show();
    await this._getUserRoleInfoDetail(this.role_id);
    this._prefillFormData();
    this._getCheckboxData();
    await this.loadingService.hide();
  }

  _changeToDetailMode() {
    this.pageTitle = 'KDETAIL_ROLE';
    this.modifyRoleForm.get('role_name')?.disable({ onlySelf: true });
    this.modifyRoleForm.get('role_status')?.disable({ onlySelf: true });
  }

  __updateListCache(payload: any) {
    const listCache: any = this.dataSharingService.getListDataFromCache(APP_CONSTANTS.CACHE_ROLE_LIST);
    if(listCache !== undefined) {
      let listUserRole = listCache.data as UserRole[];
      listUserRole = listUserRole.map((row) => {
        if(row.roleid == this.role_id) {
          row.roleid = this.role_id;
          row.role_name = payload.role_name;
          row.role_status = payload.role_status;
          row.role_permission = payload.role_permission;
        }
        return row;
      });

      listCache.data = listUserRole;

      this.dataSharingService.setListDataCache(APP_CONSTANTS.CACHE_ROLE_LIST, JSON.stringify(listCache));
    }
  }


  _getUserRoleInfoDetail(role_id:number): Promise<any>{
    return new Promise((resolve, reject) => {
      this.userRolesService.viewRoleDetail(role_id).subscribe({
        next: (res: CustomHttpResponse<UserRole>) => {
          if(res.status === 'success') {
            this.editRoleFormInfo = res.data;
          }else {
            this.dialogService.displayAlert({
              title: 'KERROR',
              message: res.message,
              messageTranslateParam: res.data
            });
          }
          resolve(res);
        },
        error: (err) => {
          this.loadingService.hide();
          reject(err);
        }
      });
    });
  }

  _prefillFormData() {
    this.modifyRoleForm.patchValue({
      role_name: this.editRoleFormInfo?.role_name,
      role_status: this.editRoleFormInfo?.role_status,
    })
    this.userPermissionlist=this.editRoleFormInfo?.role_permission;
  }

  get fr() {
    return this.modifyRoleForm.controls;
  }

  _getCheckboxData(){
    const _formatTreeData = (rowData: { label: any; id: any; value: any; status: any; child: any[] | null | undefined; }) => {
      const formatted:any = {
        text: this.translateService.instant(rowData.label),
        value: `${rowData.id}_${rowData.value}`,
        checked: rowData.status,
        collapsed: true,
        disabled: this.readOnly? true : false
      };

      let childrens;
      if(rowData.child !== undefined && rowData.child !== null && rowData.child.length > 0) {
        childrens = rowData.child.map(child_row => {
          return _formatTreeData(child_row);
        });
      }

      if(childrens !== undefined && childrens !== null) {
        formatted['children'] = childrens;
      }

      return formatted;
    };

    const processTreeData = (data: any[] | null | undefined) => {
      if(data === undefined || data === null || data.length === 0) return {};

      const processedData = data.map((row) => {
        return _formatTreeData(row);
      });

      return processedData;
    }
    const processedTreeData:any = processTreeData(this.userPermissionlist);
      this.items = [new TreeviewItem({
       text: this.translateService.instant('KUSER_ROLE_PERMISSIONS'),
       value:'',
       children:processedTreeData
      })];
  }

  onSelectedChange(event: any){
    if(event === undefined || event === null || event.length === 0) {
      this.checkedValueList = [];
      return;
    }
    this.checkedValueList = [];
    let children = this.items[0].children;
    while(children !== undefined && children.length > 0) {
      const currentChilds = [...children];
      children = [];
      currentChilds.forEach((row) => {
        if(row.checked || row.indeterminate){
          this.checkedValueList.push(row.value);
        }
        if(row.children !== undefined && row.children !== null && row.children.length > 0) {
          children = [...children, ...row.children];
        }
      })
    }
  }

  checkRoleLabel(){
    const role_name = this.modifyRoleForm.get('role_name')?.value.trim();
    this.modifyRoleForm.get('role_name')?.setValue(role_name);
  }

  modifyRoleFormSubmit(){
    this.submitted = true;
    const role_name = this.modifyRoleForm.get('role_name')?.value.trim();
    this.modifyRoleForm.get('role_name')?.setValue(role_name);
    if(this.modifyRoleForm.invalid){
      return;
    }
    if(this.checkedValueList !== undefined && this.checkedValueList !== null && this.checkedValueList.length !== 0) {
      this.checkedValueList.map((key: any) =>{
        let arr = key.split('_');
        const [id, value] = arr;
        const obj:any = {
          id: +id,
          value: +value,
          status: true
        };
        this.collectPermissionData.push(obj)
      });
      const formData=this.modifyRoleForm.getRawValue();
      const payload = {
        ...formData,
        role_permission: this.collectPermissionData
      };
      this.createUserRolePermission(payload);
    }else{
      this.dialogService.displayAlert({
        title: 'KERROR',
        message: 'KSELECT_ROLE_PERMISSIONS'
      })
    }
  }

  async createUserRolePermission(data:UserPermission){
    const observer = {
      next: async (res: any) => {
        if(res.status === "success") {
          this.__updateListCache(data);
          await this.dialogService.displayAlert({
            title: 'KSUCCESS',
            message: res.message
          })
          this.router.navigate(['/options/roles/list']);
        }else{
          await this.dialogService.displayAlert({
            title: 'KERROR',
            message: res.message,
            messageTranslateParam: res.data
          })
        }
        this.loadingService.hide();
      },
      error: (err: any) => {
        this.loadingService.hide();
      }
    };
    this.userRolesService.modifyUserRoleAPI(data,this.editRoleFormInfo.role_id).subscribe( observer);
  }
}
